@@ -50,10 +50,17 @@ ConfigOption::ConfigOption(const string& sName, const string& sValue,
 
 ConfigMgr* ConfigMgr::m_pGlobalConfigMgr = 0;
 
+void deleteConfigMgr()
+{
+    delete ConfigMgr::m_pGlobalConfigMgr;
+    ConfigMgr::m_pGlobalConfigMgr = 0;
+}
+
 ConfigMgr* ConfigMgr::get()
 {
     if (!m_pGlobalConfigMgr) {
         m_pGlobalConfigMgr = new ConfigMgr;
+        atexit(deleteConfigMgr);
     }
     return m_pGlobalConfigMgr;
 }
@@ -71,9 +78,8 @@ ConfigMgr::ConfigMgr()
             "The height of the window to use. Contents are scaled.");
     addOption("scr", "usepow2textures", "false",
             "If set to true, use only power of 2 textures.");
-    addOption("scr", "ycbcrmode", "shader",
-            "How to render YCbCr surfaces. Valid values are"
-            " shader, mesa, apple and none.");
+    addOption("scr", "useshaders", "true",
+            "Whether to use shaders.");
     addOption("scr", "usepixelbuffers", "true",
             "Whether to use pixel buffer objects.");
     addOption("scr", "multisamplesamples", "1",
@@ -242,7 +248,8 @@ bool ConfigMgr::loadFile(const std::string& sPath) {
         xmlDocPtr doc;
         doc = xmlParseFile(sPath.c_str());
         if (!doc) {
-            return false;
+            throw Exception(AVG_ERR_XML_VALID, "Error parsing "+sPath
+                    +". File is not well-formed.");
         }
         xmlNodePtr pRoot = xmlDocGetRootElement(doc);
         if (xmlStrcmp(pRoot->name, (const xmlChar *)(m_sFName.c_str()))) {
@@ -315,8 +322,10 @@ void ConfigMgr::setOption(ConfigOptionVector& OptionVector,
         xmlDocPtr doc, xmlNodePtr pNode)
 {
     string sName = (const char *)pNode->name;
-    string sValue = 
-            (const char *)xmlNodeListGetString(doc, pNode->xmlChildrenNode, 1);
+    xmlChar * pVal = 
+            xmlNodeListGetString(doc, pNode->xmlChildrenNode, 1);
+    string sValue = (const char *)pVal;
+    xmlFree(pVal);
     setOption(OptionVector, sName, sValue);    
 }
 
