@@ -22,6 +22,7 @@
 #ifndef _Directory_H_
 #define _Directory_H_
 
+#include "../api.h"
 #include "DirEntry.h"
 
 #ifdef _WIN32
@@ -34,13 +35,14 @@
 
 namespace avg {
     
-class Directory {
+class AVG_API Directory {
 public:
     Directory(std::string sName);
     virtual ~Directory();
 
     int open(bool bCreateIfMissing = false);
     DirEntryPtr getNextEntry();
+    const std::string& getName();
     void empty();
 
 private:
