@@ -22,6 +22,7 @@
 #ifndef _FileHelper_H_
 #define _FileHelper_H_
 
+#include "../api.h"
 #include <string>
 
 namespace avg {
@@ -36,6 +37,12 @@ void readWholeFile(const std::string& sFilename, std::string& sContents);
 
 void writeWholeFile(const std::string& sFilename, const std::string& sContent);
 
+void copyFile(const std::string& sSourceFile, const std::string& sDestFile);
+
+#ifdef WIN32
+#define unlink _unlink
+#endif
+
 }
 
 #endif 
