@@ -22,7 +22,10 @@
 #ifndef _Logger_H_ 
 #define _Logger_H_
 
-#include <iostream>
+#include "../api.h"
+
+#include <string>
+#include <vector>
 #include <sstream>
 
 namespace avg {
@@ -31,15 +34,14 @@ namespace avg {
 #undef ERROR
 #endif
 
-class Logger {
+class AVG_API Logger {
 public:
     static Logger* get();
     virtual ~Logger();
    
-    void setConsoleDest();
-    void setFileDest(const std::string& sFName);
-    void setSyslogDest(int facility, int logopt);
     void setCategories(int flags);
+    void pushCategories();
+    void popCategories();
     void trace(int category, const std::string& msg);
     inline bool isFlagSet(int category) {
         return (category & m_Flags) != 0;
@@ -54,25 +56,20 @@ public:
     static const long CONFIG;  
     static const long WARNING;
     static const long ERROR;  
-    static const long WATCHDOG;  
-
     static const long MEMORY;
     static const long APP;
-    static const long LOGGER;
+    static const long PLUGIN;
+    static const long PLAYER;
 
 private:
     Logger();
     static const char * categoryToString(int category);
-    void closeDest();
+    int stringToCategory(const std::string& sCategory);
    
     static Logger* m_pLogger;
 
-    enum DestType {CONSOLE, FILE, SYSLOG};
-    DestType m_DestType;
-    
-    std::ostream * m_pDest; // For console and file
-    
     int m_Flags;
+    std::vector<int> m_FlagStack;
 };
 
 #define AVG_TRACE(category, msg) { \
