@@ -22,6 +22,7 @@
 #ifndef _OSHelper_H_
 #define _OSHelper_H_
 
+#include "../api.h"
 #include <string>
 
 namespace avg {
@@ -35,6 +36,12 @@ std::string getAvgLibPath();
 bool getEnv(const std::string & sName, std::string & sVal);
 void setEnv(const std::string & sName, const std::string & sVal);
 
+unsigned getMemoryUsage();
+
+// Converts a utf-8-encoded filename to something windows can use.
+// Under other operating systems, returns the input string.
+std::string convertUTF8ToFilename(const std::string & sName);
+
 }
 
 #endif 
