@@ -26,9 +26,10 @@
 #include <assert.h>
 #include <iostream>
 #include <sstream>
+#include <vector>
+#include <algorithm>
 
 #ifdef WIN32
-#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <Dbghelp.h>
 #else
@@ -41,24 +42,39 @@ namespace avg {
 
 using namespace std;
 
-ObjectCounter* ObjectCounter::m_pObjectCounter = 0;
+ObjectCounter* ObjectCounter::s_pObjectCounter = 0;
+bool ObjectCounter::s_bDeleted = false;
 boost::mutex * pCounterMutex;
 
+void deleteObjectCounter()
+{
+    delete ObjectCounter::s_pObjectCounter;
+    delete pCounterMutex;
+    ObjectCounter::s_pObjectCounter = 0;
+}
+
 ObjectCounter::ObjectCounter()
 {
 }
 
 ObjectCounter::~ObjectCounter()
 {
+    s_bDeleted = true;
 }
 
 ObjectCounter * ObjectCounter::get()
 {
-    if (!m_pObjectCounter) {
-        m_pObjectCounter = new ObjectCounter;
-        pCounterMutex = new boost::mutex;
+    if (!s_pObjectCounter) {
+        if (s_bDeleted) {
+            // This is _after_ the deleteObjectCounter has been called.
+            return 0;
+        } else {
+            s_pObjectCounter = new ObjectCounter;
+            pCounterMutex = new boost::mutex;
+            atexit(deleteObjectCounter);
+        }
     }
-    return m_pObjectCounter;
+    return s_pObjectCounter;
 }
 
 void ObjectCounter::incRef(const std::type_info* pType)
@@ -78,9 +94,16 @@ void ObjectCounter::incRef(const std::type_info* pType)
 void ObjectCounter::decRef(const std::type_info* pType)
 {
 #ifdef DEBUG_ALLOC
+    if (!this) {
+        // This happens if there are counted static objects that are deleted after 
+        // s_pObjectCounter has been deleted.
+        return;
+    }
     boost::mutex::scoped_lock Lock(*pCounterMutex);
     TypeMap::iterator MapEntry = m_TypeMap.find(pType);
     if (MapEntry == m_TypeMap.end()) {
+        cerr << "ObjectCounter for " << demangle(pType->name()) 
+                << " does not exist." << endl;
         // Can't decref a type that hasn't been incref'd.
         assert (false);
     } else {
@@ -112,11 +135,18 @@ std::string ObjectCounter::dump()
     stringstream ss;
     ss << "Object dump: " << endl;
     TypeMap::iterator it;
+    vector<string> strings;
     for (it = m_TypeMap.begin(); it != m_TypeMap.end(); ++it) {
+        stringstream tempStream;
         if (it->second > 0) {
-            ss << "  " << demangle(it->first->name()) << ": " << it->second << endl;
+            tempStream << "  " << demangle(it->first->name()) << ": " << it->second;
+            strings.push_back(tempStream.str());
         }
     }
+    sort(strings.begin(), strings.end());
+    for (vector<string>::iterator it=strings.begin(); it != strings.end(); ++it) {
+        ss << *it << endl;
+    }
     return ss.str();
 }
 
