@@ -22,13 +22,14 @@
 #ifndef _ObjectCounter_H_
 #define _ObjectCounter_H_
 
+#include "../api.h"
 #include <string>
 #include <map>
 #include <typeinfo>
 
 namespace avg {
 
-class ObjectCounter {
+class AVG_API ObjectCounter {
 public:
     static ObjectCounter* get();
     virtual ~ObjectCounter();
@@ -42,11 +43,14 @@ public:
 private:
     ObjectCounter();
     std::string demangle(std::string s);
+    static void deleteSingleton();
 
     typedef std::map<const std::type_info *, int> TypeMap;
     TypeMap m_TypeMap;
 
-    static ObjectCounter* m_pObjectCounter; 
+    static ObjectCounter* s_pObjectCounter;
+    static bool s_bDeleted;
+    friend void deleteObjectCounter();
 };
 
 }
