@@ -22,6 +22,7 @@
 #ifndef _Profiler_H_ 
 #define _Profiler_H_
 
+#include "../api.h"
 #include <boost/thread.hpp>
 #include <boost/thread/mutex.hpp>
 #include <boost/shared_ptr.hpp>
@@ -33,7 +34,7 @@ namespace avg {
 class ThreadProfiler;
 typedef boost::shared_ptr<ThreadProfiler> ThreadProfilerPtr;
 
-class Profiler {
+class AVG_API Profiler {
 public:
     static Profiler& get();
     virtual ~Profiler();
@@ -43,7 +44,7 @@ public:
 
     // Interface to ThreadProfiler
     void registerThreadProfiler(ThreadProfilerPtr pThreadProfiler);
-    ThreadProfilerPtr getThreadProfiler();
+    void threadProfilerStopped(ThreadProfilerPtr pThreadProfiler);
 
 private:
     Profiler();
