@@ -22,15 +22,18 @@
 #ifndef _Rect_H_
 #define _Rect_H_
 
+#include "../api.h"
 #include "Point.h"
 
+#include <algorithm>
+
 namespace avg {
 
 // Simple rectangle class.
 // If NUM is an integer, contains all points from tl up to but not including 
 // br.
 template<class NUM>
-class Rect
+class AVG_TEMPLATE_API Rect
 {
 public:
     Point<NUM> tl;
@@ -48,6 +51,7 @@ public:
     Point<NUM> center() const;
     void setWidth(NUM width);
     void setHeight(NUM height);
+    void setSize(const Point<NUM>& size);
     bool contains(const Point<NUM>& pt) const;
     bool contains(const Rect<NUM>& rect) const;
     bool intersects(const Rect<NUM>& rect) const;
@@ -132,6 +136,13 @@ void Rect<NUM>::setHeight(NUM height)
 {
     br.y = tl.y+height;
 }
+    
+template<class NUM>
+void Rect<NUM>::setSize(const Point<NUM>& size)
+{
+    setWidth(size.x);
+    setHeight(size.y);
+}
 
 template<class NUM>
 bool Rect<NUM>::contains(const Point<NUM>& pt) const
@@ -157,14 +168,6 @@ bool Rect<NUM>::intersects(const Rect<NUM>& rect) const
       return true;
 }
 
-#ifndef  min
-#define min(a, b)       ((a) < (b) ? (a) : (b))
-#endif
-
-#ifndef  max
-#define max(a, b)       ((a) < (b) ? (b) : (a))
-#endif
-
 template<class NUM>
 void Rect<NUM>::expand(const Rect<NUM>& rect)
 {
