@@ -21,7 +21,6 @@
 
 #include "ThreadProfiler.h"
 #include "Logger.h"
-#include "ObjectCounter.h"
 
 #include <sstream>
 #include <iomanip>
@@ -31,11 +30,25 @@ using namespace boost;
 
 namespace avg {
     
-ThreadProfiler::ThreadProfiler(const string& sName)
-    : m_sName(sName)
+thread_specific_ptr<ThreadProfilerPtr> ThreadProfiler::s_pInstance;
+
+ThreadProfilerPtr& ThreadProfiler::get() 
+{
+    if (s_pInstance.get() == 0) {
+        s_pInstance.reset(new ThreadProfilerPtr(new ThreadProfiler()));
+    }
+    return *s_pInstance;
+}
+
+void ThreadProfiler::kill()
+{
+    s_pInstance.reset();
+}
+
+ThreadProfiler::ThreadProfiler()
+    : m_sName("")
 {
     m_bRunning = false;
-    ObjectCounter::get()->incRef(&typeid(*this));
 }
 
 ThreadProfiler::~ThreadProfiler() 
@@ -45,7 +58,6 @@ ThreadProfiler::~ThreadProfiler()
             delete *it;
         }
     }
-    ObjectCounter::get()->decRef(&typeid(*this));
 }
 
 void ThreadProfiler::addZone(ProfilingZone& Zone)
@@ -152,10 +164,20 @@ int ThreadProfiler::getIndent()
     return int(2*m_ActiveZones.size());
 }
 
-const std::string& ThreadProfiler::getName()
+int ThreadProfiler::getNumZones()
+{
+    return m_Zones.size();
+}
+
+const std::string& ThreadProfiler::getName() const
 {
     return m_sName;
 }
 
+void ThreadProfiler::setName(const std::string& sName)
+{
+    m_sName = sName;
+}
+
 }
 
