@@ -22,11 +22,12 @@
 #ifndef _ThreadProfiler_H_ 
 #define _ThreadProfiler_H_
 
-#include "Profiler.h"
+#include "../api.h"
 #include "ProfilingZone.h"
 
 #include <boost/thread.hpp>
 #include <boost/shared_ptr.hpp>
+#include <boost/thread/tss.hpp>
 
 #include <list>
 #include <map>
@@ -36,14 +37,12 @@ namespace avg {
 class ThreadProfiler;
 typedef boost::shared_ptr<ThreadProfiler> ThreadProfilerPtr;
 
-class ThreadProfiler {
+class AVG_API ThreadProfiler {
 public:
-    ThreadProfiler(const std::string& sName);
+    static ThreadProfilerPtr& get();
+    static void kill();
+    ThreadProfiler();
     virtual ~ThreadProfiler();
-    static ThreadProfilerPtr get()
-    {
-        return Profiler::get().getThreadProfiler();
-    }
  
     void addZone(ProfilingZone& Zone);
     void clear();
@@ -55,12 +54,10 @@ public:
     void dumpStatistics();
     void reset();
     int getIndent();
+    int getNumZones();
 
-    bool isCurrent() 
-    {
-        return boost::thread() == m_Thread;
-    }
-    const std::string& getName();
+    const std::string& getName() const;
+    void setName(const std::string& sName);
 
 private:
     std::string m_sName;
@@ -70,6 +67,8 @@ private:
     ZoneList m_ActiveZones;
     bool m_bRunning;
     boost::thread m_Thread;
+
+    static boost::thread_specific_ptr<ThreadProfilerPtr> s_pInstance;
 };
 
 }
