@@ -22,24 +22,13 @@
 #ifndef _TimeSource_H_ 
 #define _TimeSource_H_
 
-#ifdef _WIN32
-#define WIN32_LEAN_AND_MEAN  /* somewhat limit Win32 pollution */
-#include <Winsock2.h>
-#undef ERROR
-#undef WARNING
-#else
-#include <sys/time.h>
-#endif
-
-#ifdef __APPLE__
-#include <mach/mach_time.h>
-#endif
+#include "../api.h"
 
 namespace avg {
 
 typedef unsigned long long CycleCount;
 
-class TimeSource {
+class AVG_API TimeSource {
 public:
     static TimeSource* get();
     virtual ~TimeSource();
@@ -55,7 +44,7 @@ private:
     static TimeSource* m_pTimeSource;
 };
 
-void msleep(int millisecs);
+void AVG_API msleep(int millisecs);
 
 }
 
