@@ -22,17 +22,20 @@
 #ifndef _UTF8String_H_
 #define _UTF8String_H_
 
+#include "../api.h"
 #include <string>
 
 namespace avg {
 
-class UTF8String: public std::string
+class AVG_API UTF8String: public std::string
 {
 public:
     UTF8String();
     UTF8String(const std::string& s);
+    UTF8String(const char * psz);
 
     UTF8String& operator =(const std::string& s);
+    UTF8String& operator =(const char * psz);
 
 };
 
