@@ -22,12 +22,14 @@
 #ifndef _WorkerThread_H_
 #define _WorkerThread_H_
 
+#include "../api.h"
 #include "Command.h"
 #include "Exception.h"
 #include "Logger.h"
 #include "Queue.h"
 #include "Profiler.h"
 #include "ThreadProfiler.h"
+#include "ObjectCounter.h"
 
 #include <boost/shared_ptr.hpp>
 
@@ -37,13 +39,14 @@ namespace avg {
 
 
 template<class DERIVED_THREAD>
-class WorkerThread {
+class AVG_TEMPLATE_API WorkerThread {
 public:
     typedef Queue<Command<DERIVED_THREAD> > CmdQueue;
     typedef boost::shared_ptr<CmdQueue> CmdQueuePtr;
 
     WorkerThread(const std::string& sName, CmdQueue& CmdQ);
-    virtual ~WorkerThread() {};
+    WorkerThread(WorkerThread const& other);
+    virtual ~WorkerThread();
     void operator()();
 
     void stop();
@@ -58,8 +61,6 @@ private:
     std::string m_sName;
     bool m_bShouldStop;
     CmdQueue& m_CmdQ;
-
-    ThreadProfilerPtr m_pProfiler;
 };
 
 template<class DERIVED_THREAD>
@@ -68,20 +69,37 @@ WorkerThread<DERIVED_THREAD>::WorkerThread(const std::string& sName, CmdQueue& C
       m_bShouldStop(false),
       m_CmdQ(CmdQ)
 {
+    ObjectCounter::get()->incRef(&typeid(*this));
+}
+
+template<class DERIVED_THREAD>
+WorkerThread<DERIVED_THREAD>::WorkerThread(WorkerThread const& other)
+    : m_CmdQ(other.m_CmdQ)
+{
+    m_sName = other.m_sName;
+    m_bShouldStop = other.m_bShouldStop;
+    ObjectCounter::get()->incRef(&typeid(*this));
+}
+
+template<class DERIVED_THREAD>
+WorkerThread<DERIVED_THREAD>::~WorkerThread()
+{
+    ObjectCounter::get()->decRef(&typeid(*this));
 }
 
 template<class DERIVED_THREAD>
 void WorkerThread<DERIVED_THREAD>::operator()()
 {
     try {
-        m_pProfiler = ThreadProfilerPtr(new ThreadProfiler(m_sName));
-        Profiler::get().registerThreadProfiler(m_pProfiler);
+        ThreadProfilerPtr pProfiler = ThreadProfiler::get();
+        pProfiler->setName(m_sName);
+        Profiler::get().registerThreadProfiler(pProfiler);
         bool bOK;
         bOK = init();
         if (!bOK) {
             return;
         }
-        m_pProfiler->start();
+        pProfiler->start();
         while (!m_bShouldStop) {
             bOK = work();
             if (!bOK) {
@@ -89,9 +107,10 @@ void WorkerThread<DERIVED_THREAD>::operator()()
             } else {
                 processCommands();
             }
-            m_pProfiler->reset();
+            pProfiler->reset();
         }
         deinit();
+        Profiler::get().threadProfilerStopped(pProfiler);
     } catch (const Exception& e) {
          AVG_TRACE(Logger::ERROR, "Uncaught exception in thread " << m_sName << ": "
                   << e.GetStr());
