@@ -24,6 +24,7 @@
 #include "WorkerThread.h"
 #include "ObjectCounter.h"
 #include "Point.h"
+#include "Triangulate.h"
 #include "GeomHelper.h"
 #include "OSHelper.h"
 #include "FileHelper.h"
@@ -31,6 +32,8 @@
 #include "MathHelper.h"
 #include "CubicSpline.h"
 #include "BicubicSpline.h"
+#include "BezierCurve.h"
+#include "Signal.h"
 
 #include "TestSuite.h"
 #include "TimeSource.h"
@@ -217,6 +220,12 @@ public:
     }
 };
 
+// The following pragmas avoid a compiler warning (potential division by 0)
+#ifdef _MSC_VER
+#pragma optimize("", off)
+#pragma warning(push)
+#pragma warning(disable:4723)
+#endif
 class PointTest: public Test {
 public:
     PointTest()
@@ -226,6 +235,19 @@ public:
 
     void runTests() 
     {
+        double one = 1;
+        double zero = 0;
+
+        TEST(isinf(-one/zero) != 0);
+        TEST(isinf(one/zero) != 0);
+        TEST(isinf(one) == 0);
+        TEST(isnan(sqrt(-one)) != 0);
+        TEST(isnan(sqrt(one+one)) == 0);
+
+        // TODO: Move to a separate math test once we're done here.
+        TEST(almostEqual(invSqrt(1), 1));
+        TEST(almostEqual(invSqrt(4), 0.5));
+
         // TODO: The point tests aren't complete!
         DPoint pt1(0,0);
         DPoint pt2(3,4);
@@ -238,23 +260,30 @@ public:
         DPoint pt3(v);
         TEST(almostEqual(pt2, pt3));
         TEST(almostEqual(pt3.getNorm(), 5));
-        pt3.normalize();
-        TEST(almostEqual(pt3.getNorm(), 1));
+        DPoint pt4 = pt3.getNormalized();
+        TEST(almostEqual(pt4.getNorm(), 1, 0.0001));
         {
-            DLine l1(DPoint(0,0), DPoint(2,2));
-            DLine l2(DPoint(2,0), DPoint(0,2));
-            TEST(linesIntersect(l1, l2));
-            TEST(linesIntersect(l2, l1));
+            DLineSegment l1(DPoint(0,0), DPoint(2,2));
+            DLineSegment l2(DPoint(2,0), DPoint(0,2));
+            TEST(lineSegmentsIntersect(l1, l2));
+            TEST(lineSegmentsIntersect(l2, l1));
         }
         {
-            DLine l1(DPoint(0,0), DPoint(0,2));
-            DLine l2(DPoint(2,0), DPoint(2,2));
-            TEST(!linesIntersect(l1, l2));
+            DLineSegment l1(DPoint(0,0), DPoint(0,2));
+            DLineSegment l2(DPoint(2,0), DPoint(2,2));
+            TEST(!lineSegmentsIntersect(l1, l2));
         }
         {
-            DLine l1(DPoint(0,0), DPoint(2,0));
-            DLine l2(DPoint(0,2), DPoint(2,2));
-            TEST(!linesIntersect(l1, l2));
+            DLineSegment l1(DPoint(0,0), DPoint(2,0));
+            DLineSegment l2(DPoint(0,2), DPoint(2,2));
+            TEST(!lineSegmentsIntersect(l1, l2));
+        }
+        {
+            DLineSegment l1(DPoint(0,0), DPoint(2,0));
+            TEST(l1.isPointOver(DPoint(1,23)));
+            TEST(l1.isPointOver(DPoint(1.9,-5)));
+            TEST(!l1.isPointOver(DPoint(-1,1)));
+            TEST(!l1.isPointOver(DPoint(3,-1)));
         }
         {
             DPoint pt0(DPoint(1,1));
@@ -271,8 +300,64 @@ public:
             poly.push_back(DPoint(2,1));
             TEST(!pointInPolygon(pt0, poly));
         }
+        {
+            DPoint p1(DPoint(0,0));
+            DPoint v1(DPoint(1,1));
+            DPoint p2(DPoint(2,1));
+            DPoint v2(DPoint(1,0));
+            TEST(getLineLineIntersection(p1, v1, p2, v2) == DPoint(1,1));
+        }
+        TEST(almostEqual(DPoint(10,0).getRotatedPivot(PI, DPoint(15,5)), DPoint(20,10)));
+        TEST(almostEqual(DPoint(10,0).getRotatedPivot(PI*0.5, DPoint(15,5)), DPoint(20,0)));
+        TEST(almostEqual(DPoint(10,0).getRotatedPivot(PI*1.5, DPoint(15,5)), DPoint(10,10)));
+        TEST(almostEqual(DPoint(10,0).getRotatedPivot(PI*2, DPoint(15,5)), DPoint(10,0)));
+        TEST(almostEqual(DPoint(23,0).getRotatedPivot(PI*0.5), DPoint(0,23)));
+
+        TEST(almostEqual(DPoint(10,0), DPoint::fromPolar(0, 10)));
+        TEST(almostEqual(DPoint(0,10), DPoint::fromPolar(PI*0.5, 10)));
+        TEST(almostEqual(DPoint(0,-1), DPoint::fromPolar(PI*1.5, 1)));
     }
 };
+#ifdef _MSC_VER
+#pragma warning(pop)
+#pragma optimize("", on)
+#endif
+
+class TriangleTest: public Test {
+public:
+    TriangleTest()
+        : Test("TriangleTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        Triangle tri(DPoint(0,0), DPoint(4,4), DPoint(4,8));
+        TEST(tri.isInside(DPoint(3,4)));
+        TEST(!tri.isInside(DPoint(1,4)));
+        TEST(!tri.isInside(DPoint(2,1)));
+        TEST(!tri.isInside(DPoint(-2,5)));
+        TEST(!tri.isInside(DPoint(5,5)));
+
+        DPoint polyArray[] = {DPoint(0,0), DPoint(8,2), DPoint(9,0), DPoint(9,3), 
+                DPoint(1,1), DPoint(0,3)}; 
+
+        DPointVector poly = vectorFromCArray(6, polyArray);
+
+        vector<int> triangulation;
+        triangulatePolygon(poly, triangulation);
+
+        TEST(triangulation.size() == 4*3);
+        int baselineIndexes[] = {1,2,3, 4,5,0, 0,1,3, 3,4,0};
+        TEST(triangulation == vectorFromCArray(12, baselineIndexes));
+/*        
+        for (unsigned int i=0; i<triangulation.size(); i++) {
+            cerr << i << ":" << triangulation[i] << endl;
+        }
+*/        
+    }
+
+};
 
 class FileTest: public Test {
 public:
@@ -297,8 +382,11 @@ public:
 
     void runTests()
     {
-#if defined(__APPLE__) || defined (_WIN32)
+#if defined(__APPLE__) || defined(_WIN32)
         TEST(getAvgLibPath() != "");
+#endif
+#ifdef __APPLE__
+        TEST(getMemoryUsage() != 0);
 #endif
     }
 };
@@ -324,6 +412,13 @@ public:
             }
         }
         TEST(bExceptionThrown);
+        TEST(stringToDPoint(" ( 3.4 , 2.1 ) ")== DPoint(3.4, 2.1));
+        vector<double> v;
+        fromString("(1,2,3,4,5)", v);
+        TEST(v.size() == 5 && v[0] == 1 && v[4] == 5);
+        v.clear();
+        fromString("()", v);
+        TEST(v.size() == 0);
     }
 };
 
@@ -418,6 +513,180 @@ public:
     }
 };
 
+class BezierCurveTest: public Test {
+public:
+    BezierCurveTest()
+        : Test("BezierCurveTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        BezierCurve curve(DPoint(0,0), DPoint(1,0), DPoint(1,1), DPoint(0,1));
+        TEST(almostEqual(curve.interpolate(0), DPoint(0,0)));
+        TEST(almostEqual(curve.getDeriv(0), DPoint(3, 0)));
+        TEST(almostEqual(curve.interpolate(1), DPoint(0,1)));
+        TEST(almostEqual(curve.getDeriv(1), DPoint(-3, 0)));
+        TEST(almostEqual(curve.interpolate(0.5), DPoint(0.75,0.5)));
+    }
+};
+
+class Listener {
+public:
+    Listener(Signal<Listener>& signal)
+        : m_Signal(signal),
+          m_bFuncCalled(false)
+    {
+    }
+
+    virtual ~Listener()
+    {}
+
+    virtual void func()
+    {
+        m_bFuncCalled = true;
+    }
+
+    bool funcCalled() const
+    {
+        return m_bFuncCalled;
+    }
+
+    void reset()
+    {
+        m_bFuncCalled = false;
+    }
+
+protected:
+    Signal<Listener>& m_Signal;
+
+private:
+    bool m_bFuncCalled;
+};
+
+class DisconnectingSelfListener: public Listener {
+public:
+    DisconnectingSelfListener(Signal<Listener>& signal)
+        : Listener(signal)
+    {
+    }
+
+    virtual void func()
+    {
+        Listener::func();
+        m_Signal.disconnect(this);
+    }
+};
+
+class DisconnectingOtherListener: public Listener {
+public:
+    DisconnectingOtherListener(Signal<Listener>& signal, Listener* pOther)
+        : Listener(signal),
+          m_pOther(pOther)
+    {
+    }
+
+    virtual void func()
+    {
+        Listener::func();
+        if (m_pOther) {
+            m_Signal.disconnect(m_pOther);
+            m_pOther = 0;
+        }
+    }
+
+private:
+    Listener* m_pOther;
+};
+
+class ConnectingOtherListener: public Listener {
+public:
+    ConnectingOtherListener(Signal<Listener>& signal, Listener* pOther)
+        : Listener(signal),
+          m_pOther(pOther)
+    {
+    }
+
+    virtual void func()
+    {
+        Listener::func();
+        if (m_pOther) {
+            m_Signal.connect(m_pOther);
+            m_pOther = 0;
+        }
+    }
+
+private:
+    Listener* m_pOther;
+};
+
+
+
+class SignalTest: public Test {
+public:
+    SignalTest()
+        : Test("SignalTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        Signal<Listener> s(&Listener::func);
+        Listener l1(s);
+        Listener l2(s);
+        s.connect(&l1);
+        s.connect(&l2);
+        s.emit();
+        TEST(l1.funcCalled() && l2.funcCalled());
+        l1.reset();
+        l2.reset();
+
+        s.disconnect(&l1);
+        s.emit();
+        TEST(!(l1.funcCalled()) && l2.funcCalled());
+        l2.reset();
+
+        {
+            DisconnectingSelfListener disconnecter(s);
+            s.connect(&disconnecter);
+            s.emit();
+            TEST(l2.funcCalled() && disconnecter.funcCalled());
+            TEST(s.getNumListeners() == 1);
+            l2.reset();
+            disconnecter.reset();
+
+            s.emit();
+            TEST(l2.funcCalled() && !(disconnecter.funcCalled()));
+            l2.reset();
+        }
+        {
+            DisconnectingOtherListener disconnecter(s, &l2);
+            s.connect(&disconnecter);
+            s.emit();
+            TEST(l2.funcCalled() && disconnecter.funcCalled());
+            TEST(s.getNumListeners() == 1);
+            l2.reset();
+            disconnecter.reset();
+
+            s.emit();
+            TEST(!(l2.funcCalled()) && disconnecter.funcCalled());
+            s.disconnect(&disconnecter);
+        }
+        {
+            ConnectingOtherListener connecter(s, &l2);
+            s.connect(&connecter);
+            s.emit();
+            TEST(l2.funcCalled() && connecter.funcCalled());
+            TEST(s.getNumListeners() == 2);
+            l2.reset();
+            connecter.reset();
+
+            s.emit();
+            TEST(l2.funcCalled() && connecter.funcCalled());
+        }
+    }
+};
+
 class BaseTestSuite: public TestSuite {
 public:
     BaseTestSuite() 
@@ -427,10 +696,13 @@ public:
         addTest(TestPtr(new WorkerThreadTest));
         addTest(TestPtr(new ObjectCounterTest));
         addTest(TestPtr(new PointTest));
+        addTest(TestPtr(new TriangleTest));
         addTest(TestPtr(new FileTest));
         addTest(TestPtr(new OSTest));
         addTest(TestPtr(new StringTest));
         addTest(TestPtr(new SplineTest));
+        addTest(TestPtr(new BezierCurveTest));
+        addTest(TestPtr(new SignalTest));
     }
 };
 
