@@ -29,8 +29,16 @@
 #include "../base/Exception.h"
 #include "../base/Logger.h"
 #include "../base/ObjectCounter.h"
+#include "../base/StringHelper.h"
+#include "../base/MathHelper.h"
+#include "../base/OSHelper.h"
 
 #include <Magick++.h>
+
+#if defined(__SSE__) || defined(_WIN32)
+#include <xmmintrin.h>
+#endif
+
 #include <assert.h>
 #include <cstring>
 #include <iostream>
@@ -45,7 +53,22 @@ namespace avg {
 template<class Pixel>
 void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src);
 
-Bitmap::Bitmap(IntPoint Size, PixelFormat PF, const std::string& sName)
+bool Bitmap::s_bMagickInitialized = false;
+
+Bitmap::Bitmap(DPoint Size, PixelFormat PF, const UTF8String& sName)
+    : m_Size(Size),
+      m_PF(PF),
+      m_pBits(0),
+      m_bOwnsBits(true),
+      m_sName(sName)
+{
+//    cerr << "Bitmap::Bitmap(" << Size << ", " << getPixelFormatString(m_PF) << ", " 
+//        << sName << ")" << endl;
+    ObjectCounter::get()->incRef(&typeid(*this));
+    allocBits();
+}
+
+Bitmap::Bitmap(IntPoint Size, PixelFormat PF, const UTF8String& sName)
     : m_Size(Size),
       m_PF(PF),
       m_pBits(0),
@@ -59,7 +82,7 @@ Bitmap::Bitmap(IntPoint Size, PixelFormat PF, const std::string& sName)
 }
 
 Bitmap::Bitmap(IntPoint Size, PixelFormat PF, unsigned char * pBits, 
-        int Stride, bool bCopyBits, const std::string& sName)
+        int Stride, bool bCopyBits, const UTF8String& sName)
     : m_Size(Size),
       m_PF(PF),
       m_pBits(0),
@@ -120,7 +143,7 @@ Bitmap::Bitmap(Bitmap& Orig, const IntRect& Rect)
     initWithData(pRegionStart, Orig.getStride(), false);
 }
 
-Bitmap::Bitmap(const std::string& sURI)
+Bitmap::Bitmap(const UTF8String& sURI)
     : m_pBits(0),
       m_sName(sURI)
 {
@@ -130,12 +153,17 @@ Bitmap::Bitmap(const std::string& sURI)
 // It also returns RGB bitmaps, but I think nearly everywhere in libavg, the bytes
 // are swapped and BGR is used.
 //    cerr << "Bitmap::Bitmap(" << sURI << ")" << endl;
+    if (!s_bMagickInitialized) {
+        InitializeMagick(0);
+        s_bMagickInitialized = true;
+    }
     Image Img;
     try {
-        Img.read(sURI);
-    } catch( Magick::Warning &e) {
+        string sFilename = convertUTF8ToFilename(sURI);
+        Img.read(sFilename);
+    } catch(Magick::Warning &e) {
         cerr << e.what() << endl;
-    } catch( Magick::ErrorConfigure &e) {
+    } catch(Magick::ErrorConfigure &) {
 //        cerr << e.what() << endl;
     }
     PixelPacket * pSrcPixels = Img.getPixels(0, 0, Img.columns(), Img.rows());
@@ -201,7 +229,7 @@ void Bitmap::copyPixels(const Bitmap & Orig)
 {
 //    cerr << "Bitmap::copyPixels(): " << getPixelFormatString(Orig.getPixelFormat()) << "->" 
 //            << getPixelFormatString(m_PF) << endl;
-    if (&Orig == this) {
+    if (&Orig == this || Orig.getPixels() == m_pBits) {
         return;
     }
     if (Orig.getPixelFormat() == m_PF) {
@@ -262,11 +290,14 @@ void Bitmap::copyPixels(const Bitmap & Orig)
                         assert(false);
                 }
                 break;
+            case BAYER8_RGGB:
             case BAYER8_GBRG:
+            case BAYER8_GRBG:
+            case BAYER8_BGGR:
                 switch(m_PF) {
-                    // Bayer patterns are saved as I8 bitmaps. So simply copy that
                     case I8:
                         {
+                            // Bayer patterns are saved as I8 bitmaps. So simply copy that.
                             const unsigned char * pSrc = Orig.getPixels();
                             unsigned char * pDest = m_pBits;
                             int Height = min(Orig.getSize().y, m_Size.y);
@@ -290,8 +321,26 @@ void Bitmap::copyPixels(const Bitmap & Orig)
                         assert(false);
                 }
                 break;
+            case R32G32B32A32F:
+                if (getBytesPerPixel() == 4) {
+                    FloatRGBAtoByteRGBA(Orig);
+                } else {
+                    cerr << "Can't convert " << Orig.getPixelFormatString() 
+                        << " to " << getPixelFormatString() << endl;
+                    assert(false);
+                }
+                break;
             default:
                 switch(m_PF) {
+                    case R32G32B32A32F:
+                        if (Orig.getBytesPerPixel() == 4) {
+                            ByteRBBAtoFloatRGBA(Orig);
+                        } else {
+                            cerr << "Can't convert " << Orig.getPixelFormatString() 
+                                    << " to " << getPixelFormatString() << endl;
+                            assert(false);
+                        }
+                        break;
                     case B8G8R8A8:
                     case B8G8R8X8:
                     case A8B8G8R8:
@@ -323,8 +372,202 @@ void Bitmap::copyPixels(const Bitmap & Orig)
     }
 }
 
-void Bitmap::save(const std::string& sFilename)
+inline void YUVtoBGR32Pixel(Pixel32* pDest, int y, int u, int v)
 {
+    // u = Cb, v = Cr
+    int u1 = u - 128;
+    int v1 = v - 128;
+    int tempy = 298*(y-16);
+    int b = (tempy + 516 * u1           ) >> 8;
+    int g = (tempy - 100 * u1 - 208 * v1) >> 8;
+    int r = (tempy            + 409 * v1) >> 8;
+
+    if (b<0) b = 0;
+    if (b>255) b= 255;
+    if (g<0) g = 0;
+    if (g>255) g= 255;
+    if (r<0) r = 0;
+    if (r>255) r= 255;
+    pDest->set(b,g,r,255);
+}
+
+#if defined(__SSE__) || defined(_WIN32)
+ostream& operator<<(ostream& os, const __m64 &val)
+{
+    unsigned char * pVal = (unsigned char *)(&val);
+    for (int i=0; i<8; ++i) {
+        os << hex << setw(2) << setfill('0') << int(pVal[i]);
+        if (i%2 == 1) {
+            os << " ";
+        }
+        if (i%4 == 3) {
+            os << " ";
+        }
+    }
+    return os;
+}
+#endif
+
+void Bitmap::copyYUVPixels(const Bitmap & yOrig, const Bitmap& uOrig,
+        const Bitmap& vOrig)
+{
+    int Height = min(yOrig.getSize().y, m_Size.y);
+    int Width = min(yOrig.getSize().x, m_Size.x);
+
+    int yStride = yOrig.getStride();
+    int uStride = uOrig.getStride();
+    int vStride = vOrig.getStride();
+    int destStride = m_Stride/getBytesPerPixel();
+    Pixel32 * pDestLine = (Pixel32*)m_pBits;
+
+#if defined(__SSE__) || defined(_WIN32)
+#pragma pack(16)
+    // Original SSE conversion code taken from liboggplay: oggplay_sse_x86.c
+    int               i;     
+    const unsigned char   * ptry;
+    const unsigned char   * ptru;
+    const unsigned char   * ptrv;
+
+    register __m64    *o;
+    register __m64    y, ylo, yhi;
+    register __m64    zero, ut, vt, imm;
+    register __m64    r, g, b;
+    register __m64    tmp, tmp2;
+
+    zero = _mm_setzero_si64(); 
+
+    ptry = yOrig.getPixels();
+    ptru = uOrig.getPixels();
+    ptrv = vOrig.getPixels();
+
+    for (i = 0; i < Height; i++) {
+        int j;
+        o = (__m64*)pDestLine;
+        pDestLine += destStride;
+        for (j = 0; j < Width; j += 8) {
+
+            // y' = (298*(y-16))
+            // ylo and yhi contain 4 pixels each
+            y = *(__m64*)(&(ptry[j]));
+            ylo = _m_punpcklbw(y, zero);
+            imm = _mm_set1_pi16(16);
+            ylo = _m_psubusw(ylo, imm);
+            imm = _mm_set1_pi16(149);
+            ylo = _m_pmullw(ylo, imm);
+            ylo = _mm_srli_pi16(ylo, 7);
+           
+            yhi = _m_punpckhbw(y, zero);
+            imm = _mm_set1_pi16(16);
+            yhi = _m_psubusw(yhi, imm);
+            imm = _mm_set1_pi16(149);
+            yhi = _m_pmullw(yhi, imm);
+            yhi = _mm_srli_pi16(yhi, 7);
+
+            ut = _m_from_int(*(int *)(ptru + j/2));
+            vt = _m_from_int(*(int *)(ptrv + j/2));
+
+            ut = _m_punpcklbw(ut, zero);
+            vt = _m_punpcklbw(vt, zero);
+
+            /* subtract 128 from u and v */ 
+            imm = _mm_set1_pi16(128);
+            ut = _m_psubw(ut, imm);
+            vt = _m_psubw(vt, imm);
+
+            /* transfer and multiply into r, g, b registers */
+            imm = _mm_set1_pi16(-50);
+            g = _m_pmullw(ut, imm);
+            imm = _mm_set1_pi16(129);
+            b = _m_pmullw(ut, imm);
+            imm = _mm_set1_pi16(204);
+            r = _m_pmullw(vt, imm);
+            imm = _mm_set1_pi16(-104);
+            imm = _m_pmullw(vt, imm);
+            g = _m_paddsw(g, imm);
+
+            /* shift r, g and b registers to the right */
+            r = _m_psrawi(r, 7);
+            g = _m_psrawi(g, 7);
+            b = _m_psrawi(b, 6);
+            /* Now r, g, b contain 4 words each of the u and v inputs for the color */
+            /* channels. */
+
+            /* duplicate u and v channels and add y
+             * each of r,g, b in the form [s1(16), s2(16), s3(16), s4(16)]
+             * first interleave, so tmp is [s1(16), s1(16), s2(16), s2(16)]
+             * then add y, then interleave again
+             * then pack with saturation, to get the desired output of
+             *   [s1(8), s1(8), s2(8), s2(8), s3(8), s3(8), s4(8), s4(8)]
+             */
+            tmp = _m_punpckhwd(r, r);
+            tmp = _m_paddsw(tmp, yhi);
+            tmp2 = _m_punpcklwd(r, r);
+            tmp2 = _m_paddsw(tmp2, ylo);
+            r = _m_packuswb(tmp2, tmp);
+
+            tmp = _m_punpckhwd(g, g);
+            tmp2 = _m_punpcklwd(g, g);
+            tmp = _m_paddsw(tmp, yhi);
+            tmp2 = _m_paddsw(tmp2, ylo);
+            g = _m_packuswb(tmp2, tmp);
+
+            tmp = _m_punpckhwd(b, b);
+            tmp2 = _m_punpcklwd(b, b);
+            tmp = _m_paddsw(tmp, yhi);
+            tmp2 = _m_paddsw(tmp2, ylo);
+            b = _m_packuswb(tmp2, tmp);
+
+            /* now we have 8 8-bit r, g and b samples.  we want these to be packed
+             * into 32-bit values.
+             */
+            //r = _m_from_int(0);
+            //b = _m_from_int(0);
+            imm = _mm_set1_pi32(0xFFFFFFFF);
+            tmp = _m_punpcklbw(b, r);
+            tmp2 = _m_punpcklbw(g, imm);
+            *o++ = _m_punpcklbw(tmp, tmp2);
+            *o++ = _m_punpckhbw(tmp, tmp2);
+            //printf("tmp, tmp2, write1, write2: %llx %llx %llx %llx\n", tmp, tmp2,
+            //                _m_punpcklbw(tmp, tmp2), _m_punpckhbw(tmp, tmp2));
+            tmp = _m_punpckhbw(b, r);
+            tmp2 = _m_punpckhbw(g, imm);
+            *o++ = _m_punpcklbw(tmp, tmp2);
+            *o++ = _m_punpckhbw(tmp, tmp2);
+        }
+        if (i & 0x1) {
+            ptru += uStride;
+            ptrv += vStride;
+        }
+        ptry += yStride;
+    }
+    _m_empty();
+#pragma pack()
+#else
+    assert(m_PF==B8G8R8X8);
+    const unsigned char * pYSrc = yOrig.getPixels();
+    const unsigned char * pUSrc = uOrig.getPixels();
+    const unsigned char * pVSrc = vOrig.getPixels();
+
+    for (int y=0; y<Height; ++y) {
+        for (int x=0; x<Width; ++x) {
+            YUVtoBGR32Pixel(pDestLine + x, pYSrc[x], pUSrc[x/2], pVSrc[x/2]);
+        }
+        pDestLine += destStride;
+        pYSrc += yStride;
+        if (y % 2 == 1) {
+            pUSrc += uStride;
+            pVSrc += vStride;
+        }
+    }
+#endif
+}
+
+void Bitmap::save(const UTF8String& sFilename)
+{
+    if (!s_bMagickInitialized) {
+        InitializeMagick(0);
+        s_bMagickInitialized = true;
+    }
 //    cerr << "Bitmap::save()" << endl;
     string sPF;
     BitmapPtr pBmp;
@@ -376,6 +619,11 @@ void Bitmap::save(const std::string& sFilename)
             pBmp = BitmapPtr(new Bitmap(*this));
             sPF = "I";
             break;
+        case R32G32B32A32F:
+            pBmp = BitmapPtr(new Bitmap(*this));
+            ChannelFormat = Magick::FloatPixel;
+            sPF = "RGBA";
+            break;
         default:
             cerr << "Unsupported pixel format " << getPixelFormatString(m_PF) 
                     << endl;
@@ -463,13 +711,119 @@ std::string Bitmap::getPixelFormatString(PixelFormat PF)
             return "YCbCr420p";
         case YCbCrJ420p:
             return "YCbCrJ420p";
+        case BAYER8:
+            return "BAYER8";
+        case BAYER8_RGGB:
+            return "BAYER8_RGGB";
         case BAYER8_GBRG:
             return "BAYER8_GBRG";
+        case BAYER8_GRBG:
+            return "BAYER8_GRBG";
+        case BAYER8_BGGR:
+            return "BAYER8_BGGR";
+        case R32G32B32A32F:
+            return "R32G32B32A32F";
+        case I32F:
+            return "I32F";
         default:
             return "Unknown";
     }
 }
 
+PixelFormat Bitmap::stringToPixelFormat(const string& s)
+{
+    if (s == "B5G6R5") {
+        return B5G6R5;
+    }
+    if (s == "B8G8R8" || s == "BGR") {
+        return B8G8R8;
+    }
+    if (s == "B8G8R8A8") {
+        return B8G8R8A8;
+    }
+    if (s == "B8G8R8X8") {
+        return B8G8R8X8;
+    }
+    if (s == "A8B8G8R8") {
+        return A8B8G8R8;
+    }
+    if (s == "X8B8G8R8") {
+        return X8B8G8R8;
+    }
+    if (s == "R5G6B5") {
+        return R5G6B5;
+    }
+    if (s == "R8G8B8" || s == "RGB") {
+        return R8G8B8;
+    }
+    if (s == "R8G8B8A8") {
+        return R8G8B8A8;
+    }
+    if (s == "R8G8B8X8") {
+        return R8G8B8X8;
+    }
+    if (s == "A8R8G8B8") {
+        return A8R8G8B8;
+    }
+    if (s == "X8R8G8B8") {
+        return X8R8G8B8;
+    }
+    if (s == "I8") {
+        return I8;
+    }
+    if (s == "I16") {
+        return I16;
+    }
+    if (s == "YCbCr411" || s == "YUV411") {
+        return YCbCr411;
+    }
+    if (s == "YCbCr422" || s == "YUV422") {
+        return YCbCr422;
+    }
+    if (s == "YUYV422") {
+        return YUYV422;
+    }
+    if (s == "YCbCr420p") {
+        return YCbCr420p;
+    }
+    if (s == "YCbCrJ420p") {
+        return YCbCrJ420p;
+    }
+    if (s == "BAYER8") {
+        return BAYER8;
+    }
+    if (s == "BAYER8_RGGB") {
+        return BAYER8_RGGB;
+    }
+    if (s == "BAYER8_GBRG") {
+        return BAYER8_GBRG;
+    }
+    if (s == "BAYER8_GRBG") {
+        return BAYER8_GRBG;
+    }
+    if (s == "BAYER8_BGGR") {
+        return BAYER8_BGGR;
+    }
+    if (s == "R32G32B32A32F") {
+        return R32G32B32A32F;
+    }
+    if (s == "I32F") {
+        return I32F;
+    }
+    return NO_PIXELFORMAT;
+}
+
+bool Bitmap::pixelFormatIsColored(PixelFormat pf)
+{
+    return (pf != I8 && pf != I16 && pf != I32F);
+}
+
+bool Bitmap::pixelFormatIsBayer(PixelFormat pf)
+{
+    return (pf == BAYER8_RGGB || pf == BAYER8_GBRG
+            || pf == BAYER8_GRBG || pf == BAYER8_BGGR);
+}
+
 unsigned char * Bitmap::getPixels()
 {
     return m_pBits;
@@ -514,6 +868,8 @@ int Bitmap::getBytesPerPixel() const
 int Bitmap::getBytesPerPixel(PixelFormat PF)
 {
     switch (PF) {
+        case R32G32B32A32F:
+            return 16;
         case A8B8G8R8:
         case X8B8G8R8:
         case A8R8G8B8:
@@ -522,6 +878,7 @@ int Bitmap::getBytesPerPixel(PixelFormat PF)
         case B8G8R8X8:
         case R8G8B8A8:
         case R8G8B8X8:
+        case I32F:
             return 4;
         case R8G8B8:
         case B8G8R8:
@@ -531,13 +888,19 @@ int Bitmap::getBytesPerPixel(PixelFormat PF)
         case I16:
             return 2;
         case I8:
+        case BAYER8:
+        case BAYER8_RGGB:
         case BAYER8_GBRG:
+        case BAYER8_GRBG:
+        case BAYER8_BGGR:
             return 1;
         case YUYV422:
         case YCbCr422:
             return 2;
         default:
-            fatalError("Bitmap::getBytesPerPixel(): Unknown format.");
+            AVG_TRACE(Logger::ERROR, "Bitmap::getBytesPerPixel(): Unknown format " << 
+                    getPixelFormatString(PF) << ".");
+            assert(false);
             return 0;
     }
 }
@@ -600,6 +963,26 @@ void Bitmap::getMinMax(int Stride, int& min, int& max) const
     }
 }
 
+void Bitmap::setAlpha(const Bitmap& alphaBmp)
+{
+    assert(hasAlpha());
+    assert(alphaBmp.getPixelFormat() == I8);
+    unsigned char * pLine = m_pBits;
+    const unsigned char * pAlphaLine = alphaBmp.getPixels();
+    for (int y=0; y < m_Size.y; y++) {
+        unsigned char * pPixel = pLine;
+        const unsigned char * pAlphaPixel = pAlphaLine;
+        for (int x=0; x<m_Size.x; x++) {
+            pPixel[ALPHAPOS] = *pAlphaPixel;
+            pPixel+=4;
+            pAlphaPixel++;
+        }
+        pLine += m_Stride;
+        pAlphaLine += alphaBmp.getStride();
+    }
+
+}
+
 bool Bitmap::operator ==(const Bitmap & otherBmp)
 {
     // We allow Name, Stride and bOwnsBits to be different here, since we're looking for
@@ -637,11 +1020,22 @@ bool Bitmap::operator ==(const Bitmap & otherBmp)
 
 Bitmap * Bitmap::subtract(const Bitmap *pOtherBmp)
 {
-    assert(m_PF == pOtherBmp->getPixelFormat() && m_Size == pOtherBmp->getSize());
+    if (m_PF != pOtherBmp->getPixelFormat())
+        throw Exception(AVG_ERR_UNSUPPORTED, 
+                string("Bitmap::subtract: pixel formats differ(")
+                + getPixelFormatString(m_PF)+", "
+                + getPixelFormatString(pOtherBmp->getPixelFormat())+")");
+    if (m_Size != pOtherBmp->getSize())
+        throw Exception(AVG_ERR_UNSUPPORTED, 
+                string("Bitmap::subtract: bitmap sizes differ (this=")
+                + toString(m_Size) + ", other=" + toString(pOtherBmp->getSize()) + ")");
     Bitmap * pResultBmp = new Bitmap(m_Size, m_PF);
     const unsigned char * pSrcLine1 = pOtherBmp->getPixels();
     const unsigned char * pSrcLine2 = m_pBits;
     unsigned char * pDestLine = pResultBmp->getPixels();
+    int stride = getStride();
+    int lineLen = getLineLen();
+
     for (int y=0; y<getSize().y; ++y) {
         switch(m_PF) {
             case I16: 
@@ -662,7 +1056,7 @@ Bitmap * Bitmap::subtract(const Bitmap *pOtherBmp)
                     const unsigned char * pSrc1 = pSrcLine1;
                     const unsigned char * pSrc2 = pSrcLine2;
                     unsigned char * pDest= pDestLine;
-                    for (int x=0; x<getLineLen(); ++x) {
+                    for (int x=0; x<lineLen; ++x) {
                         *pDest = abs(*pSrc1-*pSrc2);
                         pSrc1++;
                         pSrc2++;
@@ -670,12 +1064,36 @@ Bitmap * Bitmap::subtract(const Bitmap *pOtherBmp)
                     }
                 }
         }
-        pSrcLine1 += getStride();
-        pSrcLine2 += pOtherBmp->getStride();
-        pDestLine += pResultBmp->getStride();
+        pSrcLine1 += stride;
+        pSrcLine2 += stride;
+        pDestLine += stride;
     }
     return pResultBmp;
 }
+    
+void Bitmap::blt(const Bitmap* pOtherBmp, const IntPoint& pos)
+{
+    assert(getBytesPerPixel() == 4);
+    assert(pOtherBmp->getPixelFormat() == B8G8R8A8 || 
+            pOtherBmp->getPixelFormat() == R8G8B8A8);
+
+    IntRect destRect(pos.x, pos.y, pos.x+pOtherBmp->getSize().x, 
+            pos.y+pOtherBmp->getSize().y);
+    destRect.intersect(IntRect(IntPoint(0,0), getSize()));
+    for (int y = 0; y < destRect.height(); y++) {
+        unsigned char * pSrcPixel = getPixels()+(pos.y+y)*getStride()+pos.x*4;
+        const unsigned char * pOtherPixel = pOtherBmp->getPixels()+
+                y*pOtherBmp->getStride(); 
+        for (int x = 0; x < destRect.width(); x++) {
+            int srcAlpha = 255-pOtherPixel[3];
+            pSrcPixel[0] = (srcAlpha*pSrcPixel[0]+int(pOtherPixel[3])*pOtherPixel[0])/255;
+            pSrcPixel[1] = (srcAlpha*pSrcPixel[1]+int(pOtherPixel[3])*pOtherPixel[1])/255;
+            pSrcPixel[2] = (srcAlpha*pSrcPixel[2]+int(pOtherPixel[3])*pOtherPixel[2])/255;
+            pSrcPixel += 4;
+            pOtherPixel += 4;
+        }
+    }
+}
 
 double Bitmap::getAvg() const
 {
@@ -782,12 +1200,21 @@ void Bitmap::dump(bool bDumpPixels) const
             unsigned char * pLine = m_pBits+m_Stride*y;
             cerr << "    ";
             for (int x=0; x<m_Size.x; ++x) {
-                unsigned char * pPixel = pLine+getBytesPerPixel()*x;
-                cerr << "[";
-                for (int i=0; i<getBytesPerPixel(); ++i) {
-                    cerr << hex << setw(2) << (int)(pPixel[i]) << " ";
+                if (m_PF == R32G32B32A32F) {
+                    float * pPixel = (float*)(pLine+getBytesPerPixel()*x);
+                    cerr << "[";
+                    for (int i=0; i<4; ++i) {
+                        cerr << setw(4) << setprecision(2) << pPixel[i] << " ";
+                    }
+                    cerr << "]";
+                } else {
+                    unsigned char * pPixel = pLine+getBytesPerPixel()*x;
+                    cerr << "[";
+                    for (int i=0; i<getBytesPerPixel(); ++i) {
+                        cerr << hex << setw(2) << (int)(pPixel[i]) << " ";
+                    }
+                    cerr << "]";
                 }
-                cerr << "]";
             }
             cerr << endl;
         }
@@ -851,25 +1278,6 @@ void Bitmap::allocBits()
     }
 }
 
-inline void YUVtoBGR32Pixel(Pixel32* pDest, int y, int u, int v)
-{
-    // u = Cb, v = Cr
-    int u1 = u - 128;
-    int v1 = v - 128;
-    int tempy = 298*(y-16);
-    int b = (tempy + 516 * u1           ) >> 8;
-    int g = (tempy - 100 * u1 - 208 * v1) >> 8;
-    int r = (tempy            + 409 * v1) >> 8;
-
-    if (b<0) b = 0;
-    if (b>255) b= 255;
-    if (g<0) g = 0;
-    if (g>255) g= 255;
-    if (r<0) r = 0;
-    if (r>255) r= 255;
-    pDest->set(b,g,r,255);
-}
-
 void YUYV422toBGR32Line(const unsigned char* pSrcLine, Pixel32 * pDestLine, int Width)
 {
     Pixel32 * pDestPixel = pDestLine;
@@ -984,7 +1392,6 @@ void Bitmap::YCbCrtoBGR(const Bitmap& Orig)
     int Height = min(Orig.getSize().y, m_Size.y);
     int Width = min(Orig.getSize().x, m_Size.x);
     int StrideInPixels = m_Stride/getBytesPerPixel();
-    int TotalPixels = Width * Height;
     switch(Orig.m_PF) {
         case YCbCr422:
             for (int y=0; y<Height; ++y) {
@@ -1007,22 +1414,6 @@ void Bitmap::YCbCrtoBGR(const Bitmap& Orig)
                 pSrc += Orig.getStride();
             }
             break;
-        case YCbCr420p: // this is a planar format
-            for (int y=0; y<Height; ++y) {
-                // Fast conversion algo with no interpolation
-                // TODO: interlacing aware interpolation
-                int lineoffs = Width * y;
-                int emioffs = (y/2) * (Width/2);
-                for (int x=0; x<Width; ++x) {
-                    // planes order: Y(1:1), Cb(1:4), Cr(1:4).
-                    // Offsets: Cb @ imgSize, Cr @ 5/4 * imgSize
-                    YUVtoBGR32Pixel(pDest + lineoffs + x,
-                        pSrc[lineoffs + x],
-                        pSrc[emioffs + x/2 + TotalPixels],
-                        pSrc[emioffs + x/2 + TotalPixels + TotalPixels/4]);
-                }
-            }
-            break;
         default:
             // This routine shouldn't be called with other pixel formats.
             assert(false);
@@ -1177,7 +1568,49 @@ void Bitmap::I8toRGB(const Bitmap& Orig)
     }
 }
 
+void Bitmap::ByteRBBAtoFloatRGBA(const Bitmap& Orig)
+{
+    assert(getPixelFormat() == R32G32B32A32F);
+    assert(Orig.getBytesPerPixel() == 4);
+    const unsigned char * pSrc = Orig.getPixels();
+    int Height = min(Orig.getSize().y, m_Size.y);
+    int Width = min(Orig.getSize().x, m_Size.x);
+    float * pDest = (float *)m_pBits;
+    for (int y=0; y<Height; ++y) {
+        const unsigned char * pSrcPixel = pSrc;
+        float * pDestPixel = pDest;
+        for (int x=0; x<Width*4; ++x) {
+            *pDestPixel = float(*pSrcPixel)/255;
+            pDestPixel ++;
+            pSrcPixel++;
+        }
+        pDest += m_Stride/sizeof(float);
+        pSrc += Orig.getStride();
+    }
+}
 
+void Bitmap::FloatRGBAtoByteRGBA(const Bitmap& Orig)
+{
+    assert(getBytesPerPixel() == 4);
+    assert(Orig.getPixelFormat() == R32G32B32A32F);
+    const float * pSrc = (const float *)Orig.getPixels();
+    int Height = min(Orig.getSize().y, m_Size.y);
+    int Width = min(Orig.getSize().x, m_Size.x);
+    unsigned char * pDest = m_pBits;
+    for (int y=0; y<Height; ++y) {
+        const float * pSrcPixel = pSrc;
+        unsigned char * pDestPixel = pDest;
+        for (int x=0; x<Width*4; ++x) {
+            *pDestPixel = (unsigned char)(*pSrcPixel*255+0.5);
+            pDestPixel++;
+            pSrcPixel++;
+        }
+        pDest += m_Stride;
+        pSrc += Orig.getStride()/sizeof(float);
+    }
+}
+
+/*
 // Nearest Neighbour Bayer Pattern de-mosaicking
 // Code has been taken and adapted from libdc1394 Bayer conversion
 // TODO: adapt it for RGB24, not just for RGB32
@@ -1271,16 +1704,16 @@ void Bitmap::BY8toRGBNearest(const Bitmap& Orig)
         pDestPixel += DestStride;
     }
 }
+*/
 
 // Bilinear Bayer Pattern de-mosaicking
 // Code has been taken and adapted from libdc1394 Bayer conversion
 // Original source is OpenCV Bayer pattern decoding
 // TODO: adapt it for RGB24, not just for RGB32
-// TODO: add more CFA patterns (now only the GBRG is defined and used)
 void Bitmap::BY8toRGBBilinear(const Bitmap& Orig)
 {
     assert(getBytesPerPixel() == 4);
-    assert(Orig.getPixelFormat() == BAYER8_GBRG);
+    assert(pixelFormatIsBayer(Orig.getPixelFormat()));
 
     int Height = min(Orig.getSize().y, m_Size.y);
     int Width = min(Orig.getSize().x, m_Size.x);
@@ -1291,10 +1724,21 @@ void Bitmap::BY8toRGBBilinear(const Bitmap& Orig)
     int width = Width;
     int height = Height;
 
-    // CFA Pattern selection: BGGR: blue=-1, greenFirst=0; GRBG: blue=1, greenFirst=1
-    // Assuming GBRG
-    int blue = 1;
-    int greenFirst = 1;
+    // CFA Pattern selection
+    PixelFormat pf = Orig.getPixelFormat();
+    int blue;
+    int greenFirst;
+    if (pf == BAYER8_BGGR || pf == BAYER8_GBRG) {
+        blue = -1;
+    } else {
+        blue = 1;
+    }
+
+    if (pf == BAYER8_GBRG || pf == BAYER8_GRBG) {
+        greenFirst = 1;
+    } else {
+        greenFirst = 0;
+    }
 
     const unsigned char *pSrcPixel = Orig.getPixels();
     unsigned char *pDestPixel = (unsigned char *) getPixels();
@@ -1459,7 +1903,10 @@ void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src)
             createTrueColorCopy<Pixel, Pixel16>(Dest, Src);
             break;
         case I8:
+        case BAYER8_RGGB:
         case BAYER8_GBRG:
+        case BAYER8_GRBG:
+        case BAYER8_BGGR:
             createTrueColorCopy<Pixel, Pixel8>(Dest, Src);
             break;
         default:
@@ -1467,5 +1914,5 @@ void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src)
             assert(false);
     }
 }
-    
+
 };
