@@ -22,6 +22,7 @@
 #ifndef _Filter_H_
 #define _Filter_H_
 
+#include "../api.h"
 #include "Bitmap.h"
 
 #include <boost/shared_ptr.hpp>
@@ -29,27 +30,26 @@
 namespace avg {
 
 // Base class for filters that operate on bitmaps. Derived classes need
-// to override either the ApplyInPlace or the Apply function. The base-class
+// to override either the applyInPlace or the apply function. The base-class
 // versions of these functions simply implement one function in terms of the
 // other.
-class Filter
+class AVG_API Filter
 {
 public:
-  Filter();
-  virtual ~Filter();
+    Filter();
+    virtual ~Filter();
 
-  // In-Place Apply. Applies the filter to pBmp. The base-class
-  // version copies the bitmap after calling Apply (pBmp, pTempBmp).
-  virtual void applyInPlace(BitmapPtr pBmp);
+    // In-Place Apply. Applies the filter to pBmp. The base-class
+    // version copies the bitmap after calling Apply (pBmp, pTempBmp).
+    virtual void applyInPlace(BitmapPtr pBmp);
 
-  // Applies the Filter to pBmpSource and returns the result
-  // The base-class version copies the bitmap before calling
-  // applyInPlace.
-  virtual BitmapPtr apply(BitmapPtr pBmpSource);
+    // Applies the Filter to pBmpSource and returns the result
+    // The base-class version copies the bitmap before calling
+    // applyInPlace.
+    virtual BitmapPtr apply(BitmapPtr pBmpSource);
 };
 
 typedef boost::shared_ptr<Filter> FilterPtr;
 
-} // namespace
+}
 #endif
-
