@@ -22,6 +22,7 @@
 #ifndef _Filtercolorize_H_
 #define _Filtercolorize_H_
 
+#include "../api.h"
 #include "Filter.h"
 
 namespace avg {
@@ -29,7 +30,7 @@ namespace avg {
 // Filter that colorizes a bitmap given a hue and saturation. Corresponds
 // loosely to the photoshop hue/saturation control when set to 'colorize'.
 // The range of hue is 0..359, the range of saturation is 0..100.
-class FilterColorize : public Filter
+class AVG_API FilterColorize : public Filter
 {
 public:
   FilterColorize(double Hue, double Saturation);
