@@ -22,13 +22,16 @@
 #ifndef _Pixel8_H_
 #define _Pixel8_H_
 
+#include "../api.h"
 #include "Pixeldefs.h"
 #include "Pixel32.h"
 
 #include <stdlib.h>
 #include <assert.h>
 
-class Pixel8
+namespace avg {
+
+class AVG_API Pixel8
 {
   public:
     Pixel8 ();
@@ -147,4 +150,5 @@ inline void Pixel8::operator -= (const Pixel8& Pix)
   m_i -= Pix.m_i;
 }
 
+}
 #endif
