@@ -40,6 +40,11 @@
 #include "FilterBandpass.h"
 #include "FilterFastDownscale.h"
 #include "FilterMask.h"
+#include "FilterFloodfill.h"
+#include "FilterDilation.h"
+#include "FilterErosion.h"
+#include "FilterGetAlpha.h"
+#include "FilterResizeBilinear.h"
 
 #include "../base/TestSuite.h"
 #include "../base/Exception.h"
@@ -63,13 +68,15 @@ using namespace std;
 
 BitmapPtr initBmp(PixelFormat PF)
 {
-    BitmapPtr pBmp;
+    int height;
     if (PF == YCbCr422) {
-        pBmp = BitmapPtr(new Bitmap(IntPoint(4,10), PF));
+        height = 10;
     } else {
-        pBmp = BitmapPtr(new Bitmap(IntPoint(4,7), PF));
+        height = 7;
+
     }
-    for(int y=0; y<7; ++y) {
+    BitmapPtr pBmp(new Bitmap(IntPoint(4,height), PF));
+    for(int y=0; y<height; ++y) {
         for (int x=0; x<4; ++x) {
             unsigned char * pPixel = 
                 pBmp->getPixels()+y*pBmp->getStride()+x*pBmp->getBytesPerPixel();
@@ -110,6 +117,9 @@ public:
 
         cerr << "    Testing OwnsBits." << endl;
         unsigned char pData[4*7*3];
+        for (int i=0; i<4*7*3; ++i) {
+            pData[i] = i;
+        }
         Bitmap Bmp1 = Bitmap(IntPoint(4,7), R8G8B8, pData, 12, true, "");
         Bitmap Bmp2 = Bitmap(IntPoint(4,7), R8G8B8, pData, 12, false, "");
         testEqual(Bmp1, Bmp2, "BmpOwnsBits");
@@ -152,6 +162,15 @@ public:
             pBmp->copyPixels(*pCopyBmp);
             testEqual(*pBmp, *pBaselineBmp, "BmpCopyPixels_3");
         }
+        {
+            cerr << "    Testing copyPixels - R8G8B8A8->R32G32B32A32F->R8G8B8A8." << endl;
+            BitmapPtr pBmp = initBmp(R8G8B8A8);
+            BitmapPtr pBaselineBmp = initBmp(R8G8B8A8);
+            BitmapPtr pCopyBmp = BitmapPtr(new Bitmap(IntPoint(4,7), R32G32B32A32F));
+            pCopyBmp->copyPixels(*pBmp);
+            pBmp->copyPixels(*pCopyBmp);
+            testEqual(*pBmp, *pBaselineBmp, "BmpCopyPixels_3");
+        }
         {
             cerr << "    Testing statistics." << endl;
             cerr << "      I8" << endl;
@@ -167,6 +186,10 @@ public:
                     Pixel24(2,2,2), Pixel24(2,2,2));
 
         }
+        {
+            cerr << "    Testing YUV->RGB conversion." << endl;
+            testYUV2RGB();
+        }
         runSaveTest(R8G8B8A8);
         runSaveTest(R8G8B8X8);
     }
@@ -185,7 +208,8 @@ private:
         TEST(pBmp->getStdDev() == 1);
     }
 
-    void runPFTests(PixelFormat PF) {
+    void runPFTests(PixelFormat PF)
+    {
         cerr << "    Testing " << Bitmap::getPixelFormatString(PF) << endl;
         BitmapPtr pBmp = initBmp(PF);
         {
@@ -228,7 +252,8 @@ private:
         }
     }
 
-    void runSaveTest(PixelFormat PF) {
+    void runSaveTest(PixelFormat PF)
+    {
         cerr << "    Testing save for " << Bitmap::getPixelFormatString(PF) << endl;
         BitmapPtr pBmp = initBmp(PF);
         pBmp->save("test.tif");
@@ -238,7 +263,8 @@ private:
     }
 
     template<class Pixel>
-    void runLineTest(PixelFormat PF, Pixel Color) {
+    void runLineTest(PixelFormat PF, Pixel Color)
+    {
         cerr << "    Testing line drawing for " << Bitmap::getPixelFormatString(PF) << endl;
         Bitmap Bmp(IntPoint(15, 15), PF);
         memset(Bmp.getPixels(), 0, Bmp.getStride()*15);
@@ -251,12 +277,32 @@ private:
         Bmp.drawLine(IntPoint(7,7), IntPoint(14, 2), Color);
         Bmp.drawLine(IntPoint(7,7), IntPoint(14,12), Color);
         string sFName = getSrcDirName()+"baseline/LineResult"+Bitmap::getPixelFormatString(PF)+".png";
-//        Bmp.save(sFName);
         Bitmap BaselineBmp(sFName);
         Bitmap BaselineBmp2(IntPoint(15,15), PF);
         BaselineBmp2.copyPixels(BaselineBmp);
         testEqual(Bmp, BaselineBmp2, "BmpLineDraw");
     }
+
+    void testYUV2RGB()
+    {
+        BitmapPtr pYBmp = BitmapPtr(new Bitmap(IntPoint(16, 16), I8));
+        for (int x=0; x<16; ++x) {
+            FilterFillRect<Pixel8>(IntRect(x, 0, x+1, 16), 255-x*16).applyInPlace(pYBmp);
+        }
+        BitmapPtr pUBmp = BitmapPtr(new Bitmap(IntPoint(8, 8), I8));
+        BitmapPtr pVBmp = BitmapPtr(new Bitmap(IntPoint(8, 8), I8));
+        FilterFillRect<Pixel8>(IntRect(0, 0, 8, 4), 128).applyInPlace(pUBmp);
+        FilterFillRect<Pixel8>(IntRect(0, 0, 8, 4), 128).applyInPlace(pVBmp);
+        for (int y=4; y<8; ++y) {
+            FilterFillRect<Pixel8>(IntRect(0, y, 8, y+1), y*64).applyInPlace(pUBmp);
+            FilterFillRect<Pixel8>(IntRect(0, y, 8, y+1), 255-y*64).applyInPlace(pVBmp);
+        }
+        BitmapPtr pRGBBmp = BitmapPtr(new Bitmap(IntPoint(16, 16), B8G8R8X8));
+        FilterFill<Pixel32>(Pixel32(255,0,0,255)).applyInPlace(pRGBBmp);
+        pRGBBmp->copyYUVPixels(*pYBmp, *pUBmp, *pVBmp);
+        testEqual(*pRGBBmp, "YUV2RGBResult1", B8G8R8X8, 0.5, 0.5);
+    }
+
 };
 
 class FilterColorizeTest: public GraphicsTest {
@@ -733,6 +779,98 @@ private:
     }
 };
 
+class FilterFloodfillTest: public GraphicsTest {
+public:
+    FilterFloodfillTest()
+        : GraphicsTest("FilterFloodfillTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        BitmapPtr pBmp = loadTestBmp("floodfill");
+        BitmapPtr pDestBmp = FilterFloodfill<ColorTester>(
+                ColorTester(Pixel32(255,255,255,255)), IntPoint(4,3)).apply(pBmp);
+        testEqual(*pDestBmp, "FloodfillResult", B8G8R8A8, 0, 0);
+    }
+
+};
+
+class FilterDilationTest: public GraphicsTest {
+public:
+    FilterDilationTest()
+        : GraphicsTest("FilterDilationTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        BitmapPtr pBmp = loadTestBmp("dilation", I8);
+        BitmapPtr pDestBmp = FilterDilation().apply(pBmp);
+        testEqual(*pDestBmp, "DilationResult", I8, 0, 0);
+    }
+
+};
+
+class FilterErosionTest: public GraphicsTest {
+public:
+    FilterErosionTest()
+        : GraphicsTest("FilterErosionTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        BitmapPtr pBmp = loadTestBmp("erosion", I8);
+        BitmapPtr pDestBmp = FilterErosion().apply(pBmp);
+        testEqual(*pDestBmp, "ErosionResult", I8, 0, 0);
+    }
+
+};
+
+class FilterAlphaTest: public GraphicsTest {
+public:
+    FilterAlphaTest()
+        : GraphicsTest("FilterAlphaTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        BitmapPtr pBmp = loadTestBmp("rgb24alpha-64x64", R8G8B8A8);
+        BitmapPtr pAlphaBmp = FilterGetAlpha().apply(pBmp);
+        testEqual(*pAlphaBmp, "GetAlphaResult", I8, 0, 0);
+        BitmapPtr pDestBmp(new Bitmap(*pBmp));
+        pDestBmp->setAlpha(*pAlphaBmp);
+        testEqual(*pDestBmp, *pBmp, "SetAlphaResult", 0, 0);
+    }
+
+};
+
+class FilterResizeBilinearTest: public GraphicsTest {
+public:
+    FilterResizeBilinearTest()
+        : GraphicsTest("FilterResizeBilinearTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        BitmapPtr pBmp = loadTestBmp("rgb24alpha-64x64", B8G8R8A8);
+        runTestWithBitmap(pBmp);
+        pBmp = loadTestBmp("rgb24-64x64", B8G8R8X8);
+        runTestWithBitmap(pBmp);
+    }
+
+private:
+    void runTestWithBitmap(BitmapPtr pBmp)
+    {
+        BitmapPtr pDestBmp = FilterResizeBilinear(IntPoint(32,32)).apply(pBmp);
+        string sName = string("ResizeBilinearResult")+pBmp->getPixelFormatString();
+        testEqual(*pDestBmp, sName, pBmp->getPixelFormat());
+    }
+
+};
 
 class GraphicsTestSuite: public TestSuite {
 public:
@@ -757,12 +895,18 @@ public:
         addTest(TestPtr(new FilterFastBandpassTest));
         addTest(TestPtr(new FilterFastDownscaleTest));
         addTest(TestPtr(new FilterMaskTest));
+        addTest(TestPtr(new FilterFloodfillTest));
+        addTest(TestPtr(new FilterDilationTest));
+        addTest(TestPtr(new FilterErosionTest));
+        addTest(TestPtr(new FilterAlphaTest));
+        addTest(TestPtr(new FilterResizeBilinearTest));
     }
 };
 
 
 int main(int nargs, char** args)
 {
+    GraphicsTest::createResultImgDir();
     GraphicsTestSuite Suite;
     Suite.runTests();
     bool bOK = Suite.isOk();
