@@ -56,7 +56,6 @@ DeDistort::DeDistort()
       m_DisplayOffset(0,0),
       m_DisplayScale(1,1)
 {
-    m_DistortionParams.push_back(0);
     m_DistortionParams.push_back(0);
     m_DistortionParams.push_back(0);
     m_RescaleFactor = calc_rescale();
@@ -70,7 +69,6 @@ DeDistort::DeDistort(const DPoint& CamExtents, const DPoint& DisplayExtents)
     m_CamExtents = CamExtents; 
     m_DistortionParams.push_back(0);
     m_DistortionParams.push_back(0);
-    m_DistortionParams.push_back(0);
     m_DisplayScale.x = DisplayExtents.x/CamExtents.x;
     m_DisplayScale.y = DisplayExtents.y/CamExtents.y;
     m_RescaleFactor = calc_rescale();
@@ -120,8 +118,6 @@ void DeDistort::load(const DPoint &CameraExtents, const TrackerConfig& Config)
             ("/transform/distortionparams/@p2"));
     m_DistortionParams.push_back(Config.getDoubleParam
             ("/transform/distortionparams/@p3"));
-    m_DistortionParams.push_back(Config.getDoubleParam
-            ("/transform/distortionparams/@p4"));
     m_TrapezoidFactor = Config.getDoubleParam("/transform/trapezoid/@value");
     m_Angle = Config.getDoubleParam("/transform/angle/@value");
     m_DisplayOffset = Config.getPointParam("/transform/displaydisplacement/");
@@ -136,8 +132,6 @@ void DeDistort::save(TrackerConfig& Config)
             toString(m_DistortionParams[0]));
     Config.setParam("/transform/distortionparams/@p3", 
             toString(m_DistortionParams[1]));
-    Config.setParam("/transform/distortionparams/@p4", 
-            toString(m_DistortionParams[2]));
     Config.setParam("/transform/trapezoid/@value", 
             toString(m_TrapezoidFactor));
     Config.setParam("/transform/angle/@value", 
@@ -188,7 +182,7 @@ DPoint DeDistort::inverse_transform_point(const DPoint &pt)
     DPoint DestPt = pt-m_CamExtents/2;
     DestPt = DPoint(2*DestPt.x/m_CamExtents.x, 2*DestPt.y/m_CamExtents.y);
     DestPt = inv_trapezoid(m_TrapezoidFactor, DestPt);
-    DestPt = rotate(DestPt, -m_Angle);
+    DestPt = DestPt.getRotated(-m_Angle);
     DestPt *= m_RescaleFactor;
     DestPt = inverse_undistort(m_DistortionParams, DestPt);
     DestPt = DPoint(DestPt.x*m_CamExtents.x/2, DestPt.y*m_CamExtents.y/2);
@@ -209,7 +203,7 @@ DPoint DeDistort::transform_point(const DPoint &pt)
     DestPt = DPoint(2*DestPt.x/m_CamExtents.x, 2*DestPt.y/m_CamExtents.y);
     DestPt = undistort(m_DistortionParams, DestPt);
     DestPt /= m_RescaleFactor;
-    DestPt = rotate(DestPt, m_Angle);
+    DestPt = DestPt.getRotated(m_Angle);
     DestPt = trapezoid(m_TrapezoidFactor, DestPt);
     DestPt = DPoint(DestPt.x*m_CamExtents.x/2, DestPt.y*m_CamExtents.y/2);
     DestPt += m_CamExtents/2;
