@@ -33,16 +33,16 @@
 #include "../base/Queue.h"
 #include "../base/WorkerThread.h"
 
-#ifdef AVG_ENABLE_1394
-#include <libraw1394/raw1394.h>
-#include <libdc1394/dc1394_control.h>
-#endif
 #ifdef AVG_ENABLE_1394_2
 #include <dc1394/control.h>
+#include <dc1394/register.h>
 #endif
 #ifndef AVG_ENABLE_1394_2
 typedef unsigned int dc1394feature_t;
 #endif
+#ifndef uint64_t
+#define uint64_t unsigned long long
+#endif
 
 #include <string>
 #include <map>
@@ -51,53 +51,49 @@ namespace avg {
 
 typedef Queue<BitmapPtr> BitmapQueue;
 
-class FWCamera: public Camera {
+class AVG_API FWCamera: public Camera {
 public:
-    FWCamera(std::string sDevice, IntPoint Size, std::string sPF,
-            double FrameRate, bool bColor);
+    FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint Size, PixelFormat camPF, 
+            PixelFormat destPF, double FrameRate);
     virtual ~FWCamera();
-    virtual void open();
-    virtual void close();
 
     virtual IntPoint getImgSize();
     virtual BitmapPtr getImage(bool bWait);
-    virtual bool isCameraAvailable();
 
     virtual const std::string& getDevice() const; 
     virtual const std::string& getDriverName() const; 
     virtual double getFrameRate() const;
 
-    virtual unsigned int getFeature(CameraFeature Feature) const;
-    virtual void setFeature(CameraFeature Feature, int Value);
-    void setFeature(dc1394feature_t Feature, int Value);
+    virtual int getFeature(CameraFeature Feature) const;
+    virtual void setFeature(CameraFeature Feature, int Value, bool bIgnoreOldValue=false);
+    virtual void setFeatureOneShot(CameraFeature Feature);
+    virtual int getWhitebalanceU() const;
+    virtual int getWhitebalanceV() const;
+    virtual void setWhitebalance(int u, int v, bool bIgnoreOldValue=false);
+
+    static void dumpCameras();
+    static void resetBus();
 
 private:
-    std::string m_sDevice;
-    std::string m_sPF;
+    void setFeature(dc1394feature_t Feature, int Value);
+    void setStrobeDuration(int microsecs);
+    void getWhitebalance(int* pU, int* pV) const;
+    void enablePtGreyBayer();
+
     IntPoint m_Size;
     double m_FrameRate;
-    bool m_bColor;
 
-#ifdef AVG_ENABLE_1394
-    bool findCameraOnPort(int port, raw1394handle_t& FWHandle);
-
-    dc1394_cameracapture m_Camera;
-    raw1394handle_t m_FWHandle;
-    int m_FrameRateConstant;  // libdc1394 constant for framerate.
-    int m_Mode;               // libdc1394 constant for mode.
-#elif AVG_ENABLE_1394_2
+#ifdef AVG_ENABLE_1394_2
     dc1394_t * m_pDC1394;
     dc1394camera_t * m_pCamera;
     dc1394framerate_t m_FrameRateConstant; 
     dc1394video_mode_t m_Mode;            
 #endif
-    void checkDC1394Error(int Code, const std::string & sMsg);
-    void fatalError(const std::string & sMsg);
     void dumpCameraInfo();
 
-    bool m_bCameraAvailable;
-    typedef std::map<dc1394feature_t, int> FeatureMap;
     FeatureMap m_Features;
+    int m_WhitebalanceU;
+    int m_WhitebalanceV;
 };
 
 }
