@@ -37,11 +37,20 @@ using namespace std;
 
 namespace avg {
 
-
-FakeCamera::FakeCamera(std::vector<std::string> &pictures)
-    : m_pBmpQ(new std::queue<BitmapPtr>()),
+FakeCamera::FakeCamera(PixelFormat camPF, PixelFormat destPF)
+    : Camera(camPF, destPF),
+      m_pBmpQ(new std::queue<BitmapPtr>()),
       m_bIsOpen(false)
 {
+    m_ImgSize = IntPoint(640, 480);
+}
+
+FakeCamera::FakeCamera(std::vector<std::string> &pictures)
+    : Camera(I8, I8),
+      m_pBmpQ(new std::queue<BitmapPtr>()),
+      m_bIsOpen(false)
+{
+    m_ImgSize = IntPoint(640, 480);
     for(std::vector<std::string>::iterator it=pictures.begin();it!=pictures.end();++it){
         try {
             BitmapPtr pBmp (new Bitmap(*it));
@@ -80,10 +89,10 @@ IntPoint FakeCamera::getImgSize()
 
 BitmapPtr FakeCamera::getImage(bool bWait)
 {
-    if (!m_bIsOpen || !bWait) {
+    msleep(100);
+    if (!m_bIsOpen || !bWait || m_pBmpQ->empty()) {
         return BitmapPtr();
     } else {
-        msleep(100);
         BitmapPtr pBmp = m_pBmpQ->front();
         m_pBmpQ->pop(); 
         return pBmp; 
@@ -120,12 +129,30 @@ const string& FakeCamera::getMode() const
 }
 
 
-unsigned int FakeCamera::getFeature(CameraFeature Feature) const
+int FakeCamera::getFeature(CameraFeature Feature) const
 {
     return 0;
 }
 
-void FakeCamera::setFeature(CameraFeature Feature, int Value)
+void FakeCamera::setFeature(CameraFeature Feature, int Value, bool bIgnoreOldValue)
+{
+}
+
+void FakeCamera::setFeatureOneShot(CameraFeature Feature)
+{
+}
+
+int FakeCamera::getWhitebalanceU() const
+{
+    return 0;
+}
+
+int FakeCamera::getWhitebalanceV() const
+{
+    return 0;
+}
+
+void FakeCamera::setWhitebalance(int u, int v, bool bIgnoreOldValue)
 {
 }
 
