@@ -21,6 +21,7 @@
 #ifndef _TrackerConfig_H_
 #define _TrackerConfig_H_
 
+#include "../api.h"
 #include "../base/Rect.h"
 
 #include <boost/shared_ptr.hpp>
@@ -33,15 +34,15 @@ namespace avg {
 class DeDistort;
 typedef boost::shared_ptr<DeDistort> DeDistortPtr;
 
-class TrackerConfig
+class AVG_API TrackerConfig
 {
 public:
     TrackerConfig();
     TrackerConfig(const TrackerConfig& Other);
     virtual ~TrackerConfig();
     
-    void load(const std::string& sFilename);
-    void save(const std::string& sFilename);
+    void load();
+    void save();
     void setParam(const std::string& sXPathExpr, const std::string& sValue);
     std::string getParam(const std::string& sXPathExpr) const;
     bool getBoolParam(const std::string& sXPathExpr) const;
@@ -62,6 +63,7 @@ private:
     xmlNodePtr m_pRoot;
 
     std::string m_sFilename;
+    void loadConfigFile(const std::string& sFilename);
 };
 typedef boost::shared_ptr<TrackerConfig> TrackerConfigPtr;
 
