@@ -20,6 +20,7 @@
 
 #include "TrackerThread.h"
 #include "FilterDistortion.h"
+#include "FilterWipeBorder.h"
 
 #include "../base/Logger.h"
 #include "../base/ProfilingZone.h"
@@ -68,6 +69,9 @@ TrackerThread::TrackerThread(IntRect ROI,
     : WorkerThread<TrackerThread>("Tracker", CmdQ),
       m_TouchThreshold(0),
       m_TrackThreshold(0),
+      m_WipeBorder(0),
+      m_HistoryDelay(-1),
+      m_StartTime(0),
       m_pMutex(pMutex),
       m_pCamera(pCamera),
       m_pTarget(target),
@@ -110,6 +114,14 @@ bool TrackerThread::init()
         m_pImagingContext = 0;
         m_pBandpassFilter = FilterPtr(new FilterFastBandpass());
     }
+    try {
+        m_StartTime = TimeSource::get()->getCurrentMillisecs(); 
+        m_HistoryDelay = m_pConfig->getIntParam("/tracker/historydelay/@value");
+    } catch (Exception& e) {
+        AVG_TRACE(Logger::WARNING, e.GetStr());
+    }
+    
+    
 // Done in TrackerEventSource::ctor to work around Leopard/libdc1394 threading issue.
 //    m_pCamera->open();
     return true;
@@ -117,6 +129,13 @@ bool TrackerThread::init()
 
 bool TrackerThread::work()
 {
+    if ((m_HistoryDelay + m_StartTime) < TimeSource::get()->getCurrentMillisecs() 
+            && m_HistoryDelay != -1) 
+    {   
+        resetHistory();
+        m_HistoryDelay = -1;
+    }
+    
     BitmapPtr pCamBmp;
     {
         ScopeTimer Timer(ProfilingZoneCapture);
@@ -154,6 +173,7 @@ bool TrackerThread::work()
             pDistortedBmp = m_pDistorter->apply(pCamBmp);
         }
         BitmapPtr pCroppedBmp(new Bitmap(*pDistortedBmp, m_ROI));
+        FilterWipeBorder(m_WipeBorder).applyInPlace(pCroppedBmp);
         if (m_bCreateDebugImages) {
             boost::mutex::scoped_lock Lock(*m_pMutex);
             m_pBitmaps[TRACKER_IMG_DISTORTED]->copyPixels(*pCroppedBmp);
@@ -187,9 +207,12 @@ bool TrackerThread::work()
 
 void TrackerThread::deinit()
 {
-    m_pCamera->close();
+    m_pCamera = CameraPtr();
     AVG_TRACE(Logger::PROFILE, "Total camera frames: " << m_NumFrames);
     AVG_TRACE(Logger::PROFILE, "Camera frames discarded: " << m_NumCamFramesDiscarded);
+    if (m_pBandpassFilter) {
+        m_pBandpassFilter.reset();
+    }
     if (m_pImagingContext) {
         delete m_pImagingContext;
     }
@@ -215,6 +238,7 @@ void TrackerThread::setConfig(TrackerConfig Config, IntRect ROI,
         m_pHistoryPreProcessor->setInterval(Config.getIntParam
                 ("/tracker/historyupdateinterval/@value"));
     }
+    m_WipeBorder = Config.getIntParam("/tracker/wipeborder/@value");
     DeDistortPtr pDeDistort = Config.getTransform();
     if (!(*m_pTrafo == *pDeDistort)) {
         m_pDistorter = FilterDistortionPtr(new FilterDistortion(
@@ -222,28 +246,28 @@ void TrackerThread::setConfig(TrackerConfig Config, IntRect ROI,
         *m_pTrafo = *pDeDistort;
     }
     int Brightness = Config.getIntParam("/camera/brightness/@value");
-    int Exposure = Config.getIntParam("/camera/exposure/@value");
     int Gamma = Config.getIntParam("/camera/gamma/@value");
     int Gain = Config.getIntParam("/camera/gain/@value");
     int Shutter = Config.getIntParam("/camera/shutter/@value");
+    int StrobeDuration = Config.getIntParam("/camera/strobeduration/@value");
     string sCameraMaskFName = Config.getParam("/tracker/mask/@value");
     bool bNewCameraMask = (!m_pConfig || 
             m_pConfig->getParam("/tracker/mask/@value") != sCameraMaskFName);
     if (int(m_pCamera->getFeature(CAM_FEATURE_BRIGHTNESS)) != Brightness ||
-            int(m_pCamera->getFeature(CAM_FEATURE_EXPOSURE)) != Exposure ||
              int(m_pCamera->getFeature(CAM_FEATURE_GAMMA)) != Gamma ||
              int(m_pCamera->getFeature(CAM_FEATURE_GAIN)) != Gain ||
              int(m_pCamera->getFeature(CAM_FEATURE_SHUTTER)) != Shutter ||
+             int(m_pCamera->getFeature(CAM_FEATURE_STROBE_DURATION)) != StrobeDuration ||
              bNewCameraMask)
     {
         m_pHistoryPreProcessor->reset();
     }
 
     m_pCamera->setFeature(CAM_FEATURE_BRIGHTNESS, Brightness);
-    m_pCamera->setFeature(CAM_FEATURE_EXPOSURE, Exposure);
     m_pCamera->setFeature(CAM_FEATURE_GAMMA, Gamma);
     m_pCamera->setFeature(CAM_FEATURE_GAIN, Gain);
     m_pCamera->setFeature(CAM_FEATURE_SHUTTER, Shutter);
+    m_pCamera->setFeature(CAM_FEATURE_STROBE_DURATION, StrobeDuration, true);
 
     if (bNewCameraMask) {
         if (sCameraMaskFName == "") {
@@ -256,7 +280,7 @@ void TrackerThread::setConfig(TrackerConfig Config, IntRect ROI,
         }
     }
     m_pConfig = TrackerConfigPtr(new TrackerConfig(Config));
-
+        
     setBitmaps(ROI, ppBitmaps);
     createBandpassFilter();
 }
@@ -285,14 +309,16 @@ void TrackerThread::setBitmaps(IntRect ROI, BitmapPtr ppBitmaps[NUM_TRACKER_IMAG
 
 void TrackerThread::createBandpassFilter()
 {
-    double bandpassMin = m_pConfig->getDoubleParam("/tracker/touch/bandpass/@min");
-    double bandpassMax = m_pConfig->getDoubleParam("/tracker/touch/bandpass/@max");
-    double bandpassPostMult = 
-        m_pConfig->getDoubleParam("/tracker/touch/bandpasspostmult/@value");
-    if (m_pImagingContext) {
-        m_pImagingContext->setSize(m_ROI.size());
-        m_pBandpassFilter = FilterPtr(new GPUBandpassFilter(m_ROI.size(), I8,
-                bandpassMin, bandpassMax, bandpassPostMult, m_bTrackBrighter));
+    if (m_TouchThreshold != 0) {
+        double bandpassMin = m_pConfig->getDoubleParam("/tracker/touch/bandpass/@min");
+        double bandpassMax = m_pConfig->getDoubleParam("/tracker/touch/bandpass/@max");
+        double bandpassPostMult = 
+            m_pConfig->getDoubleParam("/tracker/touch/bandpasspostmult/@value");
+        if (m_pImagingContext) {
+            m_pImagingContext->setSize(m_ROI.size());
+            m_pBandpassFilter = FilterPtr(new GPUBandpassFilter(m_ROI.size(), I8,
+                        bandpassMin, bandpassMax, bandpassPostMult, m_bTrackBrighter));
+        }
     }
 }
 
