@@ -21,6 +21,7 @@
 #ifndef _TrackerThread_H_
 #define _TrackerThread_H_
 
+#include "../api.h"
 #include "TrackerConfig.h"
 #include "Camera.h"
 #include "Blob.h"
@@ -55,7 +56,7 @@ typedef enum {
 typedef boost::shared_ptr<boost::mutex> MutexPtr;
 class OGLImagingContext;
 
-class IBlobTarget {
+class AVG_API IBlobTarget {
     public:
         virtual ~IBlobTarget() {};
         // Note that this function is called by TrackerThread in it's own thread!
@@ -64,7 +65,7 @@ class IBlobTarget {
 };
 
 
-class TrackerThread: public WorkerThread<TrackerThread>
+class AVG_API TrackerThread: public WorkerThread<TrackerThread>
 {
     public:
         TrackerThread(IntRect ROI, 
@@ -110,7 +111,11 @@ class TrackerThread: public WorkerThread<TrackerThread>
         int m_TouchThreshold; // 0 => no touch events.
         int m_TrackThreshold; // 0 => no generic tracking events.
         int m_Prescale;
+        int m_WipeBorder;
+        long long m_HistoryDelay;
+        long long m_StartTime;
         bool m_bTrackBrighter;
+        
         BlobVectorPtr m_pBlobVector;
         IntRect m_ROI;
         BitmapPtr m_pBitmaps[NUM_TRACKER_IMAGES];
