@@ -18,13 +18,15 @@
 //
 //  Current versions can be found at www.libavg.de
 //
-// V4L2/libavg compliance by 02L > Outside Standing Level
+//  V4L2/libavg compliance by 02L > Outside Standing Level
 
 #include "V4LCamera.h"
 
 #include "../base/ScopeTimer.h"
 #include "../base/TimeSource.h"
 #include "../base/Logger.h"
+#include "../base/Exception.h"
+#include "../base/StringHelper.h"
 
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -61,19 +63,20 @@ namespace {
 }
 
 V4LCamera::V4LCamera(std::string sDevice, int Channel, IntPoint Size,
-        const std::string &PixelFormat, bool bColor)
-           : m_Fd(-1),
-           m_Channel(Channel),
-           m_sDevice(sDevice), 
-           m_bCameraAvailable(false),
-           m_bColor(bColor),
-           m_ImgSize(Size)
+        PixelFormat camPF, PixelFormat destPF)
+    : Camera(camPF, destPF),
+      m_Fd(-1),
+      m_Channel(Channel),
+      m_sDevice(sDevice), 
+      m_ImgSize(Size)
 {
-    AVG_TRACE(Logger::CONFIG, "V4LCamera() device=" << sDevice << 
-        " ch=" << Channel << " w=" << Size.x << " h=" << Size.y <<
-        " pf=" << PixelFormat);
-    
-    m_CamPF = getCamPF(PixelFormat);
+    m_v4lPF = getV4LPF(camPF);
+    if (m_sDevice == "") {
+        m_sDevice = "/dev/video0";
+    }
+    if (m_Channel == -1) {
+        m_Channel = 0;
+    }
 
     m_FeaturesNames[V4L2_CID_BRIGHTNESS] = "brightness";
     m_FeaturesNames[V4L2_CID_CONTRAST] = "contrast";
@@ -82,36 +85,20 @@ V4LCamera::V4LCamera(std::string sDevice, int Channel, IntPoint Size,
     m_FeaturesNames[V4L2_CID_WHITENESS] = "whiteness";
     m_FeaturesNames[V4L2_CID_GAMMA] = "gamma";
     m_FeaturesNames[V4L2_CID_SATURATION] = "saturation";
-}
-
-V4LCamera::~V4LCamera() 
-{
-    close();
-}
-
-void V4LCamera::open()
-{
+    
     struct stat St; 
-
-    if ( stat(m_sDevice.c_str(), &St) == -1) {
-        AVG_TRACE(Logger::ERROR, "Unable to access v4l2 device " << 
-          m_sDevice);
-        // TODO: Disable camera instead of exit(1).
-        exit(1);
+    if (stat(m_sDevice.c_str(), &St) == -1) {
+        fatalError(string("Unable to access v4l2 device '")+m_sDevice+"'." );
     }
 
     if (!S_ISCHR (St.st_mode)) {
-        AVG_TRACE(Logger::ERROR, m_sDevice + " is not a v4l2 device");
-        // TODO: Disable camera instead of exit(1).
-        exit(1);
+        fatalError(string("'")+m_sDevice+" is not a v4l2 device.");
     }
 
     m_Fd = ::open(m_sDevice.c_str(), O_RDWR /* required */ | O_NONBLOCK, 0);
 
     if (m_Fd == -1) {
-        AVG_TRACE(Logger::ERROR, "Unable to open v4l2 device " << m_sDevice);
-        // TODO: Disable camera instead of exit(1).
-        exit(1);
+        fatalError(string("Unable to open v4l2 device '") + m_sDevice + "'.");
     }
     
     initDevice();
@@ -119,29 +106,29 @@ void V4LCamera::open()
     AVG_TRACE(Logger::CONFIG, "V4L2 Camera opened");
 }
 
+V4LCamera::~V4LCamera() 
+{
+    close();
+}
+
+
 void V4LCamera::close()
 {
-    if (m_bCameraAvailable) {
-        enum v4l2_buf_type Type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-        if (-1 == xioctl (m_Fd, VIDIOC_STREAMOFF, &Type)) {
-            AVG_TRACE(Logger::ERROR, "VIDIOC_STREAMOFF");
-        }
-        vector<Buffer>::iterator it;
-        for (it=m_vBuffers.begin(); it != m_vBuffers.end(); ++it) {
-            if (-1 == munmap (it->start, it->length)) {
-                AVG_TRACE(Logger::WARNING, "V4lCamera::close(): munmap failed.");
-            }
-        }
-        m_vBuffers.clear();
-
-        if ( ::close(m_Fd) == -1) {
-            AVG_TRACE(Logger::ERROR, "Error on closing v4l2 device");
-        }
-        AVG_TRACE(Logger::CONFIG, "V4L2 Camera closed");
-
-        m_Fd = -1;
-        m_bCameraAvailable = false;
+    enum v4l2_buf_type Type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+    if (-1 == xioctl (m_Fd, VIDIOC_STREAMOFF, &Type)) {
+        AVG_TRACE(Logger::ERROR, "VIDIOC_STREAMOFF");
     }
+    vector<Buffer>::iterator it;
+    for (it=m_vBuffers.begin(); it != m_vBuffers.end(); ++it) {
+        int err = munmap (it->start, it->length);
+        assert (err != -1);
+    }
+    m_vBuffers.clear();
+
+    ::close(m_Fd);
+    AVG_TRACE(Logger::CONFIG, "V4L2 Camera closed");
+
+    m_Fd = -1;
 }
 
 IntPoint V4LCamera::getImgSize()
@@ -149,46 +136,30 @@ IntPoint V4LCamera::getImgSize()
     return m_ImgSize;
 }
 
-int V4LCamera::getCamPF(const std::string& sPF)
+int V4LCamera::getV4LPF(PixelFormat pf)
 {
-    int pfDef = V4L2_PIX_FMT_BGR24;
-
-    if (sPF == "MONO8") {
-        pfDef = V4L2_PIX_FMT_GREY;
+    switch(pf) {
+        case I8:
+            return V4L2_PIX_FMT_GREY;
+        case YCbCr411:
+            return V4L2_PIX_FMT_Y41P;
+        case YCbCr422:
+            return V4L2_PIX_FMT_UYVY;
+        case YUYV422:
+            return V4L2_PIX_FMT_YUYV;
+        case YCbCr420p:
+            return V4L2_PIX_FMT_YUV420;
+        case R8G8B8:
+            return V4L2_PIX_FMT_BGR24;
+        default:
+            throw Exception(AVG_ERR_INVALID_ARGS,
+                    "Unsupported or illegal value for camera pixel format '" 
+                + Bitmap::getPixelFormatString(pf) + "'.");
     }
-    /*
-    // NOT SUPPORTED YET
-    else if (sPF == "YUV411") {
-        pfDef = V4L2_PIX_FMT_Y41P;
-    }*/
-    else if (sPF == "YUV422") {
-        pfDef = V4L2_PIX_FMT_UYVY;
-    }
-    else if (sPF == "YUYV422") {
-        pfDef = V4L2_PIX_FMT_YUYV;
-    }
-    else if (sPF == "YUV420") {
-        pfDef = V4L2_PIX_FMT_YUV420;
-    }
-    else if (sPF == "RGB") {
-        pfDef = V4L2_PIX_FMT_BGR24;
-    }
-    else {
-        AVG_TRACE (Logger::WARNING,
-                std::string("Unsupported or illegal value for camera pixel format \"") 
-                + sPF + std::string("\"."));
-    }
-    
-    return pfDef;
 }
 
-
-static ProfilingZone CameraConvertProfilingZone("V4L Camera format conversion");
-
 BitmapPtr V4LCamera::getImage(bool bWait)
 {
-    unsigned char *pSrc = 0;
-    
     struct v4l2_buffer Buf;
     CLEAR(Buf);
     
@@ -224,98 +195,40 @@ BitmapPtr V4LCamera::getImage(bool bWait)
     
     // dequeue filled buffer
     if (xioctl (m_Fd, VIDIOC_DQBUF, &Buf) == -1) {
-        switch (errno) {
-            case EAGAIN:
-                    return BitmapPtr();
-
-            case EIO:
-                    AVG_TRACE(Logger::ERROR, "EIO");
-                    exit(1);
-                    break;
-
-            case EINVAL:
-                    AVG_TRACE(Logger::ERROR, "EINVAL");
-                    exit(1);
-                    break;
-
-            default:
-                    AVG_TRACE(Logger::ERROR, "VIDIOC_DQBUF");
-                    exit(1);
+        if (errno == EAGAIN) {
+            return BitmapPtr();
+        } else {
+            cerr << strerror(errno) << endl;
+            assert(false);
         }
     }
     
-    pSrc = (unsigned char*)m_vBuffers[Buf.index].start;
-        
-    IntPoint Size = getImgSize();
+    unsigned char * pCaptureBuffer = (unsigned char*)m_vBuffers[Buf.index].start;
+   
+    IntPoint size = getImgSize();
+    double lineLen;
+    switch (getCamPF()) {
+        case YCbCr411:
+            lineLen = size.x*1.5;
+            break;
+        case YCbCr420p:
+            lineLen = size.x;
+            break;
+        default:
+            lineLen = size.x*Bitmap::getBytesPerPixel(getCamPF());
+    }
+    BitmapPtr pCamBmp(new Bitmap(size, getCamPF(), pCaptureBuffer, lineLen,
+            false, "TempCameraBmp"));
 
-    // target bitmap and uchar* to
-    BitmapPtr pCurBitmap;
-    if (m_bColor) {
-        pCurBitmap = BitmapPtr(new Bitmap(Size, B8G8R8X8));
-    }
-    else {
-        pCurBitmap = BitmapPtr(new Bitmap(Size, I8));
-    }
-    
-    unsigned char *pDest = pCurBitmap->getPixels();
-
-    ScopeTimer Timer(CameraConvertProfilingZone);
-
-    switch (m_CamPF) {
-    // fast BGR24 -> B8G8R8X8 conversion
-    case V4L2_PIX_FMT_BGR24:
-        for (int imgp = 0 ; imgp < Size.x * Size.y ; ++imgp) {
-            *pDest++ = *pSrc++; // Blue
-            *pDest++ = *pSrc++; // Green
-            *pDest++ = *pSrc++; // Red
-            *pDest++ = 0xFF;    // Alpha
-        }
-        break;
-    case V4L2_PIX_FMT_GREY:
-        {
-            Bitmap TempBmp(Size, I8, pSrc, Size.x, false, 
-                    "TempCameraBmp");
-            pCurBitmap->copyPixels(TempBmp);
-        }
-        break;
-    case V4L2_PIX_FMT_UYVY:
-        {
-            Bitmap TempBmp(Size, YCbCr422, pSrc, Size.x*2, false, 
-                    "TempCameraBmp");
-            pCurBitmap->copyPixels(TempBmp);
-        }
-        break;
-    case V4L2_PIX_FMT_YUYV:
-        {
-            Bitmap TempBmp(Size, YUYV422, pSrc, Size.x*2, false, 
-                    "TempCameraBmp");
-            pCurBitmap->copyPixels(TempBmp);
-        }
-        break;
-    case V4L2_PIX_FMT_YUV420:
-        {
-            Bitmap TempBmp(Size, YCbCr420p, pSrc, Size.x, false, 
-                    "TempCameraBmp");
-            pCurBitmap->copyPixels(TempBmp);
-        }
-        break;
-    /*
-    // NOT SUPPORTED YET
-    case V4L2_PIX_FMT_Y41P:
-    {
-        Bitmap TempBmp(Size, YCbCr411, pSrc, (int)(Size.x*1.5), false, "TempCameraBmp");
-        pCurBitmap->copyPixels(TempBmp);
-    }
-    break; */
-    }
+    BitmapPtr pDestBmp = convertCamFrameToDestPF(pCamBmp);
 
     // enqueues free buffer for mmap
     if (-1 == xioctl (m_Fd, VIDIOC_QBUF, &Buf)) {
-        AVG_TRACE(Logger::ERROR, "VIDIOC_DQBUF");
-        exit(1);
+        throw(Exception(AVG_ERR_CAMERA_FATAL, 
+                "V4L Camera: failed to enqueue image buffer."));
     }
     
-    return pCurBitmap;
+    return pDestBmp;
 }
 
 bool V4LCamera::isCameraAvailable()
@@ -363,8 +276,6 @@ V4LCID_t V4LCamera::getFeatureID(CameraFeature Feature) const
         V4LFeature = V4L2_CID_GAIN;
     } else if (Feature == CAM_FEATURE_EXPOSURE) {
         V4LFeature = V4L2_CID_EXPOSURE;
-    } else if (Feature == CAM_FEATURE_WHITE_BALANCE) {
-        V4LFeature = V4L2_CID_WHITENESS;
     } else if (Feature == CAM_FEATURE_GAMMA) {
         V4LFeature = V4L2_CID_GAMMA;
     } else if (Feature == CAM_FEATURE_SATURATION) {
@@ -387,10 +298,9 @@ bool V4LCamera::isFeatureSupported(V4LCID_t V4LFeature) const
     
     if (ioctl (m_Fd, VIDIOC_QUERYCTRL, &QueryCtrl) == -1) {
             if (errno != EINVAL) {
-                    AVG_TRACE(Logger::ERROR,"VIDIOC_QUERYCTRL");
-                    exit(1);
-            } 
-            else {
+                cerr << "Got " << strerror(errno) << endl;
+                assert(false);
+            } else {
                 return false;
             }
     } else if (QueryCtrl.flags & V4L2_CTRL_FLAG_DISABLED) {
@@ -400,7 +310,7 @@ bool V4LCamera::isFeatureSupported(V4LCID_t V4LFeature) const
     }
 }
 
-unsigned int V4LCamera::getFeature(CameraFeature Feature) const
+int V4LCamera::getFeature(CameraFeature Feature) const
 {
     V4LCID_t V4LFeature = getFeatureID(Feature);
     
@@ -421,7 +331,7 @@ void V4LCamera::setFeature(V4LCID_t V4LFeature, int Value)
     }
 
     if (!isFeatureSupported(V4LFeature)) {
-        AVG_TRACE(Logger::WARNING, "Feature " << getFeatureName(V4LFeature) <<
+        AVG_TRACE(Logger::WARNING, "Camera feature " << getFeatureName(V4LFeature) <<
             " is not supported by hardware");
         return;
     }
@@ -441,7 +351,34 @@ void V4LCamera::setFeature(V4LCID_t V4LFeature, int Value)
     }
 }
 
-void V4LCamera::setFeature(CameraFeature Feature, int Value)
+void V4LCamera::setFeatureOneShot(CameraFeature Feature)
+{
+    AVG_TRACE(Logger::WARNING, "setFeatureOneShot is not supported by V4L cameras.");
+}
+
+int V4LCamera::getWhitebalanceU() const
+{
+    AVG_TRACE(Logger::WARNING, "getWhitebalance is not supported by V4L cameras.");
+    return 0;
+}
+
+int V4LCamera::getWhitebalanceV() const
+{
+    AVG_TRACE(Logger::WARNING, "getWhitebalance is not supported by V4L cameras.");
+    return 0;
+}
+
+void V4LCamera::setWhitebalance(int u, int v, bool bIgnoreOldValue)
+{
+    setFeature(V4L2_CID_RED_BALANCE, u); 
+    setFeature(V4L2_CID_BLUE_BALANCE, v); 
+}
+
+void V4LCamera::dumpCameras()
+{
+}
+
+void V4LCamera::setFeature(CameraFeature Feature, int Value, bool bIgnoreOldValue)
 {
     // ignore -1 coming from default unbiased cameranode parameters
     if (Value < 0) return;
@@ -473,19 +410,14 @@ void V4LCamera::startCapture()
         Buf.memory      = V4L2_MEMORY_MMAP;
         Buf.index       = i;
 
-        if (-1 == xioctl (m_Fd, VIDIOC_QBUF, &Buf)) {
-            AVG_TRACE(Logger::ERROR, "VIDIOC_QBUF");
-            exit(1);
-        }
+        int err = xioctl (m_Fd, VIDIOC_QBUF, &Buf);
+        assert(err != -1);
     }
     
     Type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
-    if (-1 == xioctl (m_Fd, VIDIOC_STREAMON, &Type)) {
-        AVG_TRACE(Logger::ERROR, "VIDIOC_STREAMON");
-        exit(1);
-    }
-
+    int err= xioctl (m_Fd, VIDIOC_STREAMON, &Type);
+    assert(err != -1);
 }
 
 void V4LCamera::initDevice()
@@ -498,23 +430,23 @@ void V4LCamera::initDevice()
     struct v4l2_format Fmt;
 
     if (xioctl(m_Fd, VIDIOC_QUERYCAP, &Cap) == -1) {
-        AVG_TRACE(Logger::ERROR, m_sDevice << " is not a valid V4L2 device");
-        exit(1);
+        close();
+        fatalError(m_sDevice + " is not a valid V4L2 device.");
     }
 
     if (!(Cap.capabilities & V4L2_CAP_VIDEO_CAPTURE)) {
-        AVG_TRACE(Logger::ERROR, m_sDevice << " does not support capturing");
-        exit(1);
+        close();
+        fatalError(m_sDevice + " does not support capturing");
     }
 
     if (!(Cap.capabilities & V4L2_CAP_STREAMING)) {
-        AVG_TRACE(Logger::ERROR, m_sDevice << " does not support streaming i/os");
-        exit(1);
+        close();
+        fatalError(m_sDevice + " does not support streaming i/os");
     }
     m_sDriverName = (const char *)Cap.driver;
 
     /* Select video input, video standard and tune here. */
-    CLEAR (CropCap);
+    CLEAR(CropCap);
     CropCap.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
     if (xioctl(m_Fd, VIDIOC_CROPCAP, &CropCap) == 0) {
@@ -540,14 +472,15 @@ void V4LCamera::initDevice()
     Fmt.type                = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     Fmt.fmt.pix.width       = getImgSize().x; 
     Fmt.fmt.pix.height      = getImgSize().y;
-    Fmt.fmt.pix.pixelformat = m_CamPF;
+    Fmt.fmt.pix.pixelformat = m_v4lPF;
     Fmt.fmt.pix.field       = V4L2_FIELD_ANY;
 
     if (xioctl(m_Fd, VIDIOC_S_FMT, &Fmt) == -1) {
-        AVG_TRACE(Logger::ERROR, m_sDevice << " could not set image format (" 
-                << strerror(errno) << ")");
         close();
-        exit(1);
+        throw(Exception(AVG_ERR_CAMERA_NONFATAL, 
+                string("Unable to set V4L camera image format: '")
+                +strerror(errno)
+                +"'. Try using v4l-info to find out what the device supports."));
     }
 
     initMMap ();
@@ -555,9 +488,8 @@ void V4LCamera::initDevice()
     // TODO: string channel instead of numeric
     // select channel
     if (xioctl(m_Fd, VIDIOC_S_INPUT, &m_Channel) == -1) {
-        AVG_TRACE(Logger::ERROR, "Cannot set MUX channel " << m_Channel);
         close();
-        exit(1);
+        fatalError(string("Cannot set MUX channel ")+toString(m_Channel));
     }
     
     m_bCameraAvailable = true;
@@ -580,18 +512,17 @@ void V4LCamera::initMMap()
 
     if (xioctl (m_Fd, VIDIOC_REQBUFS, &Req) == -1) {
         if (EINVAL == errno) {
-            AVG_TRACE(Logger::ERROR, m_sDevice << " does not support \
-                memory mapping");
-            exit(1);
+            close();
+            fatalError(m_sDevice+" does not support memory mapping");
         } else {
-            AVG_TRACE(Logger::ERROR, "V4LCamera::initMMap: " << strerror(errno));
-            exit(1);
+            cerr << "errno: " << strerror(errno);
+            assert(false);
         }
     }
     
     if (Req.count < 2) {
-        AVG_TRACE(Logger::ERROR, "Insufficient buffer memory on " << m_sDevice);
-        exit(1);
+        cerr << "Insufficient buffer memory on " << m_sDevice;
+        assert(false);
     }
 
     m_vBuffers.clear();
@@ -607,8 +538,7 @@ void V4LCamera::initMMap()
         Buf.index = i;
 
         if (xioctl (m_Fd, VIDIOC_QUERYBUF, &Buf) == -1) {
-            AVG_TRACE(Logger::ERROR, "VIDIOC_QUERYBUF index=" << i);
-            exit(1);
+            assert(false);
         }
 
         Tmp.length = Buf.length;
@@ -620,8 +550,7 @@ void V4LCamera::initMMap()
             m_Fd, Buf.m.offset);
 
         if (MAP_FAILED == Tmp.start) {
-            AVG_TRACE(Logger::ERROR, "mmap() failed on buffer index=" << i);
-            exit(1);
+            assert(false);
         }
                 
         m_vBuffers.push_back(Tmp);
