@@ -22,6 +22,7 @@
 #ifndef _V4LCamera_H_
 #define _V4LCamera_H_
 
+#include "../api.h"
 #include "../avgconfigwrapper.h"
 
 #include "Camera.h"
@@ -32,7 +33,7 @@ namespace avg {
 
 typedef unsigned int V4LCID_t;
 
-class V4LCamera: public Camera {
+class AVG_API V4LCamera: public Camera {
 
     struct Buffer {
         void * start;
@@ -41,10 +42,8 @@ class V4LCamera: public Camera {
     
     public:
         V4LCamera(std::string sDevice, int Channel, IntPoint Size,
-            const std::string &PixelFormat, bool bColor);
+                PixelFormat camPF, PixelFormat destPF);
         virtual ~V4LCamera();
-        virtual void open();
-        virtual void close();
 
         virtual IntPoint getImgSize();
         virtual BitmapPtr getImage(bool bWait);
@@ -54,26 +53,33 @@ class V4LCamera: public Camera {
         virtual const std::string& getDriverName() const; 
         virtual double getFrameRate() const;
         
-        virtual unsigned int getFeature(CameraFeature Feature) const;
-        virtual void setFeature(CameraFeature Feature, int Value);
-        
+        virtual int getFeature(CameraFeature Feature) const;
+        virtual void setFeature(CameraFeature Feature, int Value, 
+                bool bIgnoreOldValue=false);
+        virtual void setFeatureOneShot(CameraFeature Feature);
+        virtual int getWhitebalanceU() const;
+        virtual int getWhitebalanceV() const;
+        virtual void setWhitebalance(int u, int v, bool bIgnoreOldValue=false);
+       
+        static void dumpCameras();
+
     private:
+        void initDevice();
+        void startCapture();
+        void initMMap();
+        virtual void close();
+        
+        int getV4LPF(PixelFormat pf);
+        
         int m_Fd;
         int m_Channel;
         std::string m_sDevice;
         std::string m_sDriverName;
         std::vector<Buffer> m_vBuffers;
         bool m_bCameraAvailable;
-        int m_CamPF;
-        bool m_bColor;
+        int m_v4lPF;
         IntPoint m_ImgSize;
         
-        int getCamPF(const std::string& sPF);
-        
-        void initDevice();
-        void startCapture();
-        void initMMap();
-        
         void setFeature(V4LCID_t V4LFeature, int Value);
         V4LCID_t getFeatureID(CameraFeature Feature) const;
         std::string getFeatureName(V4LCID_t V4LFeature);
