@@ -29,21 +29,28 @@ const char * g_pTrackerConfigDTD =
 "    y CDATA #REQUIRED\" >\n"
  
 "<!ELEMENT trackerconfig (camera|tracker|transform)* >\n"
-"<!ELEMENT camera (source|device|format|size|channel|fps|brightness|exposure|gamma|gain|shutter)* >\n"
-"<!ELEMENT tracker (mask|prescale|historyupdateinterval|brighterregions|eventonmove|contourprecision|touch|track)* >\n"
-"<!ELEMENT touch (threshold|similarity|areabounds|eccentricitybounds|bandpass|bandpasspostmult)* >\n"
+"<!ELEMENT camera (driver|device|fw800|format|size|framerate|brightness|gamma|gain|"
+"        shutter|strobeduration)* >\n"
+"<!ELEMENT tracker (mask|prescale|historyupdateinterval|brighterregions|eventonmove|"
+"       contourprecision|wipeborder|historydelay|touch|track)* >\n"
+"<!ELEMENT touch (threshold|similarity|areabounds|eccentricitybounds|bandpass|"
+"       bandpasspostmult)* >\n"
 "<!ELEMENT track (threshold|similarity|areabounds|eccentricitybounds)* >\n"
 "<!ELEMENT transform (cameradisplacement|camerascale|distortionparams|trapezoid|"
-"       angle|displaydisplacement|displayscale)* >\n"
+"       angle|displaydisplacement|displayscale|activedisplaysize)* >\n"
 
-"<!ELEMENT source EMPTY>\n"
-"<!ATTLIST source\n"
+"<!ELEMENT driver EMPTY>\n"
+"<!ATTLIST driver\n"
 "   value CDATA #REQUIRED >\n"
 
 "<!ELEMENT device EMPTY>\n"
 "<!ATTLIST device\n"
 "   value CDATA #REQUIRED >\n"
 
+"<!ELEMENT fw800 EMPTY>\n"
+"<!ATTLIST fw800\n"
+"   value CDATA #REQUIRED >\n"
+
 "<!ELEMENT format EMPTY>\n"
 "<!ATTLIST format\n"
 "   value CDATA #REQUIRED >\n"
@@ -52,22 +59,14 @@ const char * g_pTrackerConfigDTD =
 "<!ATTLIST size\n"
 "   %posAttrs; >\n"
 
-"<!ELEMENT channel EMPTY>\n"
-"<!ATTLIST channel\n"
-"   value CDATA #REQUIRED >\n"
-
-"<!ELEMENT fps EMPTY>\n"
-"<!ATTLIST fps\n"
+"<!ELEMENT framerate EMPTY>\n"
+"<!ATTLIST framerate\n"
 "   value CDATA #REQUIRED >\n"
 
 "<!ELEMENT brightness EMPTY>\n"
 "<!ATTLIST brightness\n"
 "   value CDATA #REQUIRED >\n"
 
-"<!ELEMENT exposure EMPTY>\n"
-"<!ATTLIST exposure\n"
-"   value CDATA #REQUIRED >\n"
-
 "<!ELEMENT gamma EMPTY>\n"
 "<!ATTLIST gamma\n"
 "   value CDATA #REQUIRED >\n"
@@ -80,6 +79,10 @@ const char * g_pTrackerConfigDTD =
 "<!ATTLIST shutter\n"
 "   value CDATA #REQUIRED >\n"
 
+"<!ELEMENT strobeduration EMPTY>\n"
+"<!ATTLIST strobeduration\n"
+"   value CDATA #REQUIRED >\n"
+
 "<!ELEMENT brighterregions EMPTY>\n"
 "<!ATTLIST brighterregions\n"
 "   value CDATA #REQUIRED >\n"
@@ -108,6 +111,14 @@ const char * g_pTrackerConfigDTD =
 "<!ATTLIST contourprecision\n"
 "   value CDATA #REQUIRED >\n"
 
+"<!ELEMENT wipeborder EMPTY>\n"
+"<!ATTLIST wipeborder\n"
+"   value CDATA #REQUIRED >\n"
+
+"<!ELEMENT historydelay EMPTY>\n"
+"<!ATTLIST historydelay\n"
+"   value CDATA #REQUIRED >\n"
+
 "<!ELEMENT similarity EMPTY>\n"
 "<!ATTLIST similarity\n"
 "   value CDATA #REQUIRED >\n"
@@ -131,8 +142,7 @@ const char * g_pTrackerConfigDTD =
 "<!ELEMENT distortionparams EMPTY>\n"
 "<!ATTLIST distortionparams\n"
 "   p2 CDATA #REQUIRED\n"
-"   p3 CDATA #REQUIRED\n"
-"   p4 CDATA #REQUIRED >\n"
+"   p3 CDATA #REQUIRED >\n"
 
 "<!ELEMENT trapezoid EMPTY>\n"
 "<!ATTLIST trapezoid\n"
@@ -150,6 +160,9 @@ const char * g_pTrackerConfigDTD =
 "<!ATTLIST displayscale\n"
 "   %posAttrs; >\n"
 
+"<!ELEMENT activedisplaysize EMPTY>\n"
+"<!ATTLIST activedisplaysize\n"
+"   %posAttrs; >\n"
 
 ;
 
