@@ -33,39 +33,33 @@ using namespace std;
 
 namespace avg {
 
-NodeDefinition AVGNode::getNodeDefinition()
+NodeDefinition AVGNode::createDefinition()
 {
     return NodeDefinition("avg", Node::buildNode<AVGNode>)
-        .extendDefinition(DivNode::getNodeDefinition())
-        .setGroupNode()
+        .extendDefinition(DivNode::createDefinition())
         .addArg(Arg<bool>("enablecrop", true, false, offsetof(AVGNode, m_bEnableCrop)))
         .addArg(Arg<string>("onkeyup", ""))
         .addArg(Arg<string>("onkeydown", ""));
 }
 
-AVGNode::AVGNode (const ArgList& Args, Player * pPlayer, bool bFromXML)
-    : DivNode(Args, pPlayer, bFromXML)
+AVGNode::AVGNode (const ArgList& Args)
+    : DivNode(Args)
 {
     Args.setMembers(this);
     addEventHandler(Event::KEYUP, Event::NONE, Args.getArgVal<string>("onkeyup"));
     addEventHandler(Event::KEYDOWN, Event::NONE, Args.getArgVal<string>("onkeydown"));
-    Node::setAngle(0.0);
+    AreaNode::setAngle(0.0);
 }
 
 AVGNode::~AVGNode()
 {
 }
 
-string AVGNode::getTypeStr ()
-{
-    return "AVGNode";
-}
-
 string AVGNode::getEffectiveMediaDir()
 {
     string sMediaDir = getMediaDir();
     if (!isAbsPath(sMediaDir)) {
-        sMediaDir = getPlayer()->getCurDirName()+sMediaDir;
+        sMediaDir = Player::get()->getCurDirName()+sMediaDir;
     }
     if (sMediaDir[sMediaDir.length()-1] != '/') {
         sMediaDir += '/';
