@@ -22,6 +22,7 @@
 #ifndef _AVGNode_H_
 #define _AVGNode_H_
 
+#include "../api.h"
 #include "DivNode.h"
 #include "KeyEvent.h"
 
@@ -29,15 +30,14 @@
 
 namespace avg {
 
-class AVGNode : public DivNode
+class AVG_API AVGNode : public DivNode
 {
     public:
-        static NodeDefinition getNodeDefinition();
+        static NodeDefinition createDefinition();
         
-        AVGNode (const ArgList& Args, Player * pPlayer, bool bFromXML);
+        AVGNode (const ArgList& Args);
         virtual ~AVGNode ();
 
-        virtual std::string getTypeStr ();
         virtual std::string getEffectiveMediaDir();
         bool getCropSetting();
         void setAngle(double angle);
