@@ -22,9 +22,10 @@
 #ifndef _CameraNode_H_
 #define _CameraNode_H_
 
+#include "../api.h"
 #include "../avgconfigwrapper.h"
 
-#include "VideoBase.h"
+#include "RasterNode.h"
 
 #include "../imaging/Camera.h"
 
@@ -35,16 +36,20 @@
 
 namespace avg {
 
-class CameraNode : public VideoBase
+class AVG_API CameraNode : public RasterNode
 {
     public:
-        static NodeDefinition getNodeDefinition();
+        static NodeDefinition createDefinition();
         
-        CameraNode(const ArgList& Args, Player * pPlayer, bool bFromXML);
+        CameraNode(const ArgList& Args);
         virtual ~CameraNode();
 
-        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine, AudioEngine * pAudioEngine);
-        virtual std::string getTypeStr();
+        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine,
+                AudioEngine * pAudioEngine);
+                
+        void play();
+        void stop();
+        bool isAvailable();
 
         const std::string& getDevice() const 
         {
@@ -63,8 +68,6 @@ class CameraNode : public VideoBase
 
         int getBrightness() const;
         void setBrightness(int Value);
-        int getExposure() const;
-        void setExposure(int Value);
         int getSharpness() const;
         void setSharpness(int Value);
         int getSaturation() const;
@@ -75,28 +78,35 @@ class CameraNode : public VideoBase
         void setShutter(int Value);
         int getGain() const;
         void setGain(int Value);
-        unsigned int getWhiteBalance() const;
-        void setWhiteBalance(int Value);
+        int getWhitebalanceU() const;
+        int getWhitebalanceV() const;
+        void setWhitebalance(int u, int v);
+        int getStrobeDuration() const;
+        void setStrobeDuration(int Value);
         
         virtual void preRender();
-
-        unsigned int getFeature (CameraFeature Feature) const;
-        void setFeature (CameraFeature Feature, int Value);
+        virtual void render(const DRect& Rect);
 
         int getFrameNum() const;
         IntPoint getMediaSize();
 
+        static void dumpCameras();
+        static void resetFirewireBus();
+
     private:
-        virtual bool renderToSurface(ISurface * pSurface);
-        virtual double getFPS();
-        virtual void open(YCbCrMode ycbcrMode);
-        virtual void close();
+        int getFeature (CameraFeature Feature) const;
+        void setFeature (CameraFeature Feature, int Value);
+
+        virtual double getFPS() const;
+        virtual void open();
         virtual PixelFormat getPixelFormat();
         void setFeature(int FeatureID);
 
+        bool m_bIsPlaying;
+    
         CameraPtr m_pCamera;
         int m_FrameNum;
-        BitmapPtr pCurBmp;
+        BitmapPtr m_pCurBmp;
 };
 
 }
