@@ -27,7 +27,8 @@ using namespace std;
 
 namespace avg {
 
-CursorState::CursorState(const CursorEventPtr pEvent, const vector<NodeWeakPtr>& pNodes)
+CursorState::CursorState(const CursorEventPtr pEvent, 
+        const vector<NodeWeakPtr>& pNodes)
     : m_pNodes(pNodes)
 {
     m_pLastEvent = pEvent;
@@ -37,7 +38,8 @@ CursorState::~CursorState()
 {
 }
 
-void CursorState::setInfo(const CursorEventPtr pEvent, const std::vector<NodeWeakPtr>& pNodes)
+void CursorState::setInfo(const CursorEventPtr pEvent, 
+        const vector<NodeWeakPtr>& pNodes)
 {
     m_pLastEvent = pEvent;
     m_pNodes = pNodes;
