@@ -20,75 +20,77 @@
 //
 
 #include "DivNode.h"
-#include "DisplayEngine.h"
+#include "SDLDisplayEngine.h"
 #include "Player.h"
 #include "NodeDefinition.h"
 
+#include "../base/Point.h"
 #include "../base/Exception.h"
 #include "../base/Logger.h"
-#include "../base/XMLHelper.h"
 #include "../base/StringHelper.h"
 #include "../base/FileHelper.h"
+#include "../base/MathHelper.h"
+#include "../base/ObjectCounter.h"
 
 #include <iostream>
 #include <sstream>
 
 using namespace std;
+using namespace boost;
 
 namespace avg {
 
-NodeDefinition DivNode::getNodeDefinition()
+NodeDefinition DivNode::createDefinition()
 {
+    string sChildArray[] = {"image", "div", "canvas", "words", "video", "camera", 
+            "panoimage", "sound", "line", "rect", "curve", "polyline", "polygon",
+            "circle", "mesh"};
+    vector<string> sChildren = vectorFromCArray(
+            sizeof(sChildArray) / sizeof(*sChildArray), sChildArray);
     return NodeDefinition("div", Node::buildNode<DivNode>)
-        .extendDefinition(Node::getNodeDefinition())
-        .setGroupNode()
-        .addArg(Arg<string>("mediadir", "", false, offsetof(DivNode, m_sMediaDir)))
-        .addArg(Arg<bool>("crop", true, false, offsetof(DivNode, m_bCrop)));
+        .extendDefinition(AreaNode::createDefinition())
+        .addChildren(sChildren)
+        .addArg(Arg<bool>("crop", true, false, offsetof(DivNode, m_bCrop)))
+        .addArg(Arg<string>("elementoutlinecolor", "", false, 
+                offsetof(DivNode, m_sElementOutlineColor)))
+        .addArg(Arg<UTF8String>("mediadir", "", false, offsetof(DivNode, m_sMediaDir)));
 }
 
-DivNode::DivNode (const ArgList& Args, Player * pPlayer, bool bFromXML)
-    : Node(pPlayer)
+DivNode::DivNode(const ArgList& Args)
 {
     Args.setMembers(this);
+    setElementOutlineColor(m_sElementOutlineColor);
+    ObjectCounter::get()->incRef(&typeid(*this));
 }
 
 DivNode::~DivNode()
 {
+    ObjectCounter::get()->decRef(&typeid(*this));
 }
 
-void DivNode::setRenderingEngines(DisplayEngine * pDisplayEngine, AudioEngine * pAudioEngine)
+void DivNode::setRenderingEngines(DisplayEngine * pDisplayEngine, 
+        AudioEngine * pAudioEngine)
 {
-    Node::setRenderingEngines(pDisplayEngine, pAudioEngine);
-    for  (int i = 0; i< (int)m_Children.size(); ++i) {
+    AreaNode::setRenderingEngines(pDisplayEngine, pAudioEngine);
+    for  (int i = 0; i<(int)m_Children.size(); ++i) {
         m_Children[i]->setRenderingEngines(pDisplayEngine, pAudioEngine);
     }
 }
 
 void DivNode::connect()
 {
-    Node::connect();
+    AreaNode::connect();
     for (int i = 0; i< (int)m_Children.size(); ++i) {
         m_Children[i]->connect();
     }
 }
 
-void DivNode::disconnect()
+void DivNode::disconnect(bool bKill)
 {
     for  (int i = 0; i< (int)m_Children.size(); ++i) {
-        m_Children[i]->disconnect();
+        m_Children[i]->disconnect(bKill);
     }
-    Node::disconnect();
-}
-
-const string& DivNode::getMediaDir() const
-{
-    return m_sMediaDir;
-}
-
-void DivNode::setMediaDir(const string& sMediaDir)
-{
-    m_sMediaDir = sMediaDir;
-    checkReload();
+    AreaNode::disconnect(bKill);
 }
 
 bool DivNode::getCrop() const
@@ -101,12 +103,38 @@ void DivNode::setCrop(bool bCrop)
     m_bCrop = bCrop;
 }
 
-int DivNode::getNumChildren ()
+const std::string& DivNode::getElementOutlineColor() const
+{
+    return m_sElementOutlineColor;
+}
+
+void DivNode::setElementOutlineColor(const std::string& sColor)
+{
+    m_sElementOutlineColor = sColor;
+    if (sColor == "") {
+        m_ElementOutlineColor = Pixel32(0,0,0,0);
+    } else {
+        m_ElementOutlineColor = colorStringToColor(m_sElementOutlineColor);
+    }
+}
+
+const UTF8String& DivNode::getMediaDir() const
+{
+    return m_sMediaDir;
+}
+
+void DivNode::setMediaDir(const UTF8String& sMediaDir)
+{
+    m_sMediaDir = sMediaDir;
+    checkReload();
+}
+
+int DivNode::getNumChildren()
 {
     return int(m_Children.size());
 }
 
-NodePtr DivNode::getChild (unsigned i)
+const NodePtr& DivNode::getChild(unsigned i)
 {
     if (i >= m_Children.size()) {
         stringstream s;
@@ -116,7 +144,7 @@ NodePtr DivNode::getChild (unsigned i)
     return m_Children[i];
 }
 
-void DivNode::appendChild (NodePtr pNewNode)
+void DivNode::appendChild(NodePtr pNewNode)
 {
     insertChild(pNewNode, unsigned(m_Children.size()));
 }
@@ -138,6 +166,12 @@ void DivNode::insertChild(NodePtr pNewNode, unsigned i)
         throw Exception(AVG_ERR_NO_NODE,
                 getID()+"::insertChild called without a node.");
     }
+    if (!isChildTypeAllowed(pNewNode->getTypeStr())) {
+        throw(Exception(AVG_ERR_ALREADY_CONNECTED,
+                "Can't insert a node of type "+pNewNode->getTypeStr()+
+                " into a node of type "+getTypeStr()+"."));
+
+    }
     if (pNewNode->getState() == NS_CONNECTED || pNewNode->getState() == NS_CANRENDER) 
     {
         throw(Exception(AVG_ERR_ALREADY_CONNECTED,
@@ -150,7 +184,7 @@ void DivNode::insertChild(NodePtr pNewNode, unsigned i)
     }
     std::vector<NodePtr>::iterator Pos = m_Children.begin()+i;
     if (getState() == NS_CONNECTED || getState() == NS_CANRENDER) {
-        getPlayer()->registerNode(pNewNode);
+        Player::get()->registerNode(pNewNode);
     }
     m_Children.insert(Pos, pNewNode);
     DivNodePtr Ptr = boost::dynamic_pointer_cast<DivNode>(getThis());           
@@ -162,19 +196,29 @@ void DivNode::insertChild(NodePtr pNewNode, unsigned i)
 
 void DivNode::removeChild(NodePtr pNode)
 {
-    int i = indexOf(pNode);
-    pNode->removeParent();
-    m_Children.erase(m_Children.begin()+i);
+    removeChild(pNode, false);
 }
 
 void DivNode::removeChild(unsigned i)
+{
+    removeChild(i, false);
+}
+
+void DivNode::removeChild(NodePtr pNode, bool bKill)
+{
+    int i = indexOf(pNode);
+    pNode->removeParent(bKill);
+    m_Children.erase(m_Children.begin()+i);
+}
+
+void DivNode::removeChild(unsigned i, bool bKill)
 {
     if (i>m_Children.size()-1) {
         throw(Exception(AVG_ERR_OUT_OF_RANGE,
                 getID()+"::removeChild: index "+toString(i)+" out of bounds."));
     }
     NodePtr pNode = getChild(i);
-    pNode->removeParent();
+    pNode->removeParent(bKill);
     m_Children.erase(m_Children.begin()+i);
 }
 
@@ -218,26 +262,34 @@ int DivNode::indexOf(NodePtr pChild)
             +getID()+"'"));
 }
 
-NodePtr DivNode::getElementByPos (const DPoint & pos)
+NodePtr DivNode::getElementByPos(const DPoint & pos)
 {
-    DPoint relPos = toLocal(pos);
-    if (relPos.x >= 0 && relPos.y >= 0 && 
-            relPos.x < getSize().x && relPos.y < getSize().y &&
-            reactsToMouseEvents())
+    if (reactsToMouseEvents() &&
+            ((getSize() == DPoint(10000, 10000) ||
+             (pos.x >= 0 && pos.y >= 0 && pos.x < getSize().x && pos.y < getSize().y))))
     {
         for (int i=getNumChildren()-1; i>=0; i--) {
-            NodePtr pFoundNode = getChild(i)->getElementByPos(relPos);
+            // TODO: Move coordinate handling to Node (& get rid of AreaNode entirely?)
+            AreaNodePtr pCurChild = dynamic_pointer_cast<AreaNode>(getChild(i));
+            NodePtr pFoundNode;
+            DPoint relPos;
+            if (pCurChild) {
+                relPos = pCurChild->toLocal(pos);
+                pFoundNode = pCurChild->getElementByPos(relPos);
+            } else {
+                pFoundNode = getChild(i)->getElementByPos(pos);
+            }
             if (pFoundNode) {
                 return pFoundNode;
             }
         }
         // Pos isn't in any of the children.
-        if (getSize() != DPoint(10000, 10000)) {
-            // Explicit width/height given for div.
-            return getThis(); 
-        } else {
+        if (getSize() == DPoint(10000, 10000)) {
             // Explicit width/height not given: div itself doesn't react.
             return NodePtr();
+        } else {
+            // Explicit width/height given for div.
+            return getThis();
         }
     } else { 
         return NodePtr();
@@ -246,6 +298,7 @@ NodePtr DivNode::getElementByPos (const DPoint & pos)
 
 void DivNode::preRender()
 {
+    Node::preRender();
     for (int i=0; i<getNumChildren(); i++) {
         getChild(i)->preRender();
     }
@@ -254,26 +307,76 @@ void DivNode::preRender()
 void DivNode::render(const DRect& rect)
 {
     DPoint Viewport = getSize();
-    if (m_bCrop) {
+    if (getCrop()) {
         DRect ClipRect(0, 0, Viewport.x, Viewport.y);
         getDisplayEngine()->pushClipRect(ClipRect);
     }
     for (int i=0; i<getNumChildren(); i++) {
         getChild(i)->maybeRender(rect);
     }
-    if (m_bCrop) {
+    if (getCrop()) {
         getDisplayEngine()->popClipRect();
     }
 }
 
-string DivNode::getTypeStr ()
+void DivNode::renderOutlines(VertexArrayPtr pVA, Pixel32 color)
 {
-    return "DivNode";
+    Pixel32 effColor = color;
+    if (m_ElementOutlineColor != Pixel32(0,0,0,0)) {
+        effColor = m_ElementOutlineColor;
+        effColor.setA(128);
+    }
+    if (effColor != Pixel32(0,0,0,0)) {
+        DPoint size = getSize();
+        if (size == DPoint(10000, 10000)) {
+            DPoint p0 = getAbsPos(DPoint(-4, 0.5));
+            DPoint p1 = getAbsPos(DPoint(5, 0.5));
+            DPoint p2 = getAbsPos(DPoint(0.5, -4));
+            DPoint p3 = getAbsPos(DPoint(0.5, 5));
+            pVA->addLineData(effColor, p0, p1, 1);
+            pVA->addLineData(effColor, p2, p3, 1);
+        } else {
+            DPoint p0 = getAbsPos(DPoint(0.5, 0.5));
+            DPoint p1 = getAbsPos(DPoint(size.x+0.5,0.5));
+            DPoint p2 = getAbsPos(DPoint(size.x+0.5,size.y+0.5));
+            DPoint p3 = getAbsPos(DPoint(0.5,size.y+0.5));
+            pVA->addLineData(effColor, p0, p1, 1);
+            pVA->addLineData(effColor, p1, p2, 1);
+            pVA->addLineData(effColor, p2, p3, 1);
+            pVA->addLineData(effColor, p3, p0, 1);
+        }
+    }
+    for (int i=0; i<getNumChildren(); i++) {
+        getChild(i)->renderOutlines(pVA, effColor);
+    }
 }
 
-string DivNode::dump (int indent)
+string DivNode::getEffectiveMediaDir()
 {
-    string dumpStr = Node::dump () + "\n";
+    string sMediaDir = m_sMediaDir;
+    if (!isAbsPath(sMediaDir)) {
+        if (getParent()) {
+            sMediaDir = getParent()->getEffectiveMediaDir()+m_sMediaDir;
+        } else {
+            sMediaDir = Player::get()->getRootMediaDir()+m_sMediaDir;
+        }
+    }
+    if (sMediaDir[sMediaDir.length()-1] != '/') {
+        sMediaDir += '/';
+    }
+    return sMediaDir;
+}
+
+void DivNode::checkReload()
+{
+    for(int i=0; i<getNumChildren(); ++i) {
+        getChild(i)->checkReload();
+    }
+}
+
+string DivNode::dump(int indent)
+{
+    string dumpStr = AreaNode::dump () + "\n";
     vector<NodePtr>::iterator it;
     for (it=m_Children.begin(); it<m_Children.end(); it++) {
         dumpStr += (*it)->dump(indent+2)+"\n";
@@ -285,29 +388,10 @@ IntPoint DivNode::getMediaSize()
 {
     return IntPoint(10000,10000);
 }
-
-string DivNode::getEffectiveMediaDir()
+ 
+bool DivNode::isChildTypeAllowed(const string& sType)
 {
-    string sMediaDir = m_sMediaDir;
-    if (!isAbsPath(sMediaDir)) {
-        if (getParent()) {
-            sMediaDir = getParent()->getEffectiveMediaDir()+m_sMediaDir;
-        } else {
-            sMediaDir = getPlayer()->getRootMediaDir()+m_sMediaDir;
-        }
-    }
-    if (sMediaDir[sMediaDir.length()-1] != '/') {
-        sMediaDir += '/';
-    }
-    return sMediaDir;
-}
-
-void DivNode::checkReload()
-{
-    vector<NodePtr>::iterator it;
-    for (it=m_Children.begin(); it<m_Children.end(); it++) {
-        (*it)->checkReload();
-    }
+    return getDefinition()->isChildAllowed(sType);
 }
 
 }
