@@ -22,55 +22,68 @@
 #ifndef _DivNode_H_
 #define _DivNode_H_
 
-#include "Node.h"
+#include "../api.h"
+#include "AreaNode.h"
 
-#include "../base/Point.h"
+#include "../base/UTF8String.h"
+#include "../graphics/VertexArray.h"
 
 #include <string>
 
 namespace avg {
     
-class DivNode : public Node
+class AVG_API DivNode : public AreaNode
 {
     public:
-        static NodeDefinition getNodeDefinition();
+        static NodeDefinition createDefinition();
         
-        DivNode (const ArgList& Args, Player * pPlayer, bool bFromXML);
-        virtual ~DivNode ();
-        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine, AudioEngine * pAudioEngine);
+        DivNode(const ArgList& Args);
+        virtual ~DivNode();
+        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine, 
+                AudioEngine * pAudioEngine);
         virtual void connect();
-        virtual void disconnect();
-
-        const std::string& getMediaDir() const;
-        void setMediaDir(const std::string& mediaDir);
+        virtual void disconnect(bool bKill);
 
         bool getCrop() const;
         void setCrop(bool bCrop);
 
+        const std::string& getElementOutlineColor() const;
+        void setElementOutlineColor(const std::string& sColor);
+
+        const UTF8String& getMediaDir() const;
+        void setMediaDir(const UTF8String& mediaDir);
+
         int getNumChildren();
-        NodePtr getChild(unsigned i);
+        const NodePtr& getChild(unsigned i);
         void appendChild(NodePtr pNewNode);
         void insertChildBefore(NodePtr pNewNode, NodePtr pOldChild);
         void insertChild(NodePtr pNewNode, unsigned i);
         void removeChild(NodePtr pNode);
         void removeChild(unsigned i);
+        void removeChild(NodePtr pNode, bool bKill);
+        void removeChild(unsigned i, bool bKill);
         void reorderChild(NodePtr pNode, unsigned j);
         void reorderChild(unsigned i, unsigned j);
         int indexOf(NodePtr pChild);
 
-        virtual NodePtr getElementByPos (const DPoint & pos);
+        virtual NodePtr getElementByPos(const DPoint & pos);
         virtual void preRender();
-        virtual void render (const DRect& rect);
-        virtual std::string getTypeStr ();
+        virtual void render(const DRect& rect);
+        virtual void renderOutlines(VertexArrayPtr pVA, Pixel32 color);
+
         virtual std::string getEffectiveMediaDir();
         virtual void checkReload();
 
-        virtual std::string dump (int indent = 0);
-        virtual IntPoint getMediaSize();
-    
+        virtual std::string dump(int indent = 0);
+        IntPoint getMediaSize();
+   
     private:
-        std::string m_sMediaDir;
+        bool isChildTypeAllowed(const std::string& sType);
+
+        UTF8String m_sMediaDir;
         bool m_bCrop;
+        std::string m_sElementOutlineColor;
+        Pixel32 m_ElementOutlineColor;
         std::vector<NodePtr> m_Children;
 };
 
