@@ -22,56 +22,62 @@
 #ifndef _Image_H_
 #define _Image_H_
 
-#include "RasterNode.h"
+#include "../api.h"
+#include "OGLTexture.h"
 
+#include "../base/Point.h"
 #include "../graphics/Bitmap.h"
 
+#include <boost/shared_ptr.hpp>
 #include <string>
 
 namespace avg {
 
-class Image : public RasterNode
+class OGLSurface;
+class OGLTiledSurface;
+class SDLDisplayEngine;
+
+class AVG_API Image
 {
     public:
-        static NodeDefinition getNodeDefinition();
-        
-        Image (const ArgList& Args, Player * pPlayer, bool bFromXML);
-        virtual ~Image ();
-        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine, 
-                AudioEngine * pAudioEngine);
-        virtual void disconnect();
+        enum State {NOT_AVAILABLE, CPU, GPU};
+
+        Image(OGLSurface * pSurface, const std::string& sFilename, bool bTiled);
+        Image(OGLSurface * pSurface, const Bitmap* pBmp, bool bTiled);
+        virtual ~Image();
 
-        const std::string& getHRef() const;
-        void setHRef(const std::string& href);
-        int getHue() const
-        {
-            return m_Hue;
-        }
-        int getSaturation() const
-        {
-            return m_Saturation;
-        }
         void setBitmap(const Bitmap * pBmp);
+
+        virtual void moveToGPU(SDLDisplayEngine* pEngine);
+        virtual void moveToCPU();
+
+        void setFilename(const std::string& sFilename);
+        const std::string& getFilename() const;
         
-        virtual void render (const DRect& Rect);
-        virtual std::string getTypeStr ();
-        virtual void checkReload();
-        
-        virtual Bitmap* getBitmap();
-        virtual IntPoint getMediaSize();
+        BitmapPtr getBitmap();
+        IntPoint getSize();
+        PixelFormat getPixelFormat();
+        OGLSurface* getSurface();
+        OGLTiledSurface* getTiledSurface();
+        State getState();
+        SDLDisplayEngine* getEngine();
 
     private:
         void load();
         void setupSurface();
-        std::string m_Filename;
-        std::string m_href;
+        PixelFormat calcSurfacePF(const Bitmap& Bmp);
+
+        std::string m_sFilename;
         BitmapPtr m_pBmp;
-        bool m_bIsImageAvailable;
-    
-        int m_Hue;
-        int m_Saturation;
+        OGLSurface * m_pSurface;
+        SDLDisplayEngine * m_pEngine;
+
+        State m_State;
+        bool m_bTiled;
 };
 
+typedef boost::shared_ptr<Image> ImagePtr;
+
 }
 
 #endif
