@@ -27,10 +27,19 @@
 #include "Words.h"
 #include "Video.h"
 #include "CameraNode.h"
-#include "Image.h"
+#include "ImageNode.h"
 #include "PanoImage.h"
 #include "Sound.h"
+#include "LineNode.h"
+#include "RectNode.h"
+#include "CurveNode.h"
+#include "PolyLineNode.h"
+#include "PolygonNode.h"
+#include "CircleNode.h"
+#include "MeshNode.h"
 #include "NodeDefinition.h"
+#include "PluginManager.h"
+#include "TextEngine.h"
 
 #include "TrackerEventSource.h"
 #include "Event.h"
@@ -49,15 +58,9 @@
 #include "../base/Profiler.h"
 #include "../base/ScopeTimer.h"
 #include "../base/TimeSource.h"
+#include "../base/MathHelper.h"
 
-#include "../imaging/FWCamera.h"
-#ifdef AVG_ENABLE_V4L2
-#include "../imaging/V4LCamera.h"
-#endif
-#ifdef _WIN32
-#include "../imaging/DSCamera.h"
-#endif
-#include "../imaging/FakeCamera.h"
+#include "../imaging/Camera.h"
 
 #include "../audio/SDLAudioEngine.h"
 
@@ -80,55 +83,63 @@
 using namespace std;
 
 namespace avg {
-
+    
 Player * Player::s_pPlayer=0;
 
 Player::Player()
     : m_pRootNode(),
       m_pDisplayEngine(0),
       m_pAudioEngine(0),
+      m_bAudioEnabled(true),
       m_pTracker(0),
       m_bInHandleTimers(false),
       m_bCurrentTimeoutDeleted(false),
       m_pEventCaptureNode(),
       m_bUseFakeCamera(false),
-      m_MaxGPUMemUsed(0),
       m_bStopOnEscape(true),
       m_bIsPlaying(false),
       m_bFakeFPS(false),
       m_FakeFPS(0),
       m_FrameTime(0),
-      m_PlayStartTime(0),
       m_Volume(1),
+      m_FrameEndSignal(&IFrameEndListener::onFrameEnd),
+      m_PlaybackEndSignal(&IPlaybackEndListener::onPlaybackEnd),
+      m_PreRenderSignal(&IPreRenderListener::onPreRender),
+      m_dtd(0),
       m_bPythonAvailable(true)
 {
     if (s_pPlayer) {
         throw Exception(AVG_ERR_UNKNOWN, "Player has already been instantiated.");
     }
-    ThreadProfilerPtr pThreadProfiler = ThreadProfilerPtr(new ThreadProfiler("Main"));
-    Profiler::get().registerThreadProfiler(pThreadProfiler);
+    ThreadProfilerPtr pProfiler = ThreadProfiler::get();
+    pProfiler->setName("main");
+    Profiler::get().registerThreadProfiler(pProfiler);
     initConfig();
 
     // Register all node types
-    registerNodeType(AVGNode::getNodeDefinition());
-    registerNodeType(DivNode::getNodeDefinition());
-    registerNodeType(Image::getNodeDefinition());
-    registerNodeType(Words::getNodeDefinition());
-    registerNodeType(Video::getNodeDefinition());
-    registerNodeType(CameraNode::getNodeDefinition());
-    registerNodeType(PanoImage::getNodeDefinition());
-    registerNodeType(Sound::getNodeDefinition());
-
-    // Find and parse dtd.
-    registerDTDEntityLoader("avg.dtd", m_NodeFactory.getDTD().c_str());
-    string sDTDFName = "avg.dtd";
-    m_dtd = xmlParseDTD(NULL, (const xmlChar*) sDTDFName.c_str());
-    if (!m_dtd) {
-        AVG_TRACE(Logger::WARNING, 
-                "DTD not found at " << sDTDFName << ". Not validating xml files.");
-    }
+    registerNodeType(AVGNode::createDefinition());
+    registerNodeType(DivNode::createDefinition());
+    registerNodeType(ImageNode::createDefinition());
+    registerNodeType(Words::createDefinition());
+    registerNodeType(Video::createDefinition());
+    registerNodeType(CameraNode::createDefinition());
+    registerNodeType(PanoImage::createDefinition());
+    registerNodeType(Sound::createDefinition());
+    registerNodeType(LineNode::createDefinition());
+    registerNodeType(RectNode::createDefinition());
+    registerNodeType(CurveNode::createDefinition());
+    registerNodeType(PolyLineNode::createDefinition());
+    registerNodeType(PolygonNode::createDefinition());
+    registerNodeType(CircleNode::createDefinition());
+    registerNodeType(MeshNode::createDefinition());
+    
     m_pTestHelper = new TestHelper(this);
 
+    // Early initialization of TextEngine singletons (dualton? ;-))
+    // to avoid locale clashes with Magick (bug 54)
+    TextEngine::get(true);
+    TextEngine::get(false);
+
 #ifdef _WIN32
     Magick::InitializeMagick((getAvgLibPath()+"magick\\").c_str());
 #endif
@@ -136,6 +147,12 @@ Player::Player()
     s_pPlayer = this;
 }
 
+void deletePlayer()
+{
+    delete Player::s_pPlayer;
+    Player::s_pPlayer = 0;
+}
+
 Player::~Player()
 {
     if (m_pDisplayEngine) {
@@ -154,10 +171,16 @@ Player* Player::get()
 {
     if (!s_pPlayer) {
         s_pPlayer = new Player();
+        atexit(deletePlayer);
     }
     return s_pPlayer;
 }
 
+bool Player::exists()
+{
+    return s_pPlayer != 0;
+}
+
 void Player::setResolution(bool bFullscreen, int width, int height, int bpp)
 {
     m_DP.m_bFullscreen = bFullscreen;
@@ -178,11 +201,11 @@ void Player::setWindowPos(int x, int y)
     m_DP.m_y = y;
 }
 
-void Player::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCbCrMode, 
+void Player::setOGLOptions(bool bUsePOW2Textures, bool bUseShaders, 
                 bool bUsePixelBuffers, int MultiSampleSamples)
 {
     m_bUsePOW2Textures = bUsePOW2Textures;
-    m_YCbCrMode = DesiredYCbCrMode;
+    m_bUseShaders = bUseShaders;
     m_bUsePixelBuffers = bUsePixelBuffers;
     m_MultiSampleSamples = MultiSampleSamples;
 }
@@ -192,13 +215,22 @@ void Player::setMultiSampleSamples(int MultiSampleSamples)
     m_MultiSampleSamples = MultiSampleSamples;
 }
 
+void Player::enableAudio(bool bEnable)
+{
+    if (m_bIsPlaying) {
+        throw Exception(AVG_ERR_UNSUPPORTED, 
+                "Player.enableAudio must be called before Player.play().");
+    }
+    m_bAudioEnabled = bEnable;
+}
+
 void Player::setAudioOptions(int samplerate, int channels)
 {
     m_AP.m_SampleRate = samplerate;
     m_AP.m_Channels = channels;
 }
 
-void Player::loadFile (const std::string& sFilename)
+void Player::loadFile(const std::string& sFilename)
 {
     string RealFilename;
     try {
@@ -208,11 +240,11 @@ void Player::loadFile (const std::string& sFilename)
         // When loading an avg file, assets are loaded from a directory relative
         // to the file.
         char szBuf[1024];
+        char * pBuf = getcwd(szBuf, 1024);
         if (sFilename[0] == '/') {
             RealFilename = sFilename; 
         } else {
-            getcwd(szBuf, 1024);
-            m_CurDirName = string(szBuf)+"/";
+            m_CurDirName = string(pBuf)+"/";
             RealFilename = m_CurDirName+sFilename;
         }
         m_CurDirName = RealFilename.substr(0, RealFilename.rfind('/')+1);
@@ -222,8 +254,7 @@ void Player::loadFile (const std::string& sFilename)
         internalLoad(sAVG);
 
         // Reset the directory to load assets from to the current dir.
-        getcwd(szBuf, 1024);
-        m_CurDirName = string(szBuf)+"/";
+        m_CurDirName = string(pBuf)+"/";
     } catch (Exception& ex) {
         switch (ex.GetCode()) {
             case AVG_ERR_XML_PARSE:
@@ -244,9 +275,6 @@ void Player::loadString(const std::string& sAVG)
 {
     try {
         AVG_TRACE(Logger::MEMORY, "Player::loadString()");
-        char szBuf[1024];
-        getcwd(szBuf, 1024);
-        m_CurDirName = string(szBuf)+"/";
 
         string sEffectiveDoc = removeStartEndSpaces(sAVG);
         internalLoad(sEffectiveDoc);
@@ -280,18 +308,18 @@ void Player::play()
             }
         } catch (...) {
             cleanup();
-            m_bIsPlaying = false;
             throw;
         }
         cleanup();
-
+        AVG_TRACE(Logger::PLAYER, "Playback ended.");
     } catch (Exception& ex) {
+        m_bIsPlaying = false;
         AVG_TRACE(Logger::ERROR, ex.GetStr());
         throw;
     }
 }
 
-void Player::stop ()
+void Player::stop()
 {
     if (m_bIsPlaying) {
         m_bStopping = true;
@@ -306,11 +334,18 @@ void Player::initPlayback()
     if (!m_pRootNode) {
         throw Exception(AVG_ERR_NO_NODE, "Play called, but no xml file loaded.");
     }
-    
+    AVG_TRACE(Logger::PLAYER, "Playback started.");
     initGraphics();
-    initAudio();
-    
-    m_pRootNode->setRenderingEngines(m_pDisplayEngine, m_pAudioEngine);
+    if (m_bAudioEnabled) {
+        initAudio();
+    }
+    try {
+        m_pRootNode->setRenderingEngines(m_pDisplayEngine, m_pAudioEngine);
+    } catch (Exception&) {
+        m_pDisplayEngine = 0;
+        m_pAudioEngine = 0;
+        throw;
+    }
 
     m_pEventDispatcher->addSource(m_pEventSource);
     m_pEventDispatcher->addSource(m_pTestHelper);
@@ -322,10 +357,8 @@ void Player::initPlayback()
         m_pTracker->start();
     }
 
-    m_PlayStartTime = TimeSource::get()->getCurrentMillisecs();
     m_FrameTime = 0;
     m_NumFrames = 0;
-    m_MaxGPUMemUsed = 0;
 }
 
 bool Player::isPlaying()
@@ -333,7 +366,8 @@ bool Player::isPlaying()
     return m_bIsPlaying;
 }
 
-void Player::setFramerate(double rate) {
+void Player::setFramerate(double rate)
+{
     if (m_bIsPlaying) {
         m_pDisplayEngine->setFramerate(rate);
     } else {
@@ -342,18 +376,18 @@ void Player::setFramerate(double rate) {
     }
 }
 
-bool Player::setVBlankFramerate(int rate) {
-    // TODO: Why does this function return anything?
+void Player::setVBlankFramerate(int rate)
+{
     if (m_bIsPlaying) {
-        return m_pDisplayEngine->setVBlankRate(rate);
+        m_pDisplayEngine->setVBlankRate(rate);
     } else {
         m_DP.m_Framerate = 0;
         m_DP.m_VBRate = rate;
-        return true;
     }
 }
         
-double Player::getEffectiveFramerate() {
+double Player::getEffectiveFramerate()
+{
     if (m_bIsPlaying) {
         return m_pDisplayEngine->getEffectiveFramerate();
     } else {
@@ -386,58 +420,44 @@ long long Player::getFrameTime()
     return m_FrameTime;
 }
 
-TrackerEventSource * Player::addTracker(const string& sConfigFilename)
+void Player::addEventSource(IEventSource * pSource)
+{
+    if (!m_pEventDispatcher) {
+        throw Exception(AVG_ERR_UNSUPPORTED, 
+                "You must use loadFile() before addEventSource().");
+    }
+    m_pEventDispatcher->addSource(pSource);
+}
+
+TrackerEventSource * Player::addTracker()
 {
     TrackerConfig Config;
-    Config.load(sConfigFilename);
+    Config.load();
     CameraPtr pCamera;
 
-    string sSource = Config.getParam("/camera/source/@value");
+    string sDriver = Config.getParam("/camera/driver/@value");
     string sDevice = Config.getParam("/camera/device/@value");
-    IntPoint Size(Config.getPointParam("/camera/size/"));
-    string sPixFmt = Config.getParam("/camera/format/@value");
-    double FPS = Config.getDoubleParam("/camera/fps/@value");
+    bool bFW800 = Config.getBoolParam("/camera/fw800/@value");
+    IntPoint CaptureSize(Config.getPointParam("/camera/size/"));
+    string sCaptureFormat = Config.getParam("/camera/format/@value");
+    double FrameRate = Config.getDoubleParam("/camera/framerate/@value");
 
-    if (!m_pEventDispatcher) {
-        throw Exception(AVG_ERR_UNSUPPORTED, "You must use loadFile() before addTracker().");
-    }
-
-    if (sSource == "v4l") {
-#ifdef AVG_ENABLE_V4L2
-        int Channel = Config.getIntParam("/camera/channel/@value");
-        AVG_TRACE(Logger::CONFIG, "Adding a Tracker for V4L camera " << sDevice
-                << " size=" << Size << " channel=" << Channel << " format=" 
-                << sPixFmt);
-        
-        pCamera = CameraPtr(new V4LCamera(sDevice, Channel, Size, sPixFmt, false));
-#else
-        AVG_TRACE(Logger::ERROR, "Video4Linux camera tracker requested, but " 
-                "Video4Linux support not compiled in.");
-        exit(1);
-#endif
-    } else if (sSource == "fw") {
-        AVG_TRACE(Logger::CONFIG, "Adding a Tracker for FW camera " << 
-                sDevice << " size=" << Size << " format=" << sPixFmt);
-        pCamera = CameraPtr(new FWCamera(sDevice, Size, sPixFmt, FPS, false));
-    } else if (sSource == "ds") {
-#ifdef _WIN32        
-        AVG_TRACE(Logger::CONFIG, "Adding a Tracker for DS camera " << 
-                sDevice << " size=" << Size << " format=" << sPixFmt);
-        pCamera = CameraPtr(new DSCamera(sDevice, Size, sPixFmt, FPS, false));
-#else
-        AVG_TRACE(Logger::ERROR, "DS camera tracker requested, but " 
-                "DS support not compiled in.");
-        exit(1);
-#endif        
-    } else {
-        throw Exception(AVG_ERR_INVALID_CAPTURE,
-                string("Invalid camera source value '")+sSource+
-                        "'. Can't initialize tracker.");
+    PixelFormat camPF = Bitmap::stringToPixelFormat(sCaptureFormat);
+    if (camPF == NO_PIXELFORMAT) {
+        throw Exception(AVG_ERR_INVALID_ARGS,
+                "Unknown camera pixel format "+sCaptureFormat+".");
     }
     
+    AVG_TRACE(Logger::CONFIG, "Trying to create a Tracker for " << sDriver
+            << " Camera: " << sDevice << " Size: " << CaptureSize << "format: "
+            << sCaptureFormat);
+    pCamera = createCamera(sDriver, sDevice, -1, bFW800, CaptureSize, camPF, I8, 
+            FrameRate);
+    AVG_TRACE(Logger::CONFIG, "Got Camera " << pCamera->getDevice() << " from driver: " 
+            << pCamera->getDriverName());
     m_pTracker = new TrackerEventSource(pCamera, Config, 
             IntPoint(m_DP.m_Width, m_DP.m_Height), true);
-    m_pEventDispatcher->addSource(m_pTracker);
+    addEventSource(m_pTracker);
     if (m_bIsPlaying) {
         m_pTracker->start();
     }
@@ -445,6 +465,11 @@ TrackerEventSource * Player::addTracker(const string& sConfigFilename)
     return m_pTracker;
 }
 
+TrackerEventSource * Player::getTracker()
+{
+    return m_pTracker;
+}
+
 int Player::setInterval(int time, PyObject * pyfunc)
 {
     Timeout *t = new Timeout(time, pyfunc, true, getFrameTime());
@@ -477,7 +502,7 @@ bool Player::clearInterval(int id)
     vector<Timeout*>::iterator it;
     for (it=m_PendingTimeouts.begin(); it!=m_PendingTimeouts.end(); it++) {
         if (id == (*it)->GetID()) {
-            if (it == m_PendingTimeouts.begin()) {
+            if (it == m_PendingTimeouts.begin() && m_bInHandleTimers) {
                 m_bCurrentTimeoutDeleted = true;
             }
             delete *it;
@@ -500,6 +525,16 @@ MouseEventPtr Player::getMouseState() const
     return m_MouseState.getLastEvent();
 }
 
+void Player::setMousePos(const IntPoint& pos)
+{
+    m_pDisplayEngine->setMousePos(pos);
+}
+
+int Player::getKeyModifierState() const
+{
+    return m_pDisplayEngine->getKeyModifierState();
+}
+
 Bitmap * Player::screenshot()
 {
     BitmapPtr pBmp = m_pDisplayEngine->screenshot();
@@ -515,18 +550,61 @@ void Player::showCursor(bool bShow)
     }
 }
 
-void Player::setEventCapture(NodeWeakPtr pNode, int cursorID) {
+void Player::setCursor(const Bitmap* pBmp, IntPoint hotSpot)
+{
+    IntPoint size = pBmp->getSize();
+    if (size.x % 8 != 0 || size.y % 8 != 0 || pBmp->getPixelFormat() != R8G8B8A8) {
+        throw Exception(AVG_ERR_INVALID_ARGS, 
+                "setCursor: Bitmap size must be divisible by 8 and in RGBA format.");
+    }
+    int i = -1;
+    unsigned char * pData = new unsigned char[size.x*size.y/8];
+    unsigned char * pMask = new unsigned char[size.x*size.y/8];
+    Pixel32 * pLine = (Pixel32*)(pBmp->getPixels());
+    int stride = pBmp->getStride()/4;
+    for (int y=0; y<size.y; ++y) {
+        Pixel32 * pPixel = pLine;
+        for (int x=0; x<size.x; ++x) {
+            if (x % 8 == 0) {
+                i++;
+                pData[i] = 0;
+                pMask[i] = 0;
+            } else {
+                pData[i] <<= 1;
+                pMask[i] <<= 1;
+            }
+            if (pPixel->getA() > 127) {
+                pMask[i] |= 0x01;
+                if (pPixel->getR() < 128) {
+                    // Black Pixel
+                    pData[i] |= 0x01;
+                }
+            }
+            pPixel++;
+        }
+        pLine += stride;
+    }
+    SDL_Cursor * pCursor = SDL_CreateCursor(pData, pMask, size.x, size.y, 
+            hotSpot.x, hotSpot.y);
+    SDL_SetCursor(pCursor);
+    delete pData;
+    delete pMask;
+}
+
+void Player::setEventCapture(NodePtr pNode, int cursorID)
+{
     std::map<int, NodeWeakPtr>::iterator it = m_pEventCaptureNode.find(cursorID);
     if (it!=m_pEventCaptureNode.end()&&!it->second.expired()) {
         throw Exception(AVG_ERR_INVALID_CAPTURE, "setEventCapture called for '"
-                + pNode.lock()->getID() + "', but cursor already captured by '"
+                + pNode->getID() + "', but cursor already captured by '"
                 + it->second.lock()->getID() + "'.");
     } else {
         m_pEventCaptureNode[cursorID] = pNode;
     }
 }
 
-void Player::releaseEventCapture(int cursorID) {
+void Player::releaseEventCapture(int cursorID)
+{
     std::map<int, NodeWeakPtr>::iterator it = m_pEventCaptureNode.find(cursorID);
     if(it==m_pEventCaptureNode.end()||(it->second.expired()) ) {
         throw Exception(AVG_ERR_INVALID_CAPTURE,
@@ -537,12 +615,11 @@ void Player::releaseEventCapture(int cursorID) {
 
 }
 
-NodePtr Player::getElementByID (const std::string& id)
+NodePtr Player::getElementByID(const std::string& id)
 {
     if (m_IDMap.find(id) != m_IDMap.end()) {
         return m_IDMap.find(id)->second;
     } else {
-        AVG_TRACE(Logger::WARNING, "getElementByID(\"" << id << "\") failed.");
         return NodePtr();
     }
 }
@@ -575,25 +652,39 @@ void Player::removeNodeID(const std::string& id)
     }
 }
 
-AVGNodePtr Player::getRootNode ()
+AVGNodePtr Player::getRootNode()
 {
     return m_pRootNode;
 }
 
-void Player::registerFrameListener(IFrameListener* pListener)
+void Player::registerFrameEndListener(IFrameEndListener* pListener)
 {
-    m_Listeners.push_back(pListener);
+    m_FrameEndSignal.connect(pListener);
 }
 
-void Player::unregisterFrameListener(IFrameListener* pListener)
+void Player::unregisterFrameEndListener(IFrameEndListener* pListener)
 {
-    std::vector<IFrameListener*>::iterator it;
-    for (it=m_Listeners.begin(); it != m_Listeners.end(); ++it) {
-        if (*it == pListener) {
-            m_Listeners.erase(it);
-            break;
-        }
-    }
+    m_FrameEndSignal.disconnect(pListener);
+}
+
+void Player::registerPlaybackEndListener(IPlaybackEndListener* pListener)
+{
+    m_PlaybackEndSignal.connect(pListener);
+}
+
+void Player::unregisterPlaybackEndListener(IPlaybackEndListener* pListener)
+{
+    m_PlaybackEndSignal.disconnect(pListener);
+}
+
+void Player::registerPreRenderListener(IPreRenderListener* pListener)
+{
+    m_PreRenderSignal.connect(pListener);
+}
+
+void Player::unregisterPreRenderListener(IPreRenderListener* pListener)
+{
+    m_PreRenderSignal.disconnect(pListener);
 }
 
 string Player::getCurDirName()
@@ -612,18 +703,29 @@ std::string Player::getRootMediaDir()
     return sMediaDir;
 }
 
+const NodeDefinition& Player::getNodeDef(const std::string& sType)
+{
+    return m_NodeRegistry.getNodeDef(sType);
+}
+
 void Player::disablePython()
 {
     m_bPythonAvailable = false;
 }
 
+bool Player::isAudioEnabled() const
+{
+    return m_bAudioEnabled;
+}
+
 static ProfilingZone MainProfilingZone("Player - Total frame time");
 static ProfilingZone TimersProfilingZone("Player - handleTimers");
 static ProfilingZone EventsProfilingZone("Player - dispatch events");
+static ProfilingZone PreRenderProfilingZone("Player - PreRender");
 static ProfilingZone RenderProfilingZone("Player - render");
-static ProfilingZone ListenerProfilingZone("Player - listeners");
+static ProfilingZone FrameEndProfilingZone("Player - onFrameEnd");
 
-void Player::doFrame ()
+void Player::doFrame()
 {
     {
         ScopeTimer Timer(MainProfilingZone);
@@ -632,7 +734,6 @@ void Player::doFrame ()
             m_FrameTime = (long long)((m_NumFrames*1000.0)/m_FakeFPS);
         } else {
             m_FrameTime = m_pDisplayEngine->getDisplayTime();
-//            m_FrameTime = TimeSource::get()->getCurrentMillisecs()-m_PlayStartTime;
         }
         {
             ScopeTimer Timer(TimersProfilingZone);
@@ -643,24 +744,28 @@ void Player::doFrame ()
             m_pEventDispatcher->dispatch();
             sendFakeEvents();
         }
+        {
+            ScopeTimer Timer(PreRenderProfilingZone);
+            m_PreRenderSignal.emit();
+        }
         if (!m_bStopping) {
             ScopeTimer Timer(RenderProfilingZone);
             if (m_bPythonAvailable) {
                 Py_BEGIN_ALLOW_THREADS;
-                m_pDisplayEngine->render(m_pRootNode);
+                try {
+                    m_pDisplayEngine->render(m_pRootNode);
+                } catch(...) {
+                    Py_BLOCK_THREADS;
+                    throw;
+                }
                 Py_END_ALLOW_THREADS;
             } else {
                 m_pDisplayEngine->render(m_pRootNode);
             }
-            if (getGPUMemoryUsage() > m_MaxGPUMemUsed) {
-                m_MaxGPUMemUsed = getGPUMemoryUsage();
-            }
         }
         {
-            ScopeTimer Timer(ListenerProfilingZone);
-            for (unsigned int i=0; i<m_Listeners.size(); ++i) {
-                m_Listeners[i]->onFrameEnd();
-            }
+            ScopeTimer Timer(FrameEndProfilingZone);
+            m_FrameEndSignal.emit();
         }
     }
     if (m_pDisplayEngine->wasFrameLate()) {
@@ -679,7 +784,7 @@ void Player::doFrame ()
     ThreadProfiler::get()->reset();
 }
 
-double Player::getFramerate ()
+double Player::getFramerate()
 {
     if (!m_pDisplayEngine) {
         return m_DP.m_Framerate;
@@ -706,7 +811,8 @@ void Player::setGamma(double Red, double Green, double Blue)
     }
 }
 
-void Player::initConfig() {
+void Player::initConfig()
+{
     // Get data from config files.
     ConfigMgr* pMgr = ConfigMgr::get();
     
@@ -739,22 +845,7 @@ void Player::initConfig() {
     m_AP.m_OutputBufferSamples = atoi(pMgr->getOption("aud", "outputbuffersamples")->c_str());
 
     m_bUsePOW2Textures = pMgr->getBoolOption("scr", "usepow2textures", false);
-
-    const string * psYCbCrMode =pMgr->getOption("scr", "ycbcrmode");
-    if (psYCbCrMode == 0 || *psYCbCrMode == "shader") {
-        m_YCbCrMode = OGL_SHADER;
-    } else if (*psYCbCrMode == "mesa") {
-        m_YCbCrMode = OGL_MESA;
-    } else if (*psYCbCrMode == "apple") {
-        m_YCbCrMode = OGL_APPLE;
-    } else if (*psYCbCrMode == "none") {
-        m_YCbCrMode = OGL_NONE;
-    } else {
-        AVG_TRACE(Logger::ERROR, 
-                "avgrc: ycbcrmode must be shader, mesa, apple or none. Current value is " 
-                << *psYCbCrMode << ". Aborting." );
-        exit(-1);
-    }
+    m_bUseShaders = pMgr->getBoolOption("scr", "useshaders", true);
 
     const string * psVSyncMode =pMgr->getOption("scr", "vsyncmode");
     if (psVSyncMode == 0 || *psVSyncMode == "auto") {
@@ -787,19 +878,10 @@ void Player::initGraphics()
         AVG_TRACE(Logger::CONFIG, "  POW2 textures: " 
                 << (m_bUsePOW2Textures?"true":"false"));
         string sMode;
-        switch (m_YCbCrMode) {
-            case OGL_NONE:
-                AVG_TRACE(Logger::CONFIG, "  No YCbCr texture support.");
-                break;
-            case OGL_MESA:
-                AVG_TRACE(Logger::CONFIG, "  Mesa YCbCr texture support.");
-                break;
-            case OGL_APPLE:
-                AVG_TRACE(Logger::CONFIG, "  Apple YCbCr texture support.");
-                break;
-            case OGL_SHADER:
-                AVG_TRACE(Logger::CONFIG, "  Fragment shader YCbCr texture support.");
-                break;
+        if (m_bUseShaders) {
+            AVG_TRACE(Logger::CONFIG, "  Use shader support.");
+        } else {
+            AVG_TRACE(Logger::CONFIG, "  No shader support.");
         }
         AVG_TRACE(Logger::CONFIG, "  Use pixel buffers: " 
                 << (m_bUsePixelBuffers?"true":"false"));
@@ -827,7 +909,7 @@ void Player::initGraphics()
     SDLDisplayEngine * pSDLDisplayEngine = 
             dynamic_cast<SDLDisplayEngine*>(m_pDisplayEngine);
     if (pSDLDisplayEngine) {
-        pSDLDisplayEngine->setOGLOptions(m_bUsePOW2Textures, m_YCbCrMode, 
+        pSDLDisplayEngine->setOGLOptions(m_bUsePOW2Textures, m_bUseShaders, 
                 m_bUsePixelBuffers, m_MultiSampleSamples, m_VSyncMode);
     }
     m_pDisplayEngine->init(m_DP);
@@ -843,8 +925,22 @@ void Player::initAudio()
     m_pAudioEngine->play();
 }
 
+void Player::updateDTD()
+{
+    // Find and parse dtd.
+    registerDTDEntityLoader("avg.dtd", m_NodeRegistry.getDTD().c_str());
+    string sDTDFName = "avg.dtd";
+    m_dtd = xmlParseDTD(NULL, (const xmlChar*) sDTDFName.c_str());
+    if (!m_dtd) {
+        AVG_TRACE(Logger::WARNING, 
+                "DTD not found at " << sDTDFName << ". Not validating xml files.");
+    }
+    m_bDirtyDTD = false;
+}
+
 void Player::internalLoad(const string& sAVG)
 {
+    xmlDocPtr doc = 0;
     try {
         if (m_pRootNode) {
             cleanup();
@@ -853,18 +949,20 @@ void Player::internalLoad(const string& sAVG)
         m_pEventDispatcher = EventDispatcherPtr(new EventDispatcher);
 
         char szBuf[1024];
-        getcwd(szBuf, 1024);
-        m_CurDirName = string(szBuf)+"/";
+        char * pBuf = getcwd(szBuf, 1024);
+        m_CurDirName = string(pBuf)+"/";
         
         xmlPedanticParserDefault(1);
         xmlDoValidityCheckingDefaultValue =0;
 
-        xmlDocPtr doc;
         doc = xmlParseMemory(sAVG.c_str(), sAVG.length());
         if (!doc) {
             throw (Exception(AVG_ERR_XML_PARSE, ""));
         }
 
+        if (m_bDirtyDTD)
+            updateDTD();
+
         xmlValidCtxtPtr cvp = xmlNewValidCtxt();
         cvp->error = xmlParserValidityError;
         cvp->warning = xmlParserValidityWarning;
@@ -875,6 +973,10 @@ void Player::internalLoad(const string& sAVG)
         }
         xmlNodePtr xmlNode = xmlDocGetRootElement(doc);
         createNodeFromXml(doc, xmlNode, DivNodePtr());
+        if (!m_pRootNode) {
+            throw (Exception(AVG_ERR_XML_PARSE, 
+                    "Root node of an avg tree needs to be an <avg> node."));
+        }
         registerNode(m_pRootNode);
         m_DP.m_Height = int(m_pRootNode->getHeight());
         m_DP.m_Width = int(m_pRootNode->getWidth());
@@ -882,13 +984,20 @@ void Player::internalLoad(const string& sAVG)
         xmlFreeDoc(doc);
     } catch (Exception& ex) {
         AVG_TRACE(Logger::ERROR, ex.GetStr());
+        if (doc) {
+            xmlFreeDoc(doc);
+        }
         throw;
     } catch (Magick::Exception & ex) {
         AVG_TRACE(Logger::ERROR, ex.what());
+        if (doc) {
+            xmlFreeDoc(doc);
+        }
         throw;
     }
 }
 
+
 void Player::registerNode(NodePtr pNode)
 {
     addNodeID(pNode);    
@@ -900,19 +1009,34 @@ void Player::registerNode(NodePtr pNode)
     }
 }
 
-void Player::registerNodeType(NodeDefinition Def)
+void Player::registerNodeType(NodeDefinition Def, const char* pParentNames[])
 {
-    m_NodeFactory.registerNodeType(Def);
+    m_NodeRegistry.registerNodeType(Def);
+
+    if (pParentNames) {
+       string sChildArray[1];
+       sChildArray[0] = Def.getName();
+       vector<string> sChildren = vectorFromCArray(1, sChildArray);
+        const char **pCurrParentName = pParentNames;
+
+        while(*pCurrParentName) {
+            NodeDefinition nodeDefinition = m_NodeRegistry.getNodeDef(*pCurrParentName);
+            nodeDefinition.addChildren(sChildren);
+            m_NodeRegistry.updateNodeDefinition(nodeDefinition);
+            
+            ++pCurrParentName;
+        }
+    }
+    m_bDirtyDTD = true;
 }
 
 NodePtr Player::createNode(const string& sType, const boost::python::dict& PyDict)
 {
-    NodePtr pNode = m_NodeFactory.createNode(sType, PyDict, this);
-    pNode->setThis(pNode);
+    NodePtr pNode = m_NodeRegistry.createNode(sType, PyDict);
     return pNode;
 }
 
-NodePtr Player::createNodeFromXmlString (const string& sXML)
+NodePtr Player::createNodeFromXmlString(const string& sXML)
 {
     xmlPedanticParserDefault(1);
     xmlDoValidityCheckingDefaultValue =0;
@@ -925,6 +1049,9 @@ NodePtr Player::createNodeFromXmlString (const string& sXML)
     }
     NodePtr pNode = createNodeFromXml(doc, xmlDocGetRootElement(doc), DivNodePtr());
 
+    if (m_bDirtyDTD)
+        updateDTD();
+
     xmlValidCtxtPtr cvp = xmlNewValidCtxt();
     cvp->error = xmlParserValidityError;
     cvp->warning = xmlParserValidityWarning;
@@ -939,7 +1066,7 @@ NodePtr Player::createNodeFromXmlString (const string& sXML)
     return pNode;
 }
 
-NodePtr Player::createNodeFromXml (const xmlDocPtr xmlDoc, 
+NodePtr Player::createNodeFromXml(const xmlDocPtr xmlDoc, 
         const xmlNodePtr xmlNode, DivNodeWeakPtr pParent)
 {
     NodePtr curNode;
@@ -950,14 +1077,13 @@ NodePtr Player::createNodeFromXml (const xmlDocPtr xmlDoc,
         // Ignore whitespace & comments
         return NodePtr();
     }
-    curNode = m_NodeFactory.createNode(nodeType, xmlNode, this);
+    curNode = m_NodeRegistry.createNode(nodeType, xmlNode);
     if (!strcmp (nodeType, "words")) {
         // TODO: This is an end-run around the generic serialization mechanism
         // that will probably break at some point.
         string s = getXmlChildrenAsString(xmlDoc, xmlNode);
         boost::dynamic_pointer_cast<Words>(curNode)->setTextFromNodeValue(s);
     }
-    curNode->setThis(curNode);
 
     // If this is the root node, remember it.
     AVGNodePtr pRootNode = boost::dynamic_pointer_cast<AVGNode>(curNode);
@@ -967,14 +1093,13 @@ NodePtr Player::createNodeFromXml (const xmlDocPtr xmlDoc,
     }
 
     // If this is a container, recurse into children
-    DivNodePtr curDivNode = boost::dynamic_pointer_cast<DivNode>(curNode);
-    if (curDivNode) {
+    DivNodePtr curGroup = boost::dynamic_pointer_cast<DivNode>(curNode);
+    if (curGroup) {
         xmlNodePtr curXmlChild = xmlNode->xmlChildrenNode;
         while (curXmlChild) {
-            NodePtr curChild = createNodeFromXml (xmlDoc, curXmlChild, 
-                    curDivNode);
+            NodePtr curChild = createNodeFromXml(xmlDoc, curXmlChild, curGroup);
             if (curChild) {
-                curDivNode->appendChild(curChild);
+                curGroup->appendChild(curChild);
             }
             curXmlChild = curXmlChild->next;
         }
@@ -986,7 +1111,6 @@ void Player::handleTimers()
 {
     vector<Timeout *>::iterator it;
     m_bInHandleTimers = true;
-    vector<Timeout *> IntervalsFired;
    
     it = m_PendingTimeouts.begin();
     while (it != m_PendingTimeouts.end() && (*it)->IsReady(getFrameTime()) && !m_bStopping)
@@ -1143,7 +1267,10 @@ void Player::handleCursorEvent(CursorEventPtr pEvent, bool bOnlyCheckCursorOver)
                 if (pNodeEvent->getType() != Event::CURSORMOTION) {
                     pNodeEvent->trace();
                 }
-                pNode->handleEvent(pNodeEvent);
+                if (pNode->handleEvent(pNodeEvent) == true) {
+                    // stop bubbling
+                    break;
+                }
             }
         }
     }
@@ -1182,7 +1309,7 @@ vector<NodeWeakPtr> Player::getElementsByPos(const DPoint& pos) const
     return Elements;
 }
 
-DisplayEngine * Player::getDisplayEngine() const 
+DisplayEngine * Player::getDisplayEngine() const
 {
     return m_pDisplayEngine;
 }
@@ -1210,8 +1337,7 @@ double Player::getVolume() const
     return m_Volume;
 }
 
-void Player::sendOver(const CursorEventPtr pOtherEvent, Event::Type Type, 
-                NodePtr pNode)
+void Player::sendOver(const CursorEventPtr pOtherEvent, Event::Type Type, NodePtr pNode)
 {
     if (pNode) {
         EventPtr pNewEvent = pOtherEvent->cloneAs(Type);
@@ -1230,7 +1356,7 @@ void Player::cleanup()
     m_PendingTimeouts.clear();
     Profiler::get().dumpStatistics();
     if (m_pRootNode) {
-        m_pRootNode->disconnect();
+        m_pRootNode->disconnect(true);
     }
     m_pRootNode = AVGNodePtr();
 
@@ -1238,6 +1364,7 @@ void Player::cleanup()
         delete m_pTracker;
         m_pTracker = 0;
     }
+    m_PlaybackEndSignal.emit();
 
     if (m_pDisplayEngine) {
         m_pDisplayEngine->deinitRender();
@@ -1246,12 +1373,10 @@ void Player::cleanup()
     if (m_pAudioEngine) {
         m_pAudioEngine->teardown();
     }
-    AVG_TRACE(Logger::PROFILE, "Max. GPU memory used: " << m_MaxGPUMemUsed/1024 << "k");
     
     m_IDMap.clear();
     m_pEventDispatcher = EventDispatcherPtr();
     m_MouseState = MouseState();
-    initConfig();
     m_FrameTime = 0;
     m_bIsPlaying = false;
 }
@@ -1277,15 +1402,19 @@ void Player::removeTimeout(Timeout* pTimeout)
     m_PendingTimeouts.erase(it);
 }
 
-long long Player::getGPUMemoryUsage()
+void Player::setPluginPath(const string& newPath)
 {
-    if (m_pDisplayEngine) {
-        return m_pDisplayEngine->getGPUMemoryUsage();
-    } else {
-        return 0;
-    }
+    PluginManager::get().setSearchPath(newPath);
 }
 
+string Player::getPluginPath() const 
+{
+    return  PluginManager::get().getSearchPath();
+}
 
+void Player::loadPlugin(const std::string& name)
+{
+    PluginManager::get().loadPlugin(name);
+}
 
 }
