@@ -22,62 +22,81 @@
 #ifndef _RasterNode_H_
 #define _RasterNode_H_
 
-#include "Node.h"
+#include "../api.h"
+#include "AreaNode.h"
 #include "DisplayEngine.h"
-#include "OGLSurface.h"
+#include "OGLTiledSurface.h"
 
 #include "../avgconfigwrapper.h"
 #include "../base/Point.h"
+#include "../base/UTF8String.h"
 
 #include <string>
 
 namespace avg {
 
-class RasterNode: public Node
+class AVG_API RasterNode: public AreaNode
 {
     public:
-        static NodeDefinition getNodeDefinition();
+        static NodeDefinition createDefinition();
         
         virtual ~RasterNode ();
-        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine, AudioEngine * pAudioEngine);
+        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine, 
+                AudioEngine * pAudioEngine);
         virtual void setArgs(const ArgList& Args);
-        virtual void disconnect();
+        virtual void disconnect(bool bKill);
+        virtual void checkReload();
         
         // Warping support.
         VertexGrid getOrigVertexCoords();
         VertexGrid getWarpedVertexCoords();
         void setWarpedVertexCoords(const VertexGrid& Grid);
 
-        int getMaxTileWidth() const
-        {
-            return m_MaxTileSize.x;
-        }
-
-        int getMaxTileHeight() const
-        {
-            return m_MaxTileSize.y;
-        }
+        int getMaxTileWidth() const;
+        int getMaxTileHeight() const;
+        bool getMipmap() const;
         
         const std::string& getBlendModeStr() const;
         void setBlendModeStr(const std::string& sBlendMode);
-        OGLSurface * getOGLSurface();
         DisplayEngine::BlendMode getBlendMode() const;
-        virtual std::string getTypeStr ();
-        NodePtr getElementByPos (const DPoint & pos);
 
-        virtual Bitmap* getBitmap();
+        const UTF8String& getMaskHRef() const;
+        void setMaskHRef(const UTF8String& href);
+
+        const DPoint& getMaskPos() const;
+        void setMaskPos(const DPoint& pos);
+
+        const DPoint& getMaskSize() const;
+        void setMaskSize(const DPoint& size);
+
+        NodePtr getElementByPos(const DPoint & pos);
+
+        virtual BitmapPtr getBitmap();
         
     protected:
-        RasterNode (Player * pPlayer);
-        ISurface * getSurface();
- 
+        RasterNode();
+        virtual OGLTiledSurface * getSurface();
+        const MaterialInfo& getMaterial() const;
+        void calcMaskPos();
+
     private:
+        void setMaterial(const MaterialInfo& material);
+        virtual void setMaterialMask(MaterialInfo& material, const DPoint& pos, 
+                const DPoint& size, const DPoint& mediaSize);
+        void downloadMask();
         void checkDisplayAvailable(std::string sMsg);
-        ISurface * m_pSurface;
+        OGLTiledSurface * m_pSurface;
         
         IntPoint m_MaxTileSize;
         std::string m_sBlendMode;
         DisplayEngine::BlendMode m_BlendMode;
+        MaterialInfo m_Material;
+
+        UTF8String m_sMaskHref;
+        std::string m_sMaskFilename;
+        BitmapPtr m_pMaskBmp;
+        DPoint m_MaskPos;
+        DPoint m_MaskSize;
 };
 
 }
