@@ -28,13 +28,12 @@
 #include "Player.h"
 #include "Node.h"
 #include "AVGNode.h"
+#include "Shape.h"
 
 #include "Event.h"
 #include "MouseEvent.h"
 #include "KeyEvent.h"
 
-#include "OGLSurface.h"
-
 #include "../base/MathHelper.h"
 #include "../base/Exception.h"
 #include "../base/Logger.h"
@@ -45,11 +44,12 @@
 #include "../graphics/Filterflip.h"
 #include "../graphics/Filterfliprgb.h"
 
+#include <SDL/SDL.h>
+
 #ifdef __APPLE__
 #include <ApplicationServices/ApplicationServices.h>
 #endif
 #ifdef linux
-#include <X11/Xlib.h>
 #include <X11/extensions/xf86vmode.h>
 #endif
 
@@ -67,7 +67,6 @@
 #include <sstream>
 
 #ifdef _WIN32
-#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #else
 #include <unistd.h>
@@ -79,86 +78,14 @@ using namespace std;
 
 namespace avg {
 
-#ifdef linux
-
-typedef enum {
-    DRM_VBLANK_ABSOLUTE = 0x0,  /**< Wait for specific vblank sequence number */
-    DRM_VBLANK_RELATIVE = 0x1,  /**< Wait for given number of vblanks */
-    DRM_VBLANK_NEXTONMISS = 0x10000000, /**< If missed, wait for next vblank */
-    DRM_VBLANK_SECONDARY = 0x20000000,  /**< Secondary display controller */
-    DRM_VBLANK_SIGNAL   = 0x40000000    /* Send signal instead of blocking */
-} drmVBlankSeqType;
-
-struct drm_wait_vblank_request {
-    int type;
-    unsigned int sequence;
-    unsigned long signal;
-};
-
-struct drm_wait_vblank_reply {
-    int type;
-    unsigned int sequence;
-    long tval_sec;
-    long tval_usec;
-};
-
-typedef union drm_wait_vblank {
-    struct drm_wait_vblank_request request;
-    struct drm_wait_vblank_reply reply;
-} drm_wait_vblank_t;
-
-#define DRM_IOCTL_BASE                  'd'
-#define DRM_IOWR(nr,type)               _IOWR(DRM_IOCTL_BASE,nr,type)
-
-#define DRM_IOCTL_WAIT_VBLANK           DRM_IOWR(0x3a, drm_wait_vblank_t)
-
-static int drmWaitVBlank(int fd, drm_wait_vblank_t *vbl)
-{
-    static bool bUseSecondaryPipe = false;
-    int ret;
-    int rc;
-
-    do {
-       ret = ioctl(fd, DRM_IOCTL_WAIT_VBLANK, vbl);
-       if (bUseSecondaryPipe) {
-           vbl->request.type |= DRM_VBLANK_SECONDARY;
-       }
-       vbl->request.type &= ~DRM_VBLANK_RELATIVE;
-       rc = errno;
-    } while (ret && rc == EINTR);
-    if (!bUseSecondaryPipe && rc == EBUSY) {
-        // rc 16 == device or resource busy
-        AVG_TRACE(Logger::CONFIG, 
-                "Using secondary pipe for vblank.");
-        bUseSecondaryPipe = true;
-        rc = 0;
-    }
-
-    return rc;
-}
-#endif
-
 double SDLDisplayEngine::s_RefreshRate = 0.0;
 
-void dumpSDLGLParams() {
-    int value;
-    cerr << "SDL display parameters used: "<< endl;
-    SDL_GL_GetAttribute(SDL_GL_RED_SIZE, &value);
-    fprintf(stderr,"  SDL_GL_RED_SIZE = %d\n",value);
-    SDL_GL_GetAttribute(SDL_GL_GREEN_SIZE,  &value);
-    fprintf(stderr,"  SDL_GL_GREEN_SIZE = %d\n",value);
-    SDL_GL_GetAttribute(SDL_GL_BLUE_SIZE , &value);
-    fprintf(stderr,"  SDL_GL_BLUE_SIZE = %d\n",value);
-    SDL_GL_GetAttribute(SDL_GL_ALPHA_SIZE , &value);
-    fprintf(stderr,"  SDL_GL_ALPHA_SIZE = %d\n",value);
-    SDL_GL_GetAttribute(SDL_GL_DEPTH_SIZE , &value);
-    fprintf(stderr,"  SDL_GL_DEPTH_SIZE = %d\n",value);
-    SDL_GL_GetAttribute(SDL_GL_DOUBLEBUFFER , &value);
-    fprintf(stderr,"  SDL_GL_DOUBLEBUFFER = %d\n",value);
-    SDL_GL_GetAttribute(SDL_GL_BUFFER_SIZE , &value);
-    fprintf(stderr,"  SDL_GL_BUFFER_SIZE = %d\n",value);
-    SDL_GL_GetAttribute(SDL_GL_STENCIL_SIZE , &value);
-    fprintf(stderr,"  SDL_GL_STENCIL_SIZE = %d\n",value);
+void safeSetAttribute( SDL_GLattr attr, int value) 
+{
+    int err = SDL_GL_SetAttribute(attr, value);
+    if (err == -1) {
+        throw Exception(AVG_ERR_VIDEO_GENERAL, SDL_GetError());
+    }
 }
 
 SDLDisplayEngine::SDLDisplayEngine()
@@ -168,10 +95,8 @@ SDLDisplayEngine::SDLDisplayEngine()
       m_pScreen(0),
       m_VBMethod(VB_NONE),
       m_VBMod(0),
-      m_dri_fd(0),
       m_bMouseOverApp(true),
       m_LastMousePos(-1, -1),
-      m_TextureMode(0),
       m_MaxTexSize(0),
       m_bCheckedMemoryMode(false)
 {
@@ -187,13 +112,13 @@ SDLDisplayEngine::SDLDisplayEngine()
         exit(-1);
     }
     initTranslationTable();
-    SDL_EnableUNICODE(true);
 }
 
 SDLDisplayEngine::~SDLDisplayEngine()
 {
+#ifndef _WIN32
     SDL_QuitSubSystem(SDL_INIT_VIDEO);
-    assert(m_pSurfaces.size() == 0);
+#endif
 }
 
 void SDLDisplayEngine::init(const DisplayParams& DP) 
@@ -261,7 +186,7 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
                     "in SDLDisplayEngine::init()");
             exit(-1);
     }
-    safeSetAttribute(SDL_GL_DEPTH_SIZE, 0);
+    safeSetAttribute(SDL_GL_DEPTH_SIZE, 24);
     safeSetAttribute(SDL_GL_STENCIL_SIZE, 8);
     safeSetAttribute(SDL_GL_DOUBLEBUFFER, 1);
     if (m_MultiSampleSamples > 1) {
@@ -296,16 +221,29 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glDisable(GL_DEPTH_TEST)");
     glEnable(GL_STENCIL_TEST);
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glEnable(GL_STENCIL_TEST)");
-    int TexMode = getTextureMode();
-    glEnable(TexMode);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glEnable(TexMode);");
-    if (m_MultiSampleSamples > 1) {
-        glEnable(GL_MULTISAMPLE);
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glEnable(GL_MULTISAMPLE);");
+    initTextureMode();
+    if (!queryOGLExtension("GL_ARB_multisample")) {
+        m_MultiSampleSamples = 1;
     } else {
-        glDisable(GL_MULTISAMPLE);
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glDisable(GL_MULTISAMPLE);");
+        if (m_MultiSampleSamples > 1) {
+            glEnable(GL_MULTISAMPLE);
+            OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glEnable(GL_MULTISAMPLE);");
+        } else {
+            glDisable(GL_MULTISAMPLE);
+            OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glDisable(GL_MULTISAMPLE);");
+        }
     }
+    if (!queryOGLExtension("GL_ARB_vertex_buffer_object")) {
+        throw Exception(AVG_ERR_UNSUPPORTED,
+            "Graphics driver lacks vertex buffer support, unable to initialize graphics.");
+    }
+    m_bEnableTexture=false;
+    enableTexture(true);
+    m_bEnableGLColorArray=true;
+    enableGLColorArray(false);
+    m_BlendMode = BLEND_ADD;
+    setBlendMode(BLEND_BLEND);
+
     glEnableClientState(GL_VERTEX_ARRAY);
     glEnableClientState(GL_TEXTURE_COORD_ARRAY);
     setGamma(DP.m_Gamma[0], DP.m_Gamma[1], DP.m_Gamma[2]);
@@ -316,17 +254,17 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
         setFramerate(DP.m_Framerate);
     }
 
-    checkYCbCrSupport();
+    checkShaderSupport();
 
     m_Width = DP.m_Width;
     m_Height = DP.m_Height;
-    initInput();
     // SDL sets up a signal handler we really don't want.
     signal(SIGSEGV, SIG_DFL);
     logConfig();
 
-//    dumpSDLGLParams();
     m_bEnableCrop = false;
+
+    SDL_EnableUNICODE(1);
 }
 
 #ifdef _WIN32
@@ -334,10 +272,16 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
 #endif
 void SDLDisplayEngine::teardown()
 {
-    if (m_VBMethod == VB_DRI) {
-        close(m_dri_fd);
-    }    
-    m_pScreen = 0;
+    if (m_pScreen) {
+        SDL_SetGamma(1.0, 1.0, 1.0);
+#ifdef linux
+        // Workaround for broken mouse cursor on exit under Ubuntu 8.04.
+        SDL_ShowCursor(SDL_ENABLE);
+//        SDL_SetVideoMode(m_WindowWidth, m_WindowHeight, 24, 0);
+#endif
+        m_pScreen = 0;
+    }
+    VertexArray::deleteBufferCache();
 }
 
 double SDLDisplayEngine::getRefreshRate() 
@@ -359,6 +303,15 @@ void SDLDisplayEngine::setGamma(double Red, double Green, double Blue)
     }
 }
 
+void SDLDisplayEngine::setMousePos(const IntPoint& pos)
+{
+    SDL_WarpMouse(pos.x, pos.y);
+}
+
+int SDLDisplayEngine::getKeyModifierState() const
+{
+    return SDL_GetModState();
+}
 
 void SDLDisplayEngine::logConfig() 
 {
@@ -366,28 +319,15 @@ void SDLDisplayEngine::logConfig()
     AVG_TRACE(Logger::CONFIG, "  OpenGL version: " << glGetString(GL_VERSION));
     AVG_TRACE(Logger::CONFIG, "  OpenGL vendor: " << glGetString(GL_VENDOR));
     AVG_TRACE(Logger::CONFIG, "  OpenGL renderer: " << glGetString(GL_RENDERER));
-    switch (m_YCbCrMode) {
-        case OGL_NONE:
-            AVG_TRACE(Logger::CONFIG, "  YCbCr texture support not enabled.");
-            break;
-        case OGL_MESA:
-            AVG_TRACE(Logger::CONFIG, "  Using Mesa YCbCr texture support.");
-            break;
-        case OGL_APPLE:
-            AVG_TRACE(Logger::CONFIG, "  Using Apple YCbCr texture support.");
-            break;
-        case OGL_SHADER:
-            AVG_TRACE(Logger::CONFIG, "  Using fragment shader YCbCr texture support.");
-            break;
+    if (m_bUseShaders) {
+        AVG_TRACE(Logger::CONFIG, "  Using shader support.");
+    } else {
+        AVG_TRACE(Logger::CONFIG, "  Shader support not enabled.");
     }
-    switch(m_TextureMode) {
-        case GL_TEXTURE_2D:
-            AVG_TRACE(Logger::CONFIG, "  Using power of 2 textures.");
-            break;
-        case GL_TEXTURE_RECTANGLE_NV:
-            AVG_TRACE(Logger::CONFIG, 
-                    "  Using NVidia texture rectangle extension.");
-            break;
+    if (m_bUsePOTTextures) {
+        AVG_TRACE(Logger::CONFIG, "  Using power of 2 textures.");
+    } else {
+        AVG_TRACE(Logger::CONFIG, "  Using non-power of 2 textures.");
     }
     if (m_MultiSampleSamples == 1) {
         AVG_TRACE(Logger::CONFIG, "  Not using multisampling.");
@@ -422,6 +362,9 @@ void SDLDisplayEngine::render(AVGNodePtr pRootNode)
     glClear(GL_STENCIL_BUFFER_BIT);
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
             "SDLDisplayEngine::render::glClear(GL_STENCIL_BUFFER_BIT)");
+    glClear(GL_DEPTH_BUFFER_BIT);
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
+            "SDLDisplayEngine::render::glClear(GL_DEPTH_BUFFER_BIT)");
     glViewport(0, 0, m_WindowWidth, m_WindowHeight);
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
             "SDLDisplayEngine::render: glViewport()");
@@ -446,6 +389,17 @@ void SDLDisplayEngine::render(AVGNodePtr pRootNode)
     {
         ScopeTimer Timer(RootRenderProfilingZone);
         pRootNode->maybeRender(rc);
+
+        Shape * pShape = new Shape("", MaterialInfo(GL_REPEAT, GL_CLAMP_TO_EDGE, false));
+        pShape->moveToGPU(this);
+        VertexArrayPtr pVA = pShape->getVertexArray();
+        pVA->changeSize(8000, 8000);  // Breaks at 1000 nodes.
+        pRootNode->renderOutlines(pVA, Pixel32(0,0,0,0));
+        if (pVA->getCurVert() != 0) {
+            pVA->update();
+            pShape->draw();
+        }
+        delete pShape;
     }
     frameWait();
     swapBuffers();
@@ -474,11 +428,6 @@ void SDLDisplayEngine::popClipRect()
     m_ClipRects.pop_back();
 }
 
-const DRect& SDLDisplayEngine::getClipRect() 
-{
-    return m_ClipRects.back();
-}
-
 void SDLDisplayEngine::pushTransform(const DPoint& translate, double angle, const DPoint& pivot)
 {
     glPushMatrix();
@@ -497,24 +446,6 @@ void SDLDisplayEngine::popTransform()
     glPopMatrix();
 }
 
-void SDLDisplayEngine::blt32(ISurface * pSurface, const DPoint& DestSize, 
-        double opacity, BlendMode Mode)
-{
-    OGLSurface * pOGLSurface = dynamic_cast<OGLSurface*>(pSurface);
-    glColor4d(1.0, 1.0, 1.0, opacity);
-    pOGLSurface->blt(DestSize, Mode);
-}
-
-void SDLDisplayEngine::blta8(ISurface * pSurface, 
-        const DPoint& DestSize, double opacity, 
-        const Pixel32& color, BlendMode Mode)
-{
-    OGLSurface * pOGLSurface = dynamic_cast<OGLSurface*>(pSurface);
-    glColor4d(double(color.getR())/256, double(color.getG())/256, 
-            double(color.getB())/256, opacity);
-    pOGLSurface->blt(DestSize, Mode);
-}
-
 void SDLDisplayEngine::clip(bool forward)
 {
     if (m_bEnableCrop && !m_ClipRects.empty()) {
@@ -569,46 +500,17 @@ void SDLDisplayEngine::swapBuffers()
     AVG_TRACE(Logger::BLTS, "GL SwapBuffers");
 }
 
-ISurface * SDLDisplayEngine::createSurface()
+bool SDLDisplayEngine::isUsingShaders() const
 {
-    OGLSurface * pOGLSurface = new OGLSurface(this);
-    m_pSurfaces.push_back(pOGLSurface);
-
-    return pOGLSurface;
+    return m_bUseShaders;
 }
 
-void SDLDisplayEngine::surfaceChanged(ISurface * pSurface) 
+OGLShaderPtr SDLDisplayEngine::getShader()
 {
-    OGLSurface * pOGLSurface = dynamic_cast<OGLSurface *>(pSurface);
-    pOGLSurface->bind();
+    return m_pShader;
 }
 
-bool SDLDisplayEngine::supportsBpp(int bpp)
-{
-    return (bpp == 24 || bpp == 32);
-}
-
-bool SDLDisplayEngine::hasRGBOrdering()
-{
-    return false;
-}
-
-YCbCrMode SDLDisplayEngine::getYCbCrMode()
-{
-    return m_YCbCrMode;
-}
-
-OGLShaderPtr SDLDisplayEngine::getYCbCr420pShader()
-{
-    return m_pYCbCrShader;
-}
-
-OGLShaderPtr SDLDisplayEngine::getYCbCrJ420pShader()
-{
-    return m_pYCbCrJShader;
-}
-
-void SDLDisplayEngine::showCursor (bool bShow)
+void SDLDisplayEngine::showCursor(bool bShow)
 {
 #ifdef _WIN32
     ShowCursor(bShow);
@@ -621,10 +523,16 @@ void SDLDisplayEngine::showCursor (bool bShow)
 #endif
 }
 
-BitmapPtr SDLDisplayEngine::screenshot ()
+BitmapPtr SDLDisplayEngine::screenshot()
 {
+    
     BitmapPtr pBmp (new Bitmap(IntPoint(m_Width, m_Height), R8G8B8X8, "screenshot"));
-    glReadBuffer(GL_BACK);
+    if (isParallels()) {
+        // Workaround for buggy GL_FRONT on virtual machines running under parallels.
+        glReadBuffer(GL_BACK);
+    } else {
+        glReadBuffer(GL_FRONT);
+    }
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "SDLDisplayEngine::screenshot:glReadBuffer()");
     glReadPixels(0, 0, m_Width, m_Height, GL_RGBA, GL_UNSIGNED_BYTE, 
             pBmp->getPixels());
@@ -649,95 +557,102 @@ int SDLDisplayEngine::getBPP()
     return m_bpp;
 }
     
-void SDLDisplayEngine::checkYCbCrSupport()
+void SDLDisplayEngine::checkShaderSupport()
 {
-    m_YCbCrMode = OGL_NONE;
-    if (queryOGLExtension("GL_ARB_fragment_shader") &&
-        queryOGLExtension("GL_ARB_texture_rectangle") &&
-        getMemoryModeSupported() == PBO &&
-        m_DesiredYCbCrMode == OGL_SHADER && 
-        getTextureMode() != GL_TEXTURE_2D
-       )
-    {
-        m_YCbCrMode = OGL_SHADER;
-        string sProgramInit =
-            "#extension GL_ARB_texture_rectangle : enable\n"
-            
-            "uniform sampler2DRect YTexture;\n"
-            "uniform sampler2DRect CbTexture;\n"
-            "uniform sampler2DRect CrTexture;\n"
-            "\n";
+    m_bUseShaders = (queryOGLExtension("GL_ARB_fragment_shader") && 
+            getMemoryModeSupported() == PBO &&
+            !m_bUsePOTTextures && m_bShouldUseShaders);
+    if (m_bUseShaders) {
         string sProgram =
-            sProgramInit + 
+            "uniform sampler2D texture;\n"
+            "uniform sampler2D yTexture;\n"
+            "uniform sampler2D cbTexture;\n"
+            "uniform sampler2D crTexture;\n"
+            "uniform sampler2D maskTexture;\n"
+            "uniform int colorModel;  // 0=rgb, 1=ycbcr, 2=ycbcrj, 3=greyscale\n"
+            "uniform bool bUseMask;\n"
+            "uniform vec2 maskPos;\n"
+            "uniform vec2 maskSize;\n"
+            "\n"
+            "vec4 convertYCbCr()\n"
+            "{\n"
+            "    vec3 ycbcr;\n"
+            "    ycbcr.r = texture2D(texture, gl_TexCoord[0].st).a-0.0625;\n"
+            "    ycbcr.g = texture2D(cbTexture, (gl_TexCoord[0].st)).a-0.5;\n"
+            "    ycbcr.b = texture2D(crTexture, (gl_TexCoord[0].st)).a-0.5;\n"
+            "    vec3 rgb;\n"
+            "    rgb = ycbcr*mat3(1.16,  0.0,   1.60,\n"
+            "                     1.16, -0.39, -0.81,\n"
+            "                     1.16,  2.01,  0.0 );\n"
+            "    return vec4(rgb, gl_Color.a);\n"
+            "}\n"
+            "\n"
+            "vec4 convertYCbCrJ()\n"
+            "{\n"
+            "    vec3 ycbcr;\n"
+            "    ycbcr.r = texture2D(texture, gl_TexCoord[0].st).a;\n"
+            "    ycbcr.g = texture2D(cbTexture, (gl_TexCoord[0].st)).a-0.5;\n"
+            "    ycbcr.b = texture2D(crTexture, (gl_TexCoord[0].st)).a-0.5;\n"
+            "    vec3 rgb;\n"
+            "    rgb = ycbcr*mat3(1,  0.0  , 1.40,\n"
+            "                     1, -0.34, -0.71,\n"
+            "                     1,  1.77,  0.0 );\n"
+            "    return vec4(rgb, gl_Color.a);\n"
+            "}\n"
+            "\n"
             "void main(void)\n"
             "{\n"
-            "  vec3 YCbCr;\n"
-            "  YCbCr.r = texture2DRect(YTexture, gl_TexCoord[0].st).a-0.0625;\n"
-            "  YCbCr.g = texture2DRect(CbTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
-            "  YCbCr.b = texture2DRect(CrTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
-            "  vec3 RGB;"
-            "  RGB = YCbCr*mat3(1.16,  0.0,   1.60,\n"
-            "                   1.16, -0.39, -0.81,\n"
-            "                   1.16,  2.01,  0.0 );\n"
-            "  gl_FragColor = vec4(RGB,gl_Color.a);\n"
-            "}\n"
-            ;
-        m_pYCbCrShader = OGLShaderPtr(new OGLShader(sProgram));
+            "    vec4 rgba;\n"
+            "    if (colorModel == 0) {\n"
+            "        rgba = texture2D(texture, gl_TexCoord[0].st);\n"
+            "        rgba.a *= gl_Color.a;\n"
+            "    } else if (colorModel == 1) {\n"
+            "        rgba = convertYCbCr();\n"
+            "    } else if (colorModel == 2) {\n"
+            "        rgba = convertYCbCrJ();\n"
+            "    } else if (colorModel == 3) {\n"
+            "        rgba = gl_Color;\n"
+            "        rgba.a *= texture2D(texture, gl_TexCoord[0].st).a;\n"
+            "    } else {\n"
+            "        rgba = vec4(1,1,1,1);\n"
+            "    }\n"
+            "    if (bUseMask) {\n"
+            "        rgba.a *= texture2D(maskTexture,\n"
+            "               (gl_TexCoord[0].st/maskSize)-maskPos).a;\n"
+            "    }\n"
+            "    gl_FragColor = rgba;\n"
+            "}\n";
 
-        sProgram =
-            sProgramInit + 
-            "void main(void)\n"
-            "{\n"
-            "  vec3 YCbCr;\n"
-            "  YCbCr.r = texture2DRect(YTexture, gl_TexCoord[0].st).a;\n"
-            "  YCbCr.g = texture2DRect(CbTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
-            "  YCbCr.b = texture2DRect(CrTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
-            "  vec3 RGB;"
-            "  RGB = YCbCr*mat3(1,  0.0  , 1.40,\n"
-            "                   1, -0.34, -0.71,\n"
-            "                   1,  1.77,  0.0 );\n"
-            "  gl_FragColor = vec4(RGB,gl_Color.a);\n"
-            "}\n"
-            ;
-        m_pYCbCrJShader = OGLShaderPtr(new OGLShader(sProgram));
-
-    } else if (queryOGLExtension("GL_MESA_ycbcr_texture") &&
-            (m_DesiredYCbCrMode == OGL_SHADER || m_DesiredYCbCrMode == OGL_MESA))
-    {
-        m_YCbCrMode = OGL_MESA;
-    } else if (queryOGLExtension("GL_APPLE_ycbcr_422") &&
-            (m_DesiredYCbCrMode == OGL_SHADER || m_DesiredYCbCrMode == OGL_APPLE))
-    {
-        m_YCbCrMode = OGL_APPLE;
+        m_pShader = OGLShaderPtr(new OGLShader(sProgram));
     }
 }
 
-bool SDLDisplayEngine::initVBlank(int rate) {
-    
+void SDLDisplayEngine::initMacVBlank(int rate)
+{
+#ifdef __APPLE__
+    CGLContextObj context = CGLGetCurrentContext();
+    assert (context);
+    if (rate > 1) {
+        AVG_TRACE(Logger::WARNING,
+                "VBlank rate set to " << rate 
+                << " but Mac OS X only supports 1. Assuming 1.");
+    }
+#if MAC_OS_X_VERSION_10_5
+    const GLint l = 1;
+#else
+    const long l = 1;
+#endif
+    CGLError err = CGLSetParameter(context, kCGLCPSwapInterval, &l);
+    assert(!err);
+#endif
+}
+
+bool SDLDisplayEngine::initVBlank(int rate) 
+{
     if (rate > 0 && m_DesiredVSyncMode != VSYNC_NONE) {
 #ifdef __APPLE__
-        CGLContextObj Context = CGLGetCurrentContext();
-        if (Context == 0) {
-            AVG_TRACE(Logger::WARNING,
-                    "Mac VBlank setup failed in CGLGetCurrentContext().");
-        }
-        if (rate != 1) {
-            AVG_TRACE(Logger::WARNING,
-                    "VBlank rate set to " << rate << " but Mac OS X only supports 1. Assuming 1.");
-        }
-#if MAC_OS_X_VERSION_10_5
-        const GLint l = 1;
-#else
-        const long l = 1;
-#endif
-        CGLError err = CGLSetParameter(Context, kCGLCPSwapInterval, &l);
+        initMacVBlank(rate);
         m_VBMethod = VB_APPLE;
-        if (err) {
-            AVG_TRACE(Logger::WARNING,
-                    "Mac VBlank setup failed with error code " << 
-                    CGLErrorString(err) << "(" << err << ").");
-            m_VBMethod = VB_NONE;
-        }
 #elif defined _WIN32
         if (queryOGLExtension("WGL_EXT_swap_control")) {
             glproc::SwapIntervalEXT(rate);
@@ -754,28 +669,30 @@ bool SDLDisplayEngine::initVBlank(int rate) {
                     "__GL_SYNC_TO_VBLANK set. This interferes with libavg vblank handling.");
             m_VBMethod = VB_NONE;
         } else {
-            string sVendor = (const char *)(glGetString(GL_VENDOR));
-            if (sVendor.find("VIA Technology") != string::npos || 
-                    !queryGLXExtension("GLX_SGI_video_sync") ||
-                    m_DesiredVSyncMode == VSYNC_DRI)
-            {
-                m_dri_fd = open("/dev/dri/card0", O_RDWR);
-                if (m_dri_fd < 0)
-                {
-                    AVG_TRACE(Logger::WARNING, 
-                            "Could not open /dev/dri/card0 for vblank. Reason: "
-                            <<strerror(errno));
-                    m_VBMethod = VB_NONE;
-                } else {
-                    m_VBMethod = VB_DRI;
-                }
-            } else {
-                m_VBMethod = VB_SGI;
-                m_bFirstVBFrame = true;
-            }
+            m_VBMethod = VB_SGI;
+            glproc::SwapIntervalSGI(rate);
+
+            m_bFirstVBFrame = true;
         }
 #endif
     } else {
+        switch (m_VBMethod) {
+            case VB_APPLE:
+                initMacVBlank(0);
+                break;
+            case VB_WIN:
+#ifdef _WIN32
+                glproc::SwapIntervalEXT(0);
+#endif
+                break;
+            case VB_SGI:
+#ifdef linux            
+                glproc::SwapIntervalSGI(rate);
+#endif
+                break;
+            default:
+                break;
+        }
         m_VBMethod = VB_NONE;
     }
     switch(m_VBMethod) {
@@ -783,9 +700,6 @@ bool SDLDisplayEngine::initVBlank(int rate) {
             AVG_TRACE(Logger::CONFIG, 
                     "  Using SGI OpenGL extension for vertical blank support.");
             break;
-        case VB_DRI:
-            AVG_TRACE(Logger::CONFIG, "  Using DRI vertical blank support.");
-            break;
         case VB_APPLE:
             AVG_TRACE(Logger::CONFIG, "  Using Apple GL vertical blank support.");
             break;
@@ -803,56 +717,7 @@ bool SDLDisplayEngine::initVBlank(int rate) {
 
 bool SDLDisplayEngine::vbWait(int rate) {
     switch(m_VBMethod) {
-#ifdef linux
-        case VB_SGI: {
-                unsigned int count;
-                int err = glproc::WaitVideoSyncSGI(rate, m_VBMod, &count);
-                OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
-                        "VBlank::glXWaitVideoSyncSGI");
-                if (err) {
-                    AVG_TRACE(Logger::ERROR, "glXWaitVideoSyncSGI returned " 
-                            << err << ".");
-                    AVG_TRACE(Logger::ERROR, "Rate was " << rate 
-                            << ", Mod was " << m_VBMod);
-                    AVG_TRACE(Logger::ERROR, "Aborting.");
-                    assert(false);
-                }
-                m_VBMod = count % rate;
-                bool bMissed;
-                if (!m_bFirstVBFrame && int(count) != m_LastVBCount+rate) {
-                    AVG_TRACE(Logger::PROFILE_LATEFRAMES, count-m_LastVBCount
-                            << " VBlank intervals missed, should be " 
-                            << rate);
-                    bMissed = true;
-                } else {
-                    bMissed = false;
-                }
-                m_LastVBCount = count;
-                m_bFirstVBFrame = false;
-                return !bMissed;
-            }
-            break;
-        case VB_DRI:
-            {
-                drm_wait_vblank_t blank;
-                blank.request.type = DRM_VBLANK_RELATIVE;
-                blank.request.sequence = 1;
-                int rc = drmWaitVBlank (m_dri_fd, &blank);
-                if (rc) {
-                    static bool bFirstVBlankError = true;
-                    if (bFirstVBlankError) {
-                        bFirstVBlankError = false;
-                        int err = errno;
-                        AVG_TRACE(Logger::WARNING, 
-                                "Could not wait for vblank. Reason: "
-                                << strerror(err) << " (" << err << ")");
-//                      AVG_TRACE(Logger::WARNING, "Vertical blank support disabled.");
-//                      m_Method = VB_KAPUTT;
-                    }
-                }
-                return true;
-            }
-#endif
+        case VB_SGI:
         case VB_APPLE:
         case VB_WIN:
             return true;
@@ -922,11 +787,6 @@ void SDLDisplayEngine::calcRefreshRate() {
 
 vector<long> SDLDisplayEngine::KeyCodeTranslationTable(SDLK_LAST, key::KEY_UNKNOWN);
 
-void SDLDisplayEngine::initInput()
-{
-    initJoysticks();
-}
-
 const char * getEventTypeName(unsigned char Type) 
 {
     switch(Type) {
@@ -1036,11 +896,11 @@ EventPtr SDLDisplayEngine::createMouseEvent
     if (m_LastMousePos.x == -1) {
         speed = DPoint(0,0);
     } else {
-        double lastFrameTime = 1000/getFramerate();
+        double lastFrameTime = 1000/getEffectiveFramerate();
         speed = DPoint(x-m_LastMousePos.x, y-m_LastMousePos.y)/lastFrameTime;
     }
-    MouseEventPtr pEvent(new MouseEvent(Type, bool(buttonState & SDL_BUTTON(1)),
-            bool(buttonState & SDL_BUTTON(2)), bool(buttonState & SDL_BUTTON(3)),
+    MouseEventPtr pEvent(new MouseEvent(Type, (buttonState & SDL_BUTTON(1)) != 0,
+            (buttonState & SDL_BUTTON(2)) != 0, (buttonState & SDL_BUTTON(3)) != 0,
             IntPoint(x, y), Button, speed));
     m_LastMousePos = IntPoint(x,y);
     return pEvent; 
@@ -1125,35 +985,6 @@ EventPtr SDLDisplayEngine::createKeyEvent
     return pEvent;
 }
 
-void SDLDisplayEngine::initJoysticks() 
-{
-/*
-    SDL_InitSubSystem(SDL_INIT_JOYSTICK);
-    SDL_JoystickEventState(SDL_ENABLE);
-    cerr << "**** Number of joysticks: " << SDL_NumJoysticks() << endl;
-    for (int i=0; i<SDL_NumJoysticks(); i++) {
-        SDL_Joystick * pJoystick = SDL_JoystickOpen(i);
-        if (!pJoystick) {
-            cerr << "Warning: could not open joystick # " << i << "of"
-                << SDL_NumJoysticks() << endl;
-        } else {
-            printf("Opened Joystick %i\n", i);
-            printf("Name: %s\n", SDL_JoystickName(i));
-            printf("Number of Axes: %d\n", SDL_JoystickNumAxes(pJoystick));
-            printf("Number of Buttons: %d\n", SDL_JoystickNumButtons(pJoystick));
-            printf("Number of Balls: %d\n", SDL_JoystickNumBalls(pJoystick));
-        }
-    }
-*/
-}
-
-void SDLDisplayEngine::safeSetAttribute( SDL_GLattr attr, int value) {
-    int err = SDL_GL_SetAttribute(attr, value);
-    if (err == -1) {
-        throw Exception(AVG_ERR_VIDEO_GENERAL, SDL_GetError());
-    }
-}
-
 void SDLDisplayEngine::initTranslationTable()
 {
 #define TRANSLATION_ENTRY(x) KeyCodeTranslationTable[SDLK_##x] = key::KEY_##x;
@@ -1392,24 +1223,18 @@ void SDLDisplayEngine::initTranslationTable()
     TRANSLATION_ENTRY(UNDO);
 }
 
-int SDLDisplayEngine::getTextureMode()
+void SDLDisplayEngine::initTextureMode()
 {
-    if (m_TextureMode == 0) {
-        if (m_bShouldUsePOW2Textures) {
-            m_TextureMode = GL_TEXTURE_2D;
-        } else {
-            if (queryOGLExtension("GL_NV_texture_rectangle")) {
-                m_TextureMode = GL_TEXTURE_RECTANGLE_NV;
-            } else if (queryOGLExtension("GL_EXT_texture_rectangle") ||
-                    queryOGLExtension("GL_ARB_texture_rectangle")) 
-            {
-                m_TextureMode = GL_TEXTURE_RECTANGLE_ARB;
-            } else {
-                m_TextureMode = GL_TEXTURE_2D;
-            }
-        }
+    if (m_bShouldUsePOW2Textures) {
+        m_bUsePOTTextures = true;
+    } else {
+        m_bUsePOTTextures = !queryOGLExtension("GL_ARB_texture_non_power_of_two");
     }
-    return m_TextureMode;
+}
+
+bool SDLDisplayEngine::usePOTTextures()
+{
+    return m_bUsePOTTextures;
 }
 
 int SDLDisplayEngine::getMaxTexSize() 
@@ -1420,6 +1245,73 @@ int SDLDisplayEngine::getMaxTexSize()
     return m_MaxTexSize;
 }
 
+void SDLDisplayEngine::enableTexture(bool bEnable)
+{
+    if (bEnable != m_bEnableTexture) {
+        if (bEnable) {
+            glEnable(GL_TEXTURE_2D);
+        } else {
+            glDisable(GL_TEXTURE_2D);
+        }
+        m_bEnableTexture = bEnable;
+    }
+}
+
+void SDLDisplayEngine::enableGLColorArray(bool bEnable)
+{
+    if (bEnable != m_bEnableGLColorArray) {
+        if (bEnable) {
+            glEnableClientState(GL_COLOR_ARRAY);
+        } else {
+            glDisableClientState(GL_COLOR_ARRAY);
+        }
+        m_bEnableGLColorArray = bEnable;
+    }
+}
+
+void checkBlendModeError(const char *mode) 
+{    
+    GLenum err = glGetError();
+    if (err != GL_NO_ERROR) {
+        static bool bErrorReported = false;
+        if (!bErrorReported) {
+            AVG_TRACE(Logger::WARNING, "Blendmode "<< mode <<
+                    " not supported by OpenGL implementation.");
+            bErrorReported = true;
+        }
+    }
+}
+
+void SDLDisplayEngine::setBlendMode(BlendMode mode)
+{
+    if (mode != m_BlendMode) {
+        switch (mode) {
+            case BLEND_BLEND:
+                glproc::BlendEquation(GL_FUNC_ADD);
+                glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+                checkBlendModeError("blend");
+                break;
+            case BLEND_ADD:
+                glproc::BlendEquation(GL_FUNC_ADD);
+                glBlendFunc(GL_SRC_ALPHA, GL_ONE);
+                checkBlendModeError("add");
+                break;
+            case BLEND_MIN:
+                glproc::BlendEquation(GL_MIN);
+                glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+                checkBlendModeError("min");
+                break;
+            case BLEND_MAX:
+                glproc::BlendEquation(GL_MAX);
+                glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+                checkBlendModeError("max");
+                break;
+        }
+
+        m_BlendMode = mode;
+    }
+}
+
 // This is what OpenGL calls InternalFormat
 int SDLDisplayEngine::getOGLDestMode(PixelFormat pf)
 {
@@ -1435,16 +1327,6 @@ int SDLDisplayEngine::getOGLDestMode(PixelFormat pf)
         case R8G8B8X8:
         case B8G8R8X8:
             return GL_RGBA;    
-        case YCbCr422:
-            switch (getYCbCrMode()) {
-                case OGL_MESA:
-                    return GL_YCBCR_MESA;    
-                case OGL_APPLE:
-                    return GL_RGBA;
-                default:
-                    AVG_TRACE(Logger::ERROR, 
-                            "SDLDisplayEngine: YCbCr422 not supported.");
-            }
         default:
             AVG_TRACE(Logger::ERROR, "Unsupported pixel format " << 
                     Bitmap::getPixelFormatString(pf) <<
@@ -1468,15 +1350,6 @@ int SDLDisplayEngine::getOGLSrcMode(PixelFormat pf)
         case R8G8B8X8:
         case R8G8B8A8:
             return GL_RGBA;
-        case YCbCr422:
-            switch (getYCbCrMode()) {
-                case OGL_MESA:
-                    return GL_YCBCR_MESA;    
-                case OGL_APPLE:
-                    return GL_YCBCR_422_APPLE;
-                default:
-                    AVG_TRACE(Logger::ERROR, "SDLDisplayEngine: YCbCr422 not supported.");
-            }
         default:
             AVG_TRACE(Logger::ERROR, "Unsupported pixel format " << 
                     Bitmap::getPixelFormatString(pf) <<
@@ -1488,12 +1361,6 @@ int SDLDisplayEngine::getOGLSrcMode(PixelFormat pf)
 int SDLDisplayEngine::getOGLPixelType(PixelFormat pf)
 {
     switch (pf) {
-        case YCbCr422:
-#ifdef __i386__
-            return GL_UNSIGNED_SHORT_8_8_REV_APPLE;
-#else
-            return GL_UNSIGNED_SHORT_8_8_APPLE;
-#endif 
         case B8G8R8X8:
         case B8G8R8A8:
 //            return GL_UNSIGNED_INT_8_8_8_8_REV;
@@ -1504,13 +1371,11 @@ int SDLDisplayEngine::getOGLPixelType(PixelFormat pf)
 
 OGLMemoryMode SDLDisplayEngine::getMemoryModeSupported()
 {
-    bool bIsParallels = 
-            (string((char*)glGetString(GL_VENDOR)).find("Parallels") != string::npos);
     if (!m_bCheckedMemoryMode) {
         if ((queryOGLExtension("GL_ARB_pixel_buffer_object") || 
              queryOGLExtension("GL_EXT_pixel_buffer_object")) &&
             m_bShouldUsePixelBuffers &&
-            !bIsParallels)
+            !isParallels())
         {
             m_MemoryMode = PBO;
         } else {
@@ -1521,9 +1386,14 @@ OGLMemoryMode SDLDisplayEngine::getMemoryModeSupported()
     return m_MemoryMode;
 }
 
+bool SDLDisplayEngine::isParallels()
+{
+    static bool bIsParallels = 
+            (string((char*)glGetString(GL_VENDOR)).find("Parallels") != string::npos);
+    return bIsParallels;
+}
 
-
-void SDLDisplayEngine::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCbCrMode, 
+void SDLDisplayEngine::setOGLOptions(bool bUsePOW2Textures, bool bUseShaders,
         bool bUsePixelBuffers, int MultiSampleSamples, 
         VSyncMode DesiredVSyncMode)
 {
@@ -1533,36 +1403,11 @@ void SDLDisplayEngine::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCb
         return;
     }
     m_bShouldUsePOW2Textures = bUsePOW2Textures;
-    m_DesiredYCbCrMode = DesiredYCbCrMode;
+    m_bShouldUseShaders = bUseShaders;
     m_bShouldUsePixelBuffers = bUsePixelBuffers;
     m_MultiSampleSamples = MultiSampleSamples;
     m_DesiredVSyncMode = DesiredVSyncMode;
 }
 
-long long SDLDisplayEngine::getGPUMemoryUsage()
-{
-    vector<OGLSurface *>::iterator it;
-
-    long long lAmount = 0;
-    for(it = m_pSurfaces.begin() ; it != m_pSurfaces.end() ; ++it) {
-        lAmount += (*it)->getTotalTexMemory();
-    }
-    
-    return lAmount;
 }
 
-void SDLDisplayEngine::deregisterSurface(OGLSurface *pOGLSurface)
-{
-    vector<OGLSurface *>::iterator it;
-
-    for(it = m_pSurfaces.begin() ; it != m_pSurfaces.end() ; ++it) {
-        if (pOGLSurface == *it) {
-            m_pSurfaces.erase(it);
-            return;
-        }
-    }
-
-    assert(false);
-}
-
-}
