@@ -22,11 +22,13 @@
 #include "TestHelper.h"
 #include "Player.h"
 #include "MouseEvent.h"
+#include "TouchEvent.h"
 #include "KeyEvent.h"
 
 #include "../base/ObjectCounter.h"
 
 #include <iostream>
+#include <limits>
 
 using namespace std;
 
@@ -57,6 +59,20 @@ void TestHelper::fakeMouseEvent(Event::Type eventType,
     m_Events.push_back(pEvent);
 }
 
+void TestHelper::fakeTouchEvent(int id, Event::Type eventType,
+        Event::Source source, const DPoint& pos, const DPoint& lastDownPos,
+        const DPoint& speed)
+{
+    BlobPtr pBlob(new Blob(Run(int(pos.y), int(pos.x), int(pos.x)+1)));
+    pBlob->calcStats();
+    // The id is modified to avoid collisions with real touch events.
+    TouchEventPtr pEvent(new TouchEvent(id+numeric_limits<int>::max()/2, eventType, 
+            pBlob, IntPoint(pos), source, speed, IntPoint(lastDownPos)));
+
+    m_Events.push_back(pEvent);
+}
+
+
 void TestHelper::fakeKeyEvent(Event::Type eventType,
         unsigned char scanCode, int keyCode, 
         const string& keyString, int unicode, int modifiers)
