@@ -24,10 +24,11 @@
 
 // Python docs say python.h should be included before any standard headers (!)
 #include "WrapPython.h" 
+#include "../api.h"
 
 namespace avg {
 
-class Timeout
+class AVG_API Timeout
 {
     public:
         Timeout (int time, PyObject * pyfunc, bool isInterval, long long StartTime);
