@@ -31,14 +31,10 @@
 
 #include "../graphics/HistoryPreProcessor.h"
 #include "../graphics/Filterfill.h"
-#include "../graphics/Filterflip.h"
 #include "../graphics/Pixel8.h"
 
 #include "../imaging/DeDistort.h"
 #include "../imaging/CoordTransformer.h"
-#if defined(AVG_ENABLE_1394) || defined(AVG_ENABLE_1394_2)
-#include "../imaging/CameraUtils.h"
-#endif
 
 #include <boost/thread/thread.hpp>
 #include <boost/bind.hpp>
@@ -61,7 +57,6 @@ namespace avg {
         : m_pTrackerThread(0),
           m_pCamera(pCamera),
           m_bSubtractHistory(bSubtractHistory),
-          m_DisplayExtents(DisplayExtents),
           m_pCalibrator(0),
           m_TrackerConfig(Config)
     {
@@ -71,7 +66,15 @@ namespace avg {
         m_pBitmaps[0] = BitmapPtr(new Bitmap(ImgSize, I8));
         m_pMutex = MutexPtr(new boost::mutex);
         m_pCmdQueue = TrackerThread::CmdQueuePtr(new TrackerThread::CmdQueue);
-        IntRect ROI = m_TrackerConfig.getTransform()->getActiveBlobArea(DPoint(m_DisplayExtents));
+        m_pDeDistort = m_TrackerConfig.getTransform();
+        m_ActiveDisplaySize = DisplayExtents;
+        try {
+            m_ActiveDisplaySize = IntPoint(
+                    m_TrackerConfig.getPointParam("/transform/activedisplaysize/"));
+        } catch (Exception) {
+        }
+
+        IntRect ROI = m_pDeDistort->getActiveBlobArea(DPoint(m_ActiveDisplaySize));
         if (ROI.tl.x < 0 || ROI.tl.y < 0 || ROI.br.x > ImgSize.x || ROI.br.y > ImgSize.y) {
             AVG_TRACE(Logger::ERROR, "Impossible tracker configuration: Region of interest is " 
                     << ROI << ", camera image size is " << ImgSize << ". Aborting.");
@@ -94,7 +97,6 @@ namespace avg {
     
     void TrackerEventSource::start()
     {
-        m_pCamera->open();
         m_pTrackerThread = new boost::thread(
                 TrackerThread(
                     m_InitialROI,
@@ -116,8 +118,7 @@ namespace avg {
         m_TrackerConfig.setParam(sElement, sValue);
 
         // Test if active area is outside camera.
-        DRect Area = m_TrackerConfig.getTransform()
-                ->getActiveBlobArea(DPoint(m_DisplayExtents));
+        DRect Area = m_pDeDistort->getActiveBlobArea(DPoint(m_ActiveDisplaySize));
         DPoint Size = m_TrackerConfig.getPointParam("/camera/size/");
         int Prescale = m_TrackerConfig.getIntParam("/tracker/prescale/@value");
         if (Area.br.x > Size.x/Prescale || Area.br.y > Size.y/Prescale ||
@@ -147,15 +148,15 @@ namespace avg {
                 &TrackerThread::resetHistory, _1)));
     }
 
-    void TrackerEventSource::saveConfig(const string& sFilename)
+    void TrackerEventSource::saveConfig()
     {
-        m_TrackerConfig.save(sFilename);
+        m_TrackerConfig.save();
     }
 
     void TrackerEventSource::setConfig()
     {
-        DRect Area = m_TrackerConfig.getTransform()
-                ->getActiveBlobArea(DPoint(m_DisplayExtents));
+        m_pDeDistort = m_TrackerConfig.getTransform();
+        DRect Area = m_pDeDistort->getActiveBlobArea(DPoint(m_ActiveDisplaySize));
         createBitmaps(Area);
         m_pCmdQueue->push(Command<TrackerThread>(boost::bind(
                 &TrackerThread::setConfig, _1, m_TrackerConfig, Area, m_pBitmaps)));
@@ -190,11 +191,6 @@ namespace avg {
         boost::mutex::scoped_lock Lock(*m_pMutex);
         return new Bitmap(*m_pBitmaps[ImageID]);
     }
-    
-    double getDistSquared(DPoint c1, DPoint c2)
-    {
-        return (c1.x-c2.x)*(c1.x-c2.x) + (c1.y-c2.y)*(c1.y-c2.y);
-    }
 
     static ProfilingZone ProfilingZoneCalcTrack("trackBlobIDs(track)");
     static ProfilingZone ProfilingZoneCalcTouch("trackBlobIDs(touch)");
@@ -262,7 +258,7 @@ namespace avg {
             BlobPtr pNewBlob = *it;
             for(BlobVector::iterator it2 = OldBlobs.begin(); it2!=OldBlobs.end(); ++it2) { 
                 BlobPtr pOldBlob = *it2;
-                double distSquared = getDistSquared(pNewBlob->getCenter(),
+                double distSquared = calcDistSquared(pNewBlob->getCenter(),
                         pOldBlob->getEstimatedNextCenter());
                 if (distSquared <= MaxDistSquared) {
                     BlobDistEntryPtr pEntry = BlobDistEntryPtr(
@@ -291,7 +287,12 @@ namespace avg {
                 assert (pEvents->find(pOldBlob) != pEvents->end());
                 EventStreamPtr pStream;
                 pStream = pEvents->find(pOldBlob)->second;
-                pStream->blobChanged(pNewBlob, time, bEventOnMove);
+                // EventOnMove means events are discarded when the cursor doesn't move.
+                // But even if the cursor doesn't move, we have to make sure we don't
+                // discard any events that have related info.
+                bool bKeepAllEvents = !bEventOnMove || 
+                        (pNewBlob->getFirstRelated() && !bTouch);
+                pStream->blobChanged(pNewBlob, time, !bKeepAllEvents);
                 pNewBlob->calcNextCenter(pOldBlob->getCenter());
                 // Update the mapping.
                 (*pEvents)[pNewBlob] = pStream;
@@ -322,10 +323,10 @@ namespace avg {
         assert(!m_pCalibrator);
         m_pOldTransformer = m_TrackerConfig.getTransform();
         m_TrackerConfig.setTransform(DeDistortPtr(new DeDistort(
-                DPoint(m_pBitmaps[0]->getSize()), DPoint(m_DisplayExtents))));
+                DPoint(m_pBitmaps[0]->getSize()), DPoint(m_ActiveDisplaySize))));
         setConfig();
         m_pCalibrator = new TrackerCalibrator(m_pBitmaps[0]->getSize(),
-                m_DisplayExtents);
+                m_ActiveDisplaySize);
         return m_pCalibrator;
     }
 
@@ -334,7 +335,7 @@ namespace avg {
         assert(m_pCalibrator);
         m_TrackerConfig.setTransform(m_pCalibrator->makeTransformer());
         DRect Area = m_TrackerConfig.getTransform()
-                ->getActiveBlobArea(DPoint(m_DisplayExtents));
+                ->getActiveBlobArea(DPoint(m_ActiveDisplaySize));
         if (Area.size().x*Area.size().y > 1024*1024*8) {
             AVG_TRACE(Logger::WARNING, "Ignoring calibration - resulting area would be " 
                     << Area);
@@ -374,11 +375,10 @@ namespace avg {
     {
         EventPtr pEvent;
         int kill_counter = 0;
-        DeDistortPtr pDeDistort = m_TrackerConfig.getTransform();
         bool bEventOnMove = m_TrackerConfig.getBoolParam("/tracker/eventonmove/@value");
         for (EventMap::iterator it = Events.begin(); it!= Events.end();) {
             EventStreamPtr pStream = (*it).second;
-            pEvent = pStream->pollevent(pDeDistort, m_DisplayExtents,
+            pEvent = pStream->pollevent(m_pDeDistort, m_ActiveDisplaySize,
                     source, bEventOnMove);
             if (pEvent) {
                 res.push_back(pEvent);
@@ -413,7 +413,7 @@ namespace avg {
                     pTrackEvent = boost::dynamic_pointer_cast<TouchEvent>(*it2);
                     pTrackBlob = pTrackEvent->getBlob();
                 }
-                if (it2 != pTrackEvents.end()) {
+                if (pTrackBlob == pRelatedBlob) {
                     pTouchEvent->addRelatedEvent(pTrackEvent);
                     pTrackEvent->addRelatedEvent(pTouchEvent);
                 }
