@@ -22,6 +22,7 @@
 #ifndef _TrackerEventSource_H_
 #define _TrackerEventSource_H_
 
+#include "../api.h"
 #include "CursorEvent.h"
 #include "IEventSource.h"
 #include "TrackerCalibrator.h"
@@ -46,7 +47,7 @@ class EventStream;
 typedef boost::shared_ptr<EventStream> EventStreamPtr;
 typedef std::map<BlobPtr, EventStreamPtr> EventMap;
 
-class TrackerEventSource: public IBlobTarget, public IEventSource
+class AVG_API TrackerEventSource: public IBlobTarget, public IEventSource
 {
     public:
         TrackerEventSource(CameraPtr pCamera, const TrackerConfig& Config,
@@ -61,7 +62,7 @@ class TrackerEventSource: public IBlobTarget, public IEventSource
 
         void setDebugImages(bool bImg, bool bFinger);
 
-        void saveConfig(const std::string& sFilename);
+        void saveConfig();
 
         Bitmap * getImage(TrackerImageID ImageID) const;
         std::vector<EventPtr> pollEvents(); //main thread
@@ -90,8 +91,9 @@ class TrackerEventSource: public IBlobTarget, public IEventSource
         IntRect m_InitialROI;
         CameraPtr m_pCamera;
         bool m_bSubtractHistory;
+        DeDistortPtr m_pDeDistort;
         DeDistortPtr m_pOldTransformer;
-        IntPoint m_DisplayExtents;
+        IntPoint m_ActiveDisplaySize;
         TrackerCalibrator * m_pCalibrator;
 
         // Used by tracker thread
