@@ -20,64 +20,32 @@
 //
 
 #include "Words.h"
-#include "DisplayEngine.h"
-#include "ISurface.h"
+#include "SDLDisplayEngine.h"
+#include "OGLSurface.h"
 #include "NodeDefinition.h"
+#include "TextEngine.h"
 
 #include "../base/Logger.h"
 #include "../base/Exception.h"
 #include "../base/ScopeTimer.h"
 #include "../base/XMLHelper.h"
-#include "../base/OSHelper.h"
-#include "../base/FileHelper.h"
 #include "../base/StringHelper.h"
+#include "../base/MathHelper.h"
+#include "../base/ObjectCounter.h"
 
 #include "../graphics/Filterfill.h"
 
 #include <pango/pangoft2.h>
 
 #include <iostream>
-#include <stdlib.h>
 #include <algorithm>
-#include <cctype> // for toupper
 
 using namespace std;
 
 namespace avg {
 
-set<string> Words::s_sFontsNotFound;
-set<pair<string, string> > Words::s_VariantsNotFound;
-bool Words::s_bInitialized = false;
-int Words::s_NumFontFamilies = 0;
-PangoFontFamily** Words::s_ppFontFamilies = 0;
-PangoContext * Words::s_pPangoContext = 0;
-
-void GLibLogFunc(const gchar *log_domain, GLogLevelFlags log_level, 
-        const gchar *message, gpointer unused_data)
+NodeDefinition Words::createDefinition()
 {
-    string s = "Pango ";
-    if (log_level & G_LOG_LEVEL_ERROR) {
-        s += "error: ";
-    } else if (log_level & G_LOG_LEVEL_CRITICAL) {
-        s += string("critical: ")+message;
-        AVG_TRACE(Logger::ERROR, s);
-        assert(false);
-    } else if (log_level & G_LOG_LEVEL_WARNING) {
-        s += "warning: ";
-    } else if (log_level & G_LOG_LEVEL_MESSAGE) {
-        s += "message: ";
-    } else if (log_level & G_LOG_LEVEL_INFO) {
-        s += "info: ";
-    } else if (log_level & G_LOG_LEVEL_DEBUG) {
-        s += "debug: ";
-    }
-    s += message;
-    AVG_TRACE(Logger::WARNING, s);
-}
-
-NodeDefinition Words::getNodeDefinition()
-{
-    static const string sChildren = "(#PCDATA|span|b|big|i|s|sub|sup|small|tt|u|br)*";
     static const string sDTDElements = 
         "<!ELEMENT span (#PCDATA|span|b|big|i|s|sub|sup|small|tt|u)*>\n"
         "<!ATTLIST span\n"
@@ -96,6 +64,7 @@ NodeDefinition Words::getNodeDefinition()
         "  strikethrough CDATA #IMPLIED\n"
         "  fallback CDATA #IMPLIED\n"
         "  lang CDATA #IMPLIED\n"
+        "  letter_spacing CDATA #IMPLIED\n"
         "  rawtextmode CDATA #IMPLIED >\n"
 
         "<!ELEMENT b (#PCDATA|span|b|big|i|s|sub|sup|small|tt|u)*>\n"
@@ -109,36 +78,32 @@ NodeDefinition Words::getNodeDefinition()
         "<!ELEMENT u (#PCDATA|span|b|big|i|s|sub|sup|small|tt|u)*>\n"
         "<!ELEMENT br (#PCDATA)*>\n";
 
+    string sChildArray[] = {"#PCDATA", "span", "b", "big", "i", "s", "sup", "sub", 
+            "small", "tt", "u", "br"};
+    vector<string> sChildren = vectorFromCArray(sizeof(sChildArray)/sizeof(*sChildArray),
+            sChildArray); 
     return NodeDefinition("words", Node::buildNode<Words>)
-        .extendDefinition(RasterNode::getNodeDefinition())
+        .extendDefinition(RasterNode::createDefinition())
         .addChildren(sChildren)
         .addDTDElements(sDTDElements)
         .addArg(Arg<string>("font", "arial", false, offsetof(Words, m_sFontName)))
         .addArg(Arg<string>("variant", "", false, offsetof(Words, m_sFontVariant)))
-        .addArg(Arg<string>("text", ""))
+        .addArg(Arg<UTF8String>("text", ""))
         .addArg(Arg<string>("color", "FFFFFF", false, offsetof(Words, m_sColorName)))
-        .addArg(Arg<double>("size", 15, false, offsetof(Words, m_Size)))
-        .addArg(Arg<int>("parawidth", -1, false, offsetof(Words, m_ParaWidth)))
+        .addArg(Arg<double>("fontsize", 15, false, offsetof(Words, m_FontSize)))
         .addArg(Arg<int>("indent", 0, false, offsetof(Words, m_Indent)))
         .addArg(Arg<double>("linespacing", -1, false, offsetof(Words, m_LineSpacing)))
         .addArg(Arg<string>("alignment", "left"))
-        .addArg(Arg<bool>("rawtextmode", false, false, offsetof(Words, m_bRawTextMode)));
+        .addArg(Arg<string>("wrapmode", "word"))
+        .addArg(Arg<bool>("justify", false, false, offsetof(Words, m_bJustify)))
+        .addArg(Arg<bool>("rawtextmode", false, false, offsetof(Words, m_bRawTextMode)))
+        .addArg(Arg<double>("letterspacing", 0, false, offsetof(Words, m_LetterSpacing)))
+        .addArg(Arg<bool>("hint", true, false, offsetof(Words, m_bHint)))
+        ;
 }
 
-static void
-text_subst_func (FcPattern *pattern, gpointer data)
-{
-//  GimpText *text = GIMP_TEXT (data);
-
-  FcPatternAddBool(pattern, FC_HINTING, true);
-  FcPatternAddInteger(pattern, FC_HINT_STYLE, FC_HINT_MEDIUM);
-  FcPatternAddInteger(pattern, FC_RGBA, FC_RGBA_NONE);
-  FcPatternAddBool(pattern, FC_ANTIALIAS, true);
-}
-
-Words::Words (const ArgList& Args, Player * pPlayer, bool bFromXML)
-    : RasterNode(pPlayer), 
-      m_StringExtents(0,0),
+Words::Words(const ArgList& Args)
+    : m_StringExtents(0,0),
       m_pFontDescription(0),
       m_pLayout(0),
       m_bFontChanged(true),
@@ -146,28 +111,16 @@ Words::Words (const ArgList& Args, Player * pPlayer, bool bFromXML)
 {
     m_bParsedText = false;
 
-    if (!s_pPangoContext) {
-        pango_ft2_get_context(72, 72);
-        
-        PangoFT2FontMap *fontmap;
-        fontmap = PANGO_FT2_FONT_MAP (pango_ft2_font_map_new());
-        pango_ft2_font_map_set_resolution (fontmap, 72, 72);
-        pango_ft2_font_map_set_default_substitute (fontmap, text_subst_func, 0, 0);
-        s_pPangoContext = pango_ft2_font_map_create_context (fontmap);
-        g_object_unref (fontmap);
-
-        pango_context_set_language(s_pPangoContext,
-                pango_language_from_string ("en_US"));
-        pango_context_set_base_dir(s_pPangoContext, PANGO_DIRECTION_LTR);
-    }
-
     Args.setMembers(this);
     setAlignment(Args.getArgVal<string>("alignment"));
-    setText(Args.getArgVal<string>("text"));
-    initFonts();
+    setWrapMode(Args.getArgVal<string>("wrapmode"));
+    setText(Args.getArgVal<UTF8String>("text"));
+    m_Color = colorStringToColor(m_sColorName);
+    setViewport(-32767, -32767, -32767, -32767);
+    ObjectCounter::get()->incRef(&typeid(*this));
 }
 
-Words::~Words ()
+Words::~Words()
 {
     if (m_pFontDescription) {
         pango_font_description_free(m_pFontDescription);
@@ -175,6 +128,7 @@ Words::~Words ()
     if (m_pLayout) {
         g_object_unref(m_pLayout);
     }
+    ObjectCounter::get()->decRef(&typeid(*this));
 }
 
 void Words::setTextFromNodeValue(const string& sText)
@@ -189,25 +143,40 @@ void Words::setTextFromNodeValue(const string& sText)
 
 void Words::setRenderingEngines(DisplayEngine * pDisplayEngine, AudioEngine * pAudioEngine)
 {
-    m_Color = colorStringToColor(m_sColorName);
-
     m_bFontChanged = true;
     m_bDrawNeeded = true;
     RasterNode::setRenderingEngines(pDisplayEngine, pAudioEngine);
 }
 
-void Words::disconnect()
+void Words::connect()
+{
+    RasterNode::connect();
+    checkReload();
+}
+
+void Words::disconnect(bool bKill)
 {
     if (m_pFontDescription) {
         pango_font_description_free(m_pFontDescription);
         m_pFontDescription = 0;
+        m_bFontChanged = true;
     }
-    RasterNode::disconnect();
+    RasterNode::disconnect(bKill);
 }
 
-string Words::getTypeStr ()
+string Words::getAlignment() const
 {
-    return "Words";
+    switch(m_Alignment) {
+        case PANGO_ALIGN_LEFT:
+            return "left";
+        case PANGO_ALIGN_CENTER:
+            return "center";
+        case PANGO_ALIGN_RIGHT:
+            return "right";
+        default:
+            assert(false);
+            return "";
+    }
 }
 
 void Words::setAlignment(const string& sAlign)
@@ -226,16 +195,56 @@ void Words::setAlignment(const string& sAlign)
     m_bDrawNeeded = true;
 }
 
+bool Words::getJustify() const
+{
+    return m_bJustify;
+}
+
+void Words::setJustify(bool bJustify)
+{
+    m_bJustify = bJustify;
+    m_bDrawNeeded = true;
+}
+
+double Words::getLetterSpacing() const
+{
+    return m_LetterSpacing;
+}
+
+void Words::setLetterSpacing(double letterSpacing)
+{
+    m_LetterSpacing = letterSpacing;
+    m_bDrawNeeded = true;
+}
+
+bool Words::getHint() const
+{
+    return m_bHint;
+}
+
+void Words::setHint(bool bHint)
+{
+    m_bHint = bHint;
+}
+
+
 double Words::getWidth() 
 {
     drawString();
-    return Node::getWidth();
+    return AreaNode::getWidth();
 }
 
 double Words::getHeight()
 {
     drawString();
-    return Node::getHeight();
+    return AreaNode::getHeight();
+}
+
+NodePtr Words::getElementByPos(const DPoint & pos)
+{
+    drawString();
+    DPoint relPos = pos-DPoint(m_PosOffset);
+    return AreaNode::getElementByPos(relPos);
 }
 
 const std::string& Words::getFont() const
@@ -255,6 +264,12 @@ const std::string& Words::getFontVariant() const
     return m_sFontVariant;
 }
 
+void Words::addFontDir(const std::string& sDir)
+{
+    TextEngine::get(true).addFontDir(sDir);
+    TextEngine::get(false).addFontDir(sDir);
+}
+
 void Words::setFontVariant(const std::string& sVariant)
 {
     m_sFontVariant = sVariant;
@@ -269,6 +284,10 @@ const UTF8String& Words::getText() const
 
 void Words::setText(const UTF8String& sText)
 {
+    if (sText.length() > 32767) {
+        throw(Exception(AVG_ERR_INVALID_ARGS, 
+                string("Words::setText: string too long (") + toString(sText.length()) + ")"));
+    }
 //    cerr << "setText(): " << sText << "|" << endl;
     if (m_sRawText != sText) {
         m_sRawText = sText;
@@ -296,27 +315,16 @@ void Words::setColor(const std::string& sColor)
 
 double Words::getFontSize() const
 {
-    return m_Size;
+    return m_FontSize;
 }
 
 void Words::setFontSize(double Size)
 {
-    m_Size = Size;
+    m_FontSize = Size;
     m_bFontChanged = true;
     m_bDrawNeeded = true;
 }
 
-int Words::getParaWidth() const
-{
-    return m_ParaWidth;
-}
-
-void Words::setParaWidth(int ParaWidth)
-{
-    m_ParaWidth = ParaWidth;
-    m_bDrawNeeded = true;
-}
-
 int Words::getIndent() const
 {
     return m_Indent;
@@ -358,99 +366,62 @@ void Words::setRawTextMode(bool RawTextMode)
     }
 }
 
-string Words::getAlignment() const
-{
-    switch(m_Alignment) {
-        case PANGO_ALIGN_LEFT:
-            return "left";
-        case PANGO_ALIGN_CENTER:
-            return "center";
-        case PANGO_ALIGN_RIGHT:
-            return "right";
-    }
-    return "";
-}
-
-const vector<string>& Words::getFontFamilies()
-{
-    static vector<string> sFonts;
-    initFonts();
-    if (s_ppFontFamilies == 0) {
-        PangoFT2FontMap *fontmap;
-        fontmap = PANGO_FT2_FONT_MAP (pango_ft2_font_map_new());
-        pango_ft2_font_map_set_resolution (fontmap, 72, 72);
-        pango_ft2_font_map_set_default_substitute (fontmap, text_subst_func, 0, 0);
-        PangoContext * pContext = pango_ft2_font_map_create_context (fontmap);
-        pango_context_set_language(pContext,
-                pango_language_from_string("en_US"));
-        PangoFontMap* pFontMap = pango_context_get_font_map(pContext);
-
-        string sOldLang = "";
-        getEnv("LC_CTYPE", sOldLang);
-        setEnv("LC_CTYPE", "en-us");
-        pango_font_map_list_families(pFontMap, &s_ppFontFamilies, &s_NumFontFamilies);
-        setEnv("LC_CTYPE", sOldLang);
-        for (int i=0; i<s_NumFontFamilies; ++i) {
-            sFonts.push_back(pango_font_family_get_name(s_ppFontFamilies[i]));
-        }
-        sort(sFonts.begin(), sFonts.end());
-    }
-    return sFonts;
-}
-
-const vector<string>& Words::getFontVariants(const string& sFontName)
-{
-    PangoFontFamily * pCurFamily = getFontFamily(sFontName);
-    PangoFontFace ** ppFaces;
-    int numFaces;
-    pango_font_family_list_faces (pCurFamily, &ppFaces, &numFaces);
-    static vector<string> sVariants;
-    for (int i=0; i<numFaces; ++i) {
-        sVariants.push_back(pango_font_face_get_face_name(ppFaces[i]));
-    }
-    g_free(ppFaces);
-    return sVariants;
-}
-
-IntPoint Words::getGlyphPos(int i)
+DPoint Words::getGlyphPos(int i)
 {
     PangoRectangle rect = getGlyphRect(i);
-    return IntPoint(rect.x/PANGO_SCALE, rect.y/PANGO_SCALE);
+    return DPoint(double(rect.x)/PANGO_SCALE, double(rect.y)/PANGO_SCALE);
 }
 
-IntPoint Words::getGlyphSize(int i)
+DPoint Words::getGlyphSize(int i)
 {
     PangoRectangle rect = getGlyphRect(i);
-    return IntPoint(rect.width/PANGO_SCALE, rect.height/PANGO_SCALE);
+    return DPoint(double(rect.width)/PANGO_SCALE, double(rect.height)/PANGO_SCALE);
 }
 
-PangoFontFamily * Words::getFontFamily(const string& sFamily)
+void Words::setWrapMode(const string& sWrapMode)
 {
-    getFontFamilies();
-    PangoFontFamily * pFamily = 0;
-    assert(s_NumFontFamilies != 0);
-    for (int i=0; i<s_NumFontFamilies; ++i) {
-        if (equalIgnoreCase(pango_font_family_get_name(s_ppFontFamilies[i]), sFamily)) {
-            pFamily = s_ppFontFamilies[i];
-        }
+    if (sWrapMode == "word") {
+        m_WrapMode = PANGO_WRAP_WORD;
+    } else if (sWrapMode == "char") {
+        m_WrapMode = PANGO_WRAP_CHAR;
+    } else if (sWrapMode == "wordchar") {
+        m_WrapMode = PANGO_WRAP_WORD_CHAR;
+    } else {
+        throw(Exception(AVG_ERR_UNSUPPORTED, 
+                "Words wrapping mode "+sWrapMode+" not supported."));
     }
-    if (!pFamily) {
-        throw(Exception(AVG_ERR_INVALID_ARGS, 
-                "getFontFamily: Font family "+sFamily+" not found."));
+
+    m_bDrawNeeded = true;
+}
+
+string Words::getWrapMode() const
+{
+    switch(m_WrapMode) {
+        case PANGO_WRAP_WORD:
+            return "word";
+        case PANGO_WRAP_CHAR:
+            return "char";
+        case PANGO_WRAP_WORD_CHAR:
+            return "wordchar";
+        default:
+            assert(false);
+            return "";
     }
-    return pFamily;
 }
 
 void Words::parseString(PangoAttrList** ppAttrList, char** ppText)
 {
+    UTF8String sTextWithoutBreaks = applyBR(m_sText);
     bool bOk;
     GError * pError = 0;
-    bOk = (pango_parse_markup(m_sText.c_str(), int(m_sText.length()), 0,
+    bOk = (pango_parse_markup(sTextWithoutBreaks.c_str(), 
+            int(sTextWithoutBreaks.length()), 0,
             ppAttrList, ppText, 0, &pError) != 0);
     if (!bOk) {
         string sError;
         if (getID() != "") {
-            sError = string("Can't parse string in node with id '")+getID()+"' ("+pError->message+")";
+            sError = string("Can't parse string in node with id '")+getID()+"' ("
+                    +pError->message+")";
         } else {
             sError = string("Can't parse string '")+m_sRawText+"' ("+pError->message+")";
         }
@@ -459,10 +430,27 @@ void Words::parseString(PangoAttrList** ppAttrList, char** ppText)
 
 }
 
+void Words::setMaterialMask(MaterialInfo& material, const DPoint& pos, const DPoint& size, 
+        const DPoint& mediaSize)
+{
+    DPoint maskSize;
+    DPoint maskPos;
+    if (size == DPoint(0,0)) {
+        maskSize = DPoint(1,1);
+        maskPos = DPoint(pos.x/mediaSize.x, pos.y/mediaSize.y);
+    } else {
+        maskSize = DPoint(size.x/mediaSize.x, size.y/mediaSize.y);
+        maskPos = DPoint(pos.x/size.x, pos.y/size.y);
+    }
+    material.setMask(true);
+    material.setMaskCoords(maskPos, maskSize);
+}
+
 static ProfilingZone DrawStringProfilingZone("  Words::drawString");
 
 void Words::drawString()
 {
+    assert (m_sText.length() < 32767);
     if (!m_bDrawNeeded) {
         return;
     }
@@ -471,78 +459,54 @@ void Words::drawString()
         m_StringExtents = IntPoint(0,0);
     } else {
         if (m_bFontChanged) {
-            PangoFontFamily * pFamily;
-            bool bFamilyFound = true;
-            try {
-                pFamily = getFontFamily(m_sFontName);
-            } catch (Exception&) {
-                if (s_sFontsNotFound.find(m_sFontName) == s_sFontsNotFound.end()) {
-                    AVG_TRACE(Logger::WARNING, "Could not find font face " << 
-                            m_sFontName << ". Using sans instead.");
-                    s_sFontsNotFound.insert(m_sFontName);
-                }
-                bFamilyFound = false;
-                pFamily = getFontFamily("sans");
-            }
-            PangoFontFace ** ppFaces;
-            int numFaces;
-            pango_font_family_list_faces (pFamily, &ppFaces, &numFaces);
-            PangoFontFace * pFace = 0;
-            if (m_sFontVariant == "") {
-                pFace = ppFaces[0];
-            } else {
-                for (int i=0; i<numFaces; ++i) {
-                    if (equalIgnoreCase(pango_font_face_get_face_name(ppFaces[i]), 
-                            m_sFontVariant)) 
-                    {
-                        pFace = ppFaces[i];
-                    }
-                }
-            }
-            if (!pFace) {
-                pFace = ppFaces[0];
-                if (bFamilyFound) {
-                    pair<string, string> variant(m_sFontName, m_sFontVariant);
-                    if (s_VariantsNotFound.find(variant) == s_VariantsNotFound.end()) {
-                        s_VariantsNotFound.insert(variant);
-                        AVG_TRACE(Logger::WARNING, "Could not find font variant " 
-                                << m_sFontName << ":" << m_sFontVariant << ". Using " <<
-                                pango_font_face_get_face_name(pFace) << " instead.");
-                    }
-                }
-            }
-            g_free(ppFaces);
-
             if (m_pFontDescription) {
                 pango_font_description_free(m_pFontDescription);
             }
-            m_pFontDescription = pango_font_face_describe(pFace);
+            m_pFontDescription = TextEngine::get(m_bHint).getFontDescription(m_sFontName, 
+                    m_sFontVariant);
             pango_font_description_set_absolute_size(m_pFontDescription,
-                    (int)(m_Size * PANGO_SCALE));
+                    (int)(m_FontSize * PANGO_SCALE));
 
             m_bFontChanged = false;
         }
-        pango_context_set_font_description(s_pPangoContext, m_pFontDescription);
+        PangoContext* pContext = TextEngine::get(m_bHint).getPangoContext();
+        pango_context_set_font_description(pContext, m_pFontDescription);
 
         if (m_pLayout) {
             g_object_unref(m_pLayout);
         }
-        m_pLayout = pango_layout_new (s_pPangoContext);
+        m_pLayout = pango_layout_new(pContext);
 
+        PangoAttrList * pAttrList = 0;
+#if PANGO_VERSION > PANGO_VERSION_ENCODE(1,18,2) 
+        PangoAttribute * pLetterSpacing = pango_attr_letter_spacing_new
+                (int(m_LetterSpacing*1024));
+#endif
         if (m_bParsedText) {
-            PangoAttrList * pAttrList = 0;
             char * pText = 0;
             parseString(&pAttrList, &pText);
-            pango_layout_set_text (m_pLayout, pText, -1);
-            pango_layout_set_attributes (m_pLayout, pAttrList);
-            pango_attr_list_unref (pAttrList);
+#if PANGO_VERSION > PANGO_VERSION_ENCODE(1,18,2) 
+            // Workaround for pango bug.
+            pango_attr_list_insert_before(pAttrList, pLetterSpacing);
+#endif            
+            pango_layout_set_text(m_pLayout, pText, -1);
             g_free (pText);
         } else {
+            pAttrList = pango_attr_list_new();
+#if PANGO_VERSION > PANGO_VERSION_ENCODE(1,18,2) 
+            pango_attr_list_insert_before(pAttrList, pLetterSpacing);
+#endif
             pango_layout_set_text(m_pLayout, m_sText.c_str(), -1);
         }
+        pango_layout_set_attributes(m_pLayout, pAttrList);
+        pango_attr_list_unref(pAttrList);
 
+        pango_layout_set_wrap(m_pLayout, m_WrapMode);
         pango_layout_set_alignment(m_pLayout, m_Alignment);
-        pango_layout_set_width(m_pLayout, m_ParaWidth * PANGO_SCALE);
+        pango_layout_set_justify(m_pLayout, m_bJustify);
+        if (getUserSize().x != 0) {
+            pango_layout_set_width(m_pLayout, int(getUserSize().x * PANGO_SCALE));
+        }
         pango_layout_set_indent(m_pLayout, m_Indent * PANGO_SCALE);
         if (m_Indent < 0) {
             // For hanging indentation, we add a tabstop to support lists
@@ -556,18 +520,23 @@ void Words::drawString()
         }
         PangoRectangle logical_rect;
         PangoRectangle ink_rect;
-        pango_layout_get_pixel_extents (m_pLayout, &ink_rect, &logical_rect);
+        pango_layout_get_pixel_extents(m_pLayout, &ink_rect, &logical_rect);
+        assert (logical_rect.width < 4096);
+        assert (logical_rect.height < 4096);
 //        cerr << "Ink: " << ink_rect.x << ", " << ink_rect.y << ", " 
 //                << ink_rect.width << ", " << ink_rect.height << endl;
 //        cerr << "Logical: " << logical_rect.x << ", " << logical_rect.y << ", " 
 //                << logical_rect.width << ", " << logical_rect.height << endl;
         m_StringExtents.y = logical_rect.height+2;
-        m_StringExtents.x = m_ParaWidth;
-        if (m_ParaWidth == -1) {
+        if (getUserSize().x == 0) {
             m_StringExtents.x = logical_rect.width;
-            // Work around what appears to be a pango bug when computing the 
-            // extents of italic text by adding an arbritary amount to the width.
-            m_StringExtents.x += int(m_Size/6+1);
+            if (m_Alignment == PANGO_ALIGN_LEFT) {
+                // This makes sure we have enough room for italic text.
+                // XXX: We should really do some calculations based on ink_rect here.
+                m_StringExtents.x += int(m_FontSize/6+1);
+            }
+        } else {
+            m_StringExtents.x = int(getUserSize().x);
         }
         if (m_StringExtents.x == 0) {
             m_StringExtents.x = 1;
@@ -577,7 +546,7 @@ void Words::drawString()
         }
 //        cerr << "libavg Extents: " << m_StringExtents << endl;
         if (getState() == NS_CANRENDER) {
-            getSurface()->create(m_StringExtents, I8, false);
+            getSurface()->create(m_StringExtents, I8);
 
             BitmapPtr pBmp = getSurface()->lockBmp();
             FilterFill<unsigned char>(0).applyInPlace(pBmp);
@@ -590,15 +559,23 @@ void Words::drawString()
             bitmap.num_grays = 256;
             bitmap.pixel_mode = ft_pixel_mode_grays;
 
-            int yoffset = 0;
+            m_PosOffset = IntPoint(0,0);
             if (ink_rect.y < 0) {
-                yoffset = -ink_rect.y;
+                m_PosOffset.y = ink_rect.y;
+            }
+            if (ink_rect.x < 0) {
+                m_PosOffset.x = ink_rect.x;
+            }
+            pango_ft2_render_layout(&bitmap, m_pLayout, -m_PosOffset.x, -m_PosOffset.y);
+            if (m_Alignment == PANGO_ALIGN_CENTER) {
+                m_PosOffset.x -= m_StringExtents.x/2;
+            } else if (m_Alignment == PANGO_ALIGN_RIGHT) {
+                m_PosOffset.x -= m_StringExtents.x;
             }
 
-            // Use 1 as x position here to make sure italic text is never cut off.
-            pango_ft2_render_layout(&bitmap, m_pLayout, 1, yoffset);
+            getSurface()->unlockBmps();
 
-            getDisplayEngine()->surfaceChanged(getSurface());
+            getSurface()->bind();
             if (m_LineSpacing == -1) {
                 m_LineSpacing = pango_layout_get_spacing(m_pLayout)/PANGO_SCALE;
             }
@@ -612,6 +589,7 @@ void Words::drawString()
 
 void Words::preRender()
 {
+    Node::preRender();
     drawString();
 }
 
@@ -621,24 +599,33 @@ void Words::render(const DRect& Rect)
 {
     ScopeTimer Timer(RenderProfilingZone);
     if (m_sText.length() != 0 && getEffectiveOpacity() > 0.001) {
-        getDisplayEngine()->blta8(getSurface(), getSize(),
-                getEffectiveOpacity(), m_Color, getBlendMode());
+        if (m_PosOffset != IntPoint(0,0)) {
+            getDisplayEngine()->pushTransform(DPoint(m_PosOffset), 0, DPoint(0,0));
+        }
+        getSurface()->blta8(DPoint(getMediaSize()), getEffectiveOpacity(), m_Color, 
+                getBlendMode());
+        if (m_PosOffset != IntPoint(0,0)) {
+            getDisplayEngine()->popTransform();
+        }
     }
 }
 
-Pixel32 Words::colorStringToColor(const string & colorString)
-{
-    int r,g,b;
-    sscanf(colorString.c_str(), "%2x%2x%2x", &r, &g, &b);
-    return Pixel32(r,g,b);
-}
-
 IntPoint Words::getMediaSize()
 {
     drawString();
     return m_StringExtents;
 }
 
+const vector<string>& Words::getFontFamilies()
+{
+    return TextEngine::get(true).getFontFamilies();
+}
+
+const vector<string>& Words::getFontVariants(const string& sFontName)
+{
+    return TextEngine::get(true).getFontVariants(sFontName);
+}
+
 string Words::removeExcessSpaces(const string & sText)
 {
     string s = sText;
@@ -694,44 +681,21 @@ void Words::setParsedText(const UTF8String& sText)
     m_bParsedText = true;
 }
 
-void Words::checkFontError(int Ok, const string& sMsg)
+UTF8String Words::applyBR(const UTF8String& sText)
 {
-    if (Ok == 0) {
-        throw Exception(AVG_ERR_FONT_INIT_FAILED, sMsg);
-    }
-}
-
-void Words::initFonts()
-{
-    if (!s_bInitialized) {
-        g_type_init();
-        std::string sFontConfPath = "/etc/fonts/fonts.conf"; 
-        if (!fileExists(sFontConfPath)) {
-            sFontConfPath = getAvgLibPath()+"etc/fonts/fonts.conf";
+    UTF8String sResult(sText);
+    UTF8String sLowerText = tolower(sResult); 
+    string::size_type pos=sLowerText.find("<br/>");
+    while (pos != string::npos) {
+        sResult.replace(pos, 5, "\n");
+        sLowerText.replace(pos, 5, "\n");
+        if (sLowerText[pos+1] == ' ') {
+            sLowerText.erase(pos+1, 1);
+            sResult.erase(pos+1, 1);
         }
-        FcConfig * pConfig = FcConfigCreate();
-        int Ok = (int)FcConfigParseAndLoad(pConfig, (const FcChar8 *)(sFontConfPath.c_str()), true);
-        checkFontError(Ok, string("Font error: could not load config file ")+sFontConfPath);
-        Ok = (int)FcConfigBuildFonts(pConfig);
-        checkFontError(Ok, string("Font error: FcConfigBuildFonts failed."));
-        Ok = (int)FcConfigSetCurrent(pConfig);
-        checkFontError(Ok, string("Font error: FcConfigSetCurrent failed."));
-        Ok = (int)FcConfigAppFontAddDir(pConfig, (const FcChar8 *)"fonts/");
-        checkFontError(Ok, string("Font error: FcConfigAppFontAddDir failed."));
-/*
-        FcStrList * pCacheDirs = FcConfigGetCacheDirs(pConfig);
-        FcChar8 * pDir;
-        do {
-            pDir = FcStrListNext(pCacheDirs);
-            if (pDir) {
-                cerr << pDir << endl;
-            }
-        } while (pDir);
-*/
-        g_log_set_default_handler(GLibLogFunc, 0);
-
-        s_bInitialized = true;
+        pos=sLowerText.find("<br/>");
     }
+    return sResult;
 }
 
 }
