@@ -1,20 +1,31 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
+#
+# libavg - Media Playback Engine.
+# Copyright (C) 2003-2008 Ulrich von Zadow
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 2 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+# Current versions can be found at www.libavg.de
+#
 
 import sys
 from libavg import avg
 import time
 
-Log = avg.Logger.get();
-Log.setCategories(Log.APP |
-          Log.WARNING | 
-#          Log.PROFILE |
-#          Log.PROFILE_LATEFRAMES |
-#          Log.CONFIG |
-#          Log.EVENTS |
-          0)
-
-Player = avg.Player()
+Player = avg.Player.get()
 
 Player.loadString("""
 <?xml version="1.0"?>
@@ -49,7 +60,7 @@ for variant in variants:
             { "text": text,
               "font": fontname,
               "variant": variant,
-              "size": 24,
+              "fontsize": 24,
               "x": 10,
               "y": y
             })
