@@ -38,14 +38,11 @@ using namespace std;
 namespace avg {
 
 AsyncVideoDecoder::AsyncVideoDecoder(VideoDecoderPtr pSyncDecoder)
-    : m_pSyncDecoder(pSyncDecoder),
+    : m_State(CLOSED),
+      m_pSyncDecoder(pSyncDecoder),
       m_pVDecoderThread(0),
       m_pADecoderThread(0),
-      m_Size(0,0),
-      m_NumFrames(0),
-      m_FPS(0),
       m_PF(NO_PIXELFORMAT),
-      m_Duration(0),
       m_bAudioEOF(false),
       m_bVideoEOF(false),
       m_bSeekPending(false),
@@ -64,33 +61,33 @@ AsyncVideoDecoder::~AsyncVideoDecoder()
     ObjectCounter::get()->decRef(&typeid(*this));
 }
 
-void AsyncVideoDecoder::open(const std::string& sFilename, const AudioParams* pAP,
-        YCbCrMode ycbcrMode, bool bThreadedDemuxer)
+void AsyncVideoDecoder::open(const std::string& sFilename, bool bThreadedDemuxer)
 {
     m_bAudioEOF = false;
     m_bVideoEOF = false;
     m_bSeekPending = false;
     m_sFilename = sFilename;
 
-    m_pSyncDecoder->open(m_sFilename, pAP, ycbcrMode, bThreadedDemuxer);
-    m_bHasVideo = m_pSyncDecoder->hasVideo();
-    m_bHasAudio = m_pSyncDecoder->hasAudio();
-    m_Duration = m_pSyncDecoder->getDuration();
+    m_pSyncDecoder->open(m_sFilename, bThreadedDemuxer);
+    m_VideoInfo = m_pSyncDecoder->getVideoInfo();
+    m_State = OPENED;
+}
 
-    if (m_bHasVideo) {
+void AsyncVideoDecoder::startDecoding(bool bDeliverYCbCr, const AudioParams* pAP)
+{
+    assert(m_State == OPENED);
+    m_pSyncDecoder->startDecoding(bDeliverYCbCr, pAP);
+    m_VideoInfo = m_pSyncDecoder->getVideoInfo();
+    if (m_VideoInfo.m_bHasVideo) {
         m_LastVideoFrameTime = -1000;
-        m_Size = m_pSyncDecoder->getSize();
-        m_NumFrames = m_pSyncDecoder->getNumFrames();
-        m_FPS = m_pSyncDecoder->getFPS();
         m_PF = m_pSyncDecoder->getPixelFormat();
-        m_StreamFPS = m_pSyncDecoder->getNominalFPS();
         m_pVCmdQ = VideoDecoderThread::CmdQueuePtr(new VideoDecoderThread::CmdQueue);
         m_pVMsgQ = VideoMsgQueuePtr(new VideoMsgQueue(8));
         m_pVDecoderThread = new boost::thread(
                  VideoDecoderThread(*m_pVCmdQ, *m_pVMsgQ, m_pSyncDecoder));
     }
     
-    if (m_bHasAudio) {
+    if (m_VideoInfo.m_bHasAudio) {
         m_pACmdQ = AudioDecoderThread::CmdQueuePtr(new AudioDecoderThread::CmdQueue);
         m_pAMsgQ = VideoMsgQueuePtr(new VideoMsgQueue(8));
         m_pADecoderThread = new boost::thread(
@@ -100,10 +97,12 @@ void AsyncVideoDecoder::open(const std::string& sFilename, const AudioParams* pA
         m_LastAudioFrameTime = 0;
         setVolume(m_Volume);
     }
+    m_State = DECODING;
 }
 
 void AsyncVideoDecoder::close()
 {
+    assert(m_State != CLOSED);
     if (m_pVDecoderThread) {
         m_pVCmdQ->push(Command<VideoDecoderThread>(boost::bind(
                 &VideoDecoderThread::stop, _1)));
@@ -129,8 +128,20 @@ void AsyncVideoDecoder::close()
     }        
 }
 
+IVideoDecoder::DecoderState AsyncVideoDecoder::getState() const
+{
+    return m_State;
+}
+
+VideoInfo AsyncVideoDecoder::getVideoInfo() const
+{
+    assert(m_State != CLOSED);
+    return m_VideoInfo;
+}
+
 void AsyncVideoDecoder::seek(long long DestTime)
 {
+    assert(m_State == DECODING);
     waitForSeekDone();
     scoped_lock Lock1(m_AudioMutex);
     scoped_lock Lock2(m_SeekMutex);
@@ -154,93 +165,69 @@ void AsyncVideoDecoder::seek(long long DestTime)
             } else {
                 pMsg = m_pAMsgQ->pop(false);
             }
-            SeekDoneVideoMsgPtr pSeekDoneMsg = dynamic_pointer_cast<SeekDoneVideoMsg>(pMsg);
+            SeekDoneVideoMsgPtr pSeekDoneMsg = 
+                    dynamic_pointer_cast<SeekDoneVideoMsg>(pMsg);
             if (pSeekDoneMsg) {
                 m_bSeekPending = false;
                 m_LastVideoFrameTime = pSeekDoneMsg->getVideoFrameTime();
                 m_LastAudioFrameTime = pSeekDoneMsg->getAudioFrameTime();
             }
+            FrameVideoMsgPtr pFrameMsg = dynamic_pointer_cast<FrameVideoMsg>(pMsg);
+            if (pFrameMsg) {
+                returnFrame(pFrameMsg);
+            }
         }
     } catch (Exception&) {
     }
 }
 
-StreamSelect AsyncVideoDecoder::getMasterStream()
+IntPoint AsyncVideoDecoder::getSize() const
 {
-    if (m_bHasAudio) {
-        return SS_AUDIO;
-    } else {
-        return SS_VIDEO;
-    }
+    assert(m_State != CLOSED);
+    return m_VideoInfo.m_Size;
 }
 
-bool AsyncVideoDecoder::hasVideo()
+int AsyncVideoDecoder::getCurFrame() const
 {
-    return m_bHasVideo;
+    assert(m_State == DECODING);
+    return int(getCurTime(SS_VIDEO)*m_VideoInfo.m_StreamFPS/1000.0+0.5);
 }
 
-bool AsyncVideoDecoder::hasAudio()
-{
-    return m_bHasAudio;
-}
-
-IntPoint AsyncVideoDecoder::getSize()
-{
-    return m_Size;
-}
-
-int AsyncVideoDecoder::getCurFrame()
-{
-    return int(getCurTime(SS_VIDEO)*m_StreamFPS/1000.0+0.5);
-}
-
-int AsyncVideoDecoder::getNumFrames()
-{
-    if (m_NumFrames == 0) {
-        throw Exception(AVG_ERR_VIDEO_GENERAL, "Error in AsyncVideoDecoder::getNumFrames: Video not loaded.");
-    }
-    return m_NumFrames;
-}
-
-int AsyncVideoDecoder::getNumFramesQueued()
+int AsyncVideoDecoder::getNumFramesQueued() const
 {
+    assert(m_State == DECODING);
     return m_pVMsgQ->size();
 }
 
-long long AsyncVideoDecoder::getCurTime(StreamSelect Stream)
+long long AsyncVideoDecoder::getCurTime(StreamSelect Stream) const
 {
+    assert(m_State == DECODING);
     switch(Stream) {
+        case SS_DEFAULT:
         case SS_VIDEO:
-            assert(m_bHasVideo);
+            assert(m_VideoInfo.m_bHasVideo);
             return m_LastVideoFrameTime;
             break;
         case SS_AUDIO:
-            assert(m_bHasAudio);
+            assert(m_VideoInfo.m_bHasAudio);
             return m_LastAudioFrameTime;
             break;
-        case SS_DEFAULT:
-            return getCurTime(getMasterStream());
-            break;
         default:
             assert(false);
     }
     return -1;
 }
 
-long long AsyncVideoDecoder::getDuration()
+double AsyncVideoDecoder::getNominalFPS() const
 {
-    return m_Duration;
+    assert(m_State != CLOSED);
+    return m_VideoInfo.m_StreamFPS;
 }
 
-double AsyncVideoDecoder::getNominalFPS()
+double AsyncVideoDecoder::getFPS() const
 {
-    return m_StreamFPS;
-}
-
-double AsyncVideoDecoder::getFPS()
-{
-    assert(m_pVDecoderThread);
-    return m_FPS;
+    assert(m_State != CLOSED);
+    return m_VideoInfo.m_FPS;
 }
 
 void AsyncVideoDecoder::setFPS(double FPS)
@@ -249,60 +236,66 @@ void AsyncVideoDecoder::setFPS(double FPS)
     m_pVCmdQ->push(Command<VideoDecoderThread>(boost::bind(
             &VideoDecoderThread::setFPS, _1, FPS)));
     if (FPS != 0) {
-        m_FPS = FPS;
+        m_VideoInfo.m_FPS = FPS;
     }
 }
 
-double AsyncVideoDecoder::getVolume()
+double AsyncVideoDecoder::getVolume() const
 {
+    assert(m_State != CLOSED);
     return m_Volume;
 }
 
 void AsyncVideoDecoder::setVolume(double Volume)
 {
     m_Volume = Volume;
-    if (m_bHasAudio && m_pACmdQ) {
+    if (m_VideoInfo.m_bHasAudio && m_pACmdQ) {
         m_pACmdQ->push(Command<AudioDecoderThread>(boost::bind(
                 &AudioDecoderThread::setVolume, _1, Volume)));
     }
 }
 
-PixelFormat AsyncVideoDecoder::getPixelFormat()
+PixelFormat AsyncVideoDecoder::getPixelFormat() const
 {
-    assert(m_pVDecoderThread);
+    assert(m_State == DECODING);
     return m_PF;
 }
 
-FrameAvailableCode AsyncVideoDecoder::renderToBmp(BitmapPtr pBmp, long long TimeWanted)
+FrameAvailableCode AsyncVideoDecoder::renderToBmp(BitmapPtr pBmp, long long timeWanted)
 {
+    assert(m_State == DECODING);
     FrameAvailableCode FrameAvailable;
-    FrameVideoMsgPtr pFrameMsg = getBmpsForTime(TimeWanted, FrameAvailable);
+    FrameVideoMsgPtr pFrameMsg = getBmpsForTime(timeWanted, FrameAvailable);
     if (FrameAvailable == FA_NEW_FRAME) {
         pBmp->copyPixels(*(pFrameMsg->getBitmap(0)));
+        returnFrame(pFrameMsg);
     }
     return FrameAvailable;
 }
 
-FrameAvailableCode AsyncVideoDecoder::renderToYCbCr420p(BitmapPtr pBmpY, BitmapPtr pBmpCb, 
-       BitmapPtr pBmpCr, long long TimeWanted)
+FrameAvailableCode AsyncVideoDecoder::renderToYCbCr420p(BitmapPtr pBmpY, BitmapPtr pBmpCb,
+       BitmapPtr pBmpCr, long long timeWanted)
 {
+    assert(m_State == DECODING);
     FrameAvailableCode FrameAvailable;
-    FrameVideoMsgPtr pFrameMsg = getBmpsForTime(TimeWanted, FrameAvailable);
+    FrameVideoMsgPtr pFrameMsg = getBmpsForTime(timeWanted, FrameAvailable);
     if (FrameAvailable == FA_NEW_FRAME) {
         pBmpY->copyPixels(*(pFrameMsg->getBitmap(0)));
         pBmpCb->copyPixels(*(pFrameMsg->getBitmap(1)));
         pBmpCr->copyPixels(*(pFrameMsg->getBitmap(2)));
+        returnFrame(pFrameMsg);
     }
     return FrameAvailable;
 }
 
-bool AsyncVideoDecoder::isEOF(StreamSelect Stream)
+bool AsyncVideoDecoder::isEOF(StreamSelect Stream) const
 {
+    assert(m_State == DECODING);
     switch(Stream) {
         case SS_AUDIO:
-            return (!m_bHasAudio || m_bAudioEOF);
+            return (!m_VideoInfo.m_bHasAudio || m_bAudioEOF);
         case SS_VIDEO:
-            return (!m_bHasVideo || m_bVideoEOF);
+            return (!m_VideoInfo.m_bHasVideo || m_bVideoEOF);
         case SS_ALL:
             return isEOF(SS_VIDEO) && isEOF(SS_AUDIO);
         default:
@@ -310,8 +303,16 @@ bool AsyncVideoDecoder::isEOF(StreamSelect Stream)
     }
 }
 
+void AsyncVideoDecoder::throwAwayFrame(long long timeWanted)
+{
+    assert(m_State == DECODING);
+    FrameAvailableCode FrameAvailable;
+    FrameVideoMsgPtr pFrameMsg = getBmpsForTime(timeWanted, FrameAvailable);
+}
+
 int AsyncVideoDecoder::fillAudioBuffer(AudioBufferPtr pBuffer)
 {
+    assert(m_State == DECODING);
     assert (m_pADecoderThread);
     if (m_bAudioEOF) {
         return 0;
@@ -360,48 +361,38 @@ int AsyncVideoDecoder::fillAudioBuffer(AudioBufferPtr pBuffer)
     return pBuffer->getNumFrames();
 }
         
-FrameVideoMsgPtr AsyncVideoDecoder::getBmpsForTime(long long TimeWanted, 
+FrameVideoMsgPtr AsyncVideoDecoder::getBmpsForTime(long long timeWanted, 
         FrameAvailableCode& FrameAvailable)
 {
     // XXX: This code is sort-of duplicated in FFMpegDecoder::readFrameForTime()
     long long FrameTime = -1000;
     FrameVideoMsgPtr pFrameMsg;
-    if (TimeWanted == -1) {
+    if (timeWanted == -1) {
         pFrameMsg = getNextBmps(true);
         FrameAvailable = FA_NEW_FRAME;
     } else {
-        if (getMasterStream() == SS_AUDIO) {
-            TimeWanted = m_LastAudioFrameTime;
-        }
-
-//        cerr << "getBmpsForTime " << TimeWanted << ", LastFrameTime= " << m_LastVideoFrameTime 
-//                << ", diff= " << TimeWanted-m_LastVideoFrameTime <<  endl;
         double TimePerFrame = 1000.0/getFPS();
-        if (fabs(double(TimeWanted-m_LastVideoFrameTime)) < 0.5*TimePerFrame || 
-                m_LastVideoFrameTime > TimeWanted+TimePerFrame) {
-//            cerr << "   LastFrameTime = " << m_LastVideoFrameTime << ", display again." <<  endl;
+        if (fabs(double(timeWanted-m_LastVideoFrameTime)) < 0.5*TimePerFrame || 
+                m_LastVideoFrameTime > timeWanted+TimePerFrame) {
             // The last frame is still current. Display it again.
             FrameAvailable = FA_USE_LAST_FRAME;
             return FrameVideoMsgPtr();
         } else {
             if (m_bVideoEOF) {
-//                cerr << "  EOF." << endl;
                 FrameAvailable = FA_USE_LAST_FRAME;
                 return FrameVideoMsgPtr();
             }
-            while (FrameTime-TimeWanted < -0.5*TimePerFrame && !m_bVideoEOF) {
+            while (FrameTime-timeWanted < -0.5*TimePerFrame && !m_bVideoEOF) {
+                returnFrame(pFrameMsg);
                 pFrameMsg = getNextBmps(false);
                 if (pFrameMsg) {
                     FrameTime = pFrameMsg->getFrameTime();
-//                    cerr << "   readFrame returned time " << FrameTime << "." <<  endl;
                 } else {
-//                    cerr << "   no frame available." <<  endl;
                     FrameAvailable = FA_STILL_DECODING;
                     return FrameVideoMsgPtr();
                 }
             }
             FrameAvailable = FA_NEW_FRAME;
-//            cerr << "  frame ok." << endl;
         }
     }
     if (pFrameMsg) {
@@ -455,9 +446,20 @@ void AsyncVideoDecoder::waitForSeekDone()
                 m_LastVideoFrameTime = pSeekDoneMsg->getVideoFrameTime();
                 m_LastAudioFrameTime = pSeekDoneMsg->getAudioFrameTime();
             }
-
+            FrameVideoMsgPtr pFrameMsg = dynamic_pointer_cast<FrameVideoMsg>(pMsg);
+            if (pFrameMsg) {
+                returnFrame(pFrameMsg);
+            }
         } while (m_bSeekPending);
     }
 }
 
+void AsyncVideoDecoder::returnFrame(FrameVideoMsgPtr& pFrameMsg)
+{
+    if (pFrameMsg) {
+        m_pVCmdQ->push(Command<VideoDecoderThread>(boost::bind(
+                    &VideoDecoderThread::returnFrame, _1, pFrameMsg)));
+    }
+}
+
 }
