@@ -22,6 +22,7 @@
 #ifndef _AsyncVideoDecoder_H_
 #define _AsyncVideoDecoder_H_
 
+#include "../api.h"
 #include "IVideoDecoder.h"
 #include "VideoDecoderThread.h"
 #include "AudioDecoderThread.h"
@@ -37,44 +38,44 @@
 
 namespace avg {
 
-class AsyncVideoDecoder: public IVideoDecoder
+class AVG_API AsyncVideoDecoder: public IVideoDecoder
 {
     public:
         AsyncVideoDecoder(VideoDecoderPtr pSyncDecoder);
         virtual ~AsyncVideoDecoder();
-        virtual void open(const std::string& sFilename, const AudioParams* AP,
-                YCbCrMode ycbcrMode, bool bSyncDemuxer);
+        virtual void open(const std::string& sFilename, bool bSyncDemuxer);
+        virtual void startDecoding(bool bDeliverYCbCr, const AudioParams* AP);
         virtual void close();
+        virtual DecoderState getState() const;
+        virtual VideoInfo getVideoInfo() const;
         virtual void seek(long long DestTime);
-        virtual StreamSelect getMasterStream();
-        virtual bool hasVideo();
-        virtual bool hasAudio();
-        virtual IntPoint getSize();
-        virtual int getCurFrame();
-        virtual int getNumFrames();
-        virtual int getNumFramesQueued();
-        virtual long long getCurTime(StreamSelect Stream = SS_DEFAULT);
-        virtual long long getDuration();
-        virtual double getNominalFPS();
-        virtual double getFPS();
+        virtual IntPoint getSize() const;
+        virtual int getCurFrame() const;
+        virtual int getNumFramesQueued() const;
+        virtual long long getCurTime(StreamSelect Stream = SS_DEFAULT) const;
+        virtual double getNominalFPS() const;
+        virtual double getFPS() const;
         virtual void setFPS(double FPS);
-        virtual double getVolume();
+        virtual double getVolume() const;
         virtual void setVolume(double Volume);
-        virtual PixelFormat getPixelFormat();
+        virtual PixelFormat getPixelFormat() const;
 
-        virtual FrameAvailableCode renderToBmp(BitmapPtr pBmp, long long TimeWanted);
+        virtual FrameAvailableCode renderToBmp(BitmapPtr pBmp, long long timeWanted);
         virtual FrameAvailableCode renderToYCbCr420p(BitmapPtr pBmpY, BitmapPtr pBmpCb, 
-                BitmapPtr pBmpCr, long long TimeWanted);
-        virtual bool isEOF(StreamSelect Stream = SS_ALL);
+                BitmapPtr pBmpCr, long long timeWanted);
+        virtual bool isEOF(StreamSelect Stream = SS_ALL) const;
+        virtual void throwAwayFrame(long long timeWanted);
         
         virtual int fillAudioBuffer(AudioBufferPtr pBuffer);
         
     private:
-        FrameVideoMsgPtr getBmpsForTime(long long TimeWanted, 
+        FrameVideoMsgPtr getBmpsForTime(long long timeWanted, 
                 FrameAvailableCode& FrameAvailable);
         FrameVideoMsgPtr getNextBmps(bool bWait);
         void waitForSeekDone();
+        void returnFrame(FrameVideoMsgPtr& pFrameMsg);
 
+        DecoderState m_State;
         VideoDecoderPtr m_pSyncDecoder;
         std::string m_sFilename;
 
@@ -90,16 +91,12 @@ class AsyncVideoDecoder: public IVideoDecoder
         unsigned char* m_AudioMsgData;
         int m_AudioMsgSize;
 
-        bool m_bHasAudio;
-        bool m_bHasVideo;
+        VideoInfo m_VideoInfo;
 
         IntPoint m_Size;
         int m_NumFrames;
         bool m_bUseStreamFPS;
-        double m_StreamFPS;
-        double m_FPS;
         PixelFormat m_PF;
-        long long m_Duration;
         
         bool m_bAudioEOF;
         bool m_bVideoEOF;
