@@ -29,6 +29,7 @@
 #include "../base/ObjectCounter.h"
 
 #include "../graphics/Filterflipuv.h"
+#include "../graphics/Filterfliprgba.h"
 
 #include <iostream>
 #include <sstream>
@@ -43,16 +44,21 @@ using namespace boost;
 #define SAMPLE_BUFFER_SIZE ((AVCODEC_MAX_AUDIO_FRAME_SIZE*3))
 #define VOLUME_FADE_SAMPLES 100
 
+#if LIBAVFORMAT_BUILD < ((50<<16)+(0<<8)+0)
+#define PIX_FMT_BGRA PIX_FMT_RGBA32
+#define PIX_FMT_YUYV422 PIX_FMT_YUV422
+#endif
+
 namespace avg {
 
 bool FFMpegDecoder::m_bInitialized = false;
 mutex FFMpegDecoder::s_OpenMutex;
 
 FFMpegDecoder::FFMpegDecoder ()
-    : m_pFormatContext(0),
-#ifdef AVG_ENABLE_SWSCALE
+    : m_State(CLOSED),
+      m_pFormatContext(0),
+      m_PF(NO_PIXELFORMAT),
       m_pSwsContext(0),
-#endif
       m_Size(0,0),
       m_bUseStreamFPS(true),
       m_AStreamIndex(-1),
@@ -99,7 +105,7 @@ void avcodecError(const string & sFilename, int err)
                     sFilename + ": Unknown format");
         default:
             stringstream s;
-            s << sFilename << ": Error while opening file (Num:" << err << ")";
+            s << "'" << sFilename <<  "': Error while opening file (Num:" << err << ")";
             throw Exception(AVG_ERR_VIDEO_INIT_FAILED, s.str());
     }
 }
@@ -140,15 +146,10 @@ int openCodec(AVFormatContext *formatContext, int streamIndex)
     return 0;
 }
 
-
-void FFMpegDecoder::open(const std::string& sFilename, const AudioParams* pAP,
-        YCbCrMode ycbcrMode, bool bThreadedDemuxer)
+void FFMpegDecoder::open(const std::string& sFilename, bool bThreadedDemuxer)
 {
     mutex::scoped_lock Lock(s_OpenMutex);
-    bool bAudioEnabled = (pAP && bThreadedDemuxer);
-    if (bAudioEnabled) {
-        m_AP = *pAP;
-    }
+    m_bThreadedDemuxer = bThreadedDemuxer;
     m_bAudioEOF = false;
     m_bVideoEOF = false;
     m_bEOFPending = false;
@@ -193,7 +194,7 @@ void FFMpegDecoder::open(const std::string& sFilename, const AudioParams* pAP,
                 break;
             case CODEC_TYPE_AUDIO:
                 // Ignore the audio stream if we're using sync demuxing. 
-                if (m_AStreamIndex < 0 && bAudioEnabled) {
+                if (m_AStreamIndex < 0 && bThreadedDemuxer) {
                     m_AStreamIndex = i;
                 }
                 break;
@@ -204,10 +205,6 @@ void FFMpegDecoder::open(const std::string& sFilename, const AudioParams* pAP,
     assert(!m_pDemuxer);
 //    dump_format(m_pFormatContext, 0, m_sFilename.c_str(), 0);
 //    dump_stream_info(m_pFormatContext);
-    if (m_VStreamIndex < 0 && m_AStreamIndex < 0) {
-        throw Exception(AVG_ERR_VIDEO_INIT_FAILED, 
-                sFilename + " does not contain any audio or video streams.");
-    }
     
     // Create demuxer to handle streams
     if (bThreadedDemuxer) {
@@ -220,6 +217,7 @@ void FFMpegDecoder::open(const std::string& sFilename, const AudioParams* pAP,
     if(m_VStreamIndex >= 0)
     {
         m_pVStream = m_pFormatContext->streams[m_VStreamIndex];
+        m_State = OPENED;
         m_pDemuxer->enableStream(m_VStreamIndex);
         
         // Set video parameters
@@ -227,16 +225,15 @@ void FFMpegDecoder::open(const std::string& sFilename, const AudioParams* pAP,
         if (m_bUseStreamFPS) {
             m_FPS = getNominalFPS();
         }
-    #if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
+#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
         m_Size = IntPoint(m_pVStream->codec.width, m_pVStream->codec.height);
-    #else
+#else
         m_Size = IntPoint(m_pVStream->codec->width, m_pVStream->codec->height);
-    #endif
+#endif
         m_bFirstPacket = true;
         m_PacketLenLeft = 0;
         m_sFilename = sFilename;
         m_LastVideoFrameTime = -1000;
-        m_PF = calcPixelFormat(ycbcrMode);
     
         int rc = openCodec(m_pFormatContext, m_VStreamIndex);
         if (rc == -1) {
@@ -283,10 +280,41 @@ void FFMpegDecoder::open(const std::string& sFilename, const AudioParams* pAP,
             m_pAStream = 0; 
             AVG_TRACE(Logger::WARNING, 
                     sFilename + ": unsupported codec ("+szBuf+"). Disabling audio.");
+        }
+    }
+    m_State = OPENED;
+}
+
+void FFMpegDecoder::startDecoding(bool bDeliverYCbCr, const AudioParams* pAP)
+{
+    assert(m_State == OPENED);
+    if (m_VStreamIndex >= 0) {
+        m_PF = calcPixelFormat(bDeliverYCbCr);
+    }
+    bool bAudioEnabled = (pAP && m_bThreadedDemuxer);
+    if (bAudioEnabled) {
+        m_AP = *pAP;
+    } else {
+        m_AStreamIndex = -1;
+        m_pAStream = 0;
+    }
+    if (m_AStreamIndex >= 0)
+    {
+        if (m_pAStream->codec->channels > m_AP.m_Channels) {
+            AVG_TRACE(Logger::WARNING, 
+                    m_sFilename << ": unsupported number of channels (" << 
+                            m_pAStream->codec->channels << "). Disabling audio.");
+            m_AStreamIndex = -1;
+            m_pAStream = 0; 
         } else {
             m_pDemuxer->enableStream(m_AStreamIndex);
         }
     }
+    if (m_VStreamIndex < 0 && m_AStreamIndex < 0) {
+        throw Exception(AVG_ERR_VIDEO_INIT_FAILED, 
+                m_sFilename + " does not contain any valid audio or video streams.");
+    }
+    m_State = DECODING;
 }
 
 void FFMpegDecoder::close() 
@@ -354,16 +382,48 @@ void FFMpegDecoder::close()
         m_pFormatContext = 0;
     }
     
-#ifdef AVG_ENABLE_SWSCALE
     if(m_pSwsContext) {
         sws_freeContext(m_pSwsContext);
         m_pSwsContext = 0;
     }
-#endif
+    m_State = CLOSED;
+}
+
+IVideoDecoder::DecoderState FFMpegDecoder::getState() const
+{
+    return m_State;
+}
+
+VideoInfo FFMpegDecoder::getVideoInfo() const
+{
+    assert(m_State != CLOSED);
+    long long duration = 0;
+    if (m_pVStream || m_pAStream) {
+        duration = getDuration();
+    }
+    VideoInfo info(duration, m_pFormatContext->bit_rate, m_pVStream != 0,
+            m_pAStream != 0);
+    if (m_pVStream) {
+        info.setVideoData(m_Size, getStreamPF(), getNumFrames(), getNominalFPS(), m_FPS,
+                m_pVStream->codec->codec->name);
+    }
+    if (m_pAStream) {
+        AVCodecContext * pACodec = m_pAStream->codec;
+        info.setAudioData(pACodec->codec->name, pACodec->sample_rate,
+                pACodec->channels);
+    }
+    return info;
 }
 
 void FFMpegDecoder::seek(long long DestTime) 
 {
+    assert(m_State == DECODING);
+/* XXX: Causes audio hangs.
+    if (DestTime == 0 && m_LastVideoFrameTime == -(long long)(1000.0/m_FPS)) {
+        // Hack to improve performance when looping videos
+        return;
+    }
+*/
     if (m_bFirstPacket && m_pVStream) {
         AVFrame Frame;
         long long FrameTime;
@@ -383,73 +443,50 @@ void FFMpegDecoder::seek(long long DestTime)
     m_bAudioEOF = false;
 }
 
-bool FFMpegDecoder::hasVideo()
-{
-    return (m_pVStream != 0);
-}
-
-bool FFMpegDecoder::hasAudio()
-{
-    return (m_pAStream != 0);
-}
-
-IntPoint FFMpegDecoder::getSize()
+IntPoint FFMpegDecoder::getSize() const
 {
+    assert(m_State != CLOSED);
     return m_Size;
 }
 
-int FFMpegDecoder::getCurFrame()
+int FFMpegDecoder::getCurFrame() const
 {
+    assert(m_State == DECODING);
     return int(getCurTime(SS_VIDEO)*getNominalFPS()/1000.0+0.5);
 }
 
-int FFMpegDecoder::getNumFrames()
-{
-    if (!m_pVStream) {
-        throw Exception(AVG_ERR_VIDEO_GENERAL, "Error in FFMpegDecoder::getNumFrames: Video not loaded.");
-    }
-    // This is broken for some videos, but the code here is correct.
-    // So fix ffmpeg :-).
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-    return m_pVStream->r_frame_rate*(m_pVStream->duration/AV_TIME_BASE);
-#else
-    double timeUnitsPerSecond = 1/av_q2d(m_pVStream->time_base);
-    return (m_pVStream->r_frame_rate.num/m_pVStream->r_frame_rate.den)*
-            int(m_pVStream->duration/timeUnitsPerSecond);
-#endif 
-}
-
-int FFMpegDecoder::getNumFramesQueued()
+int FFMpegDecoder::getNumFramesQueued() const
 {
     return 0;
 }
 
-long long FFMpegDecoder::getCurTime(StreamSelect Stream)
+long long FFMpegDecoder::getCurTime(StreamSelect Stream) const
 {
+    assert(m_State == DECODING);
     switch(Stream) {
+        case SS_DEFAULT:
         case SS_VIDEO:
             assert(m_pVStream);
             return m_LastVideoFrameTime;
         case SS_AUDIO:
             assert(m_pAStream);
             return (long long)m_LastAudioFrameTime;
-        case SS_DEFAULT:
-            return getCurTime(getMasterStream());
         default:
             return -1;
     }
 }
 
-long long FFMpegDecoder::getDuration()
+long long FFMpegDecoder::getDuration() const
 {
+    assert(m_State != CLOSED);
     long long duration;
     AVRational time_base;
-    if (getMasterStream() == SS_AUDIO) {
-        duration = m_pAStream->duration;
-        time_base=m_pAStream->time_base;
-    } else {
-        duration = m_pVStream->duration;
+    if (m_pVStream) {
+        duration=m_pVStream->duration;
         time_base=m_pVStream->time_base;
+    } else {
+        duration=m_pAStream->duration;
+        time_base=m_pAStream->time_base;
     }
 #if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
     return (long long)(1000*duration/AV_TIME_BASE);
@@ -458,8 +495,9 @@ long long FFMpegDecoder::getDuration()
 #endif 
 }
 
-double FFMpegDecoder::getNominalFPS()
+double FFMpegDecoder::getNominalFPS() const
 {
+    assert(m_State != CLOSED);
 #if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
     return m_pVStream->r_frame_rate;
 #else
@@ -467,8 +505,9 @@ double FFMpegDecoder::getNominalFPS()
 #endif 
 }
 
-double FFMpegDecoder::getFPS()
+double FFMpegDecoder::getFPS() const
 {
+    assert(m_State != CLOSED);
     return m_FPS;
 }
 
@@ -482,8 +521,9 @@ void FFMpegDecoder::setFPS(double FPS)
     }
 }
 
-double FFMpegDecoder::getVolume()
+double FFMpegDecoder::getVolume() const
 {
+    assert(m_State != CLOSED);
     return m_Volume;
 }
 
@@ -492,11 +532,12 @@ void FFMpegDecoder::setVolume(double Volume)
     m_Volume = Volume;
 }
 
-FrameAvailableCode FFMpegDecoder::renderToBmp(BitmapPtr pBmp, long long TimeWanted)
+FrameAvailableCode FFMpegDecoder::renderToBmp(BitmapPtr pBmp, long long timeWanted)
 {
+    assert(m_State == DECODING);
 //    ScopeTimer Timer(*m_pRenderToBmpProfilingZone);
     AVFrame Frame;
-    FrameAvailableCode FrameAvailable = readFrameForTime(Frame, TimeWanted);
+    FrameAvailableCode FrameAvailable = readFrameForTime(Frame, timeWanted);
     if (!m_bVideoEOF && FrameAvailable == FA_NEW_FRAME) {
         convertFrameToBmp(Frame, pBmp);
         return FA_NEW_FRAME;
@@ -520,11 +561,12 @@ void copyPlaneToBmp(BitmapPtr pBmp, unsigned char * pData, int Stride)
 }
 
 FrameAvailableCode FFMpegDecoder::renderToYCbCr420p(BitmapPtr pBmpY, BitmapPtr pBmpCb, 
-        BitmapPtr pBmpCr, long long TimeWanted)
+        BitmapPtr pBmpCr, long long timeWanted)
 {
+    assert(m_State == DECODING);
 //    ScopeTimer Timer(*m_pRenderToBmpProfilingZone);
     AVFrame Frame;
-    FrameAvailableCode FrameAvailable = readFrameForTime(Frame, TimeWanted);
+    FrameAvailableCode FrameAvailable = readFrameForTime(Frame, timeWanted);
     if (!m_bVideoEOF && FrameAvailable == FA_NEW_FRAME) {
 //        ScopeTimer Timer(*m_pConvertImageProfilingZone);
         copyPlaneToBmp(pBmpY, Frame.data[0], Frame.linesize[0]);
@@ -535,8 +577,16 @@ FrameAvailableCode FFMpegDecoder::renderToYCbCr420p(BitmapPtr pBmpY, BitmapPtr p
     return FA_USE_LAST_FRAME;
 }
 
-bool FFMpegDecoder::isEOF(StreamSelect Stream)
+void FFMpegDecoder::throwAwayFrame(long long timeWanted)
 {
+    assert(m_State == DECODING);
+    AVFrame Frame;
+    readFrameForTime(Frame, timeWanted);
+}
+
+bool FFMpegDecoder::isEOF(StreamSelect Stream) const
+{
+    assert(m_State == DECODING);
     switch(Stream) {
         case SS_AUDIO:
             return (!m_pAStream || m_bAudioEOF);
@@ -668,6 +718,7 @@ int FFMpegDecoder::decodeAudio()
 
 int FFMpegDecoder::fillAudioBuffer(AudioBufferPtr pBuffer)
 {
+    assert(m_State == DECODING);
     mutex::scoped_lock Lock(m_AudioMutex);
 
     unsigned char* outputAudioBuffer = (unsigned char*)(pBuffer->getData());
@@ -753,14 +804,14 @@ int FFMpegDecoder::fillAudioBuffer(AudioBufferPtr pBuffer)
     }
 }
 
-PixelFormat FFMpegDecoder::calcPixelFormat(YCbCrMode ycbcrMode)
+PixelFormat FFMpegDecoder::calcPixelFormat(bool bUseYCbCr)
 {
 #if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
         AVCodecContext *enc = &m_pVStream->codec;
 #else
         AVCodecContext *enc = m_pVStream->codec;
 #endif
-    if (ycbcrMode == OGL_SHADER) {
+    if (bUseYCbCr) {
         switch(enc->pix_fmt) {
             case PIX_FMT_YUV420P:
                 return YCbCr420p;
@@ -770,12 +821,7 @@ PixelFormat FFMpegDecoder::calcPixelFormat(YCbCrMode ycbcrMode)
                 break;
         }
     }
-    if ((ycbcrMode == OGL_MESA || ycbcrMode == OGL_APPLE) &&
-         enc->pix_fmt == PIX_FMT_YUV420P) 
-    {
-        return YCbCr422;
-    }
-    if (enc->pix_fmt == PIX_FMT_RGBA32) {
+    if (enc->pix_fmt == PIX_FMT_BGRA) {
         return B8G8R8A8;
     }
     return B8G8R8X8;
@@ -787,16 +833,20 @@ void FFMpegDecoder::convertFrameToBmp(AVFrame& Frame, BitmapPtr pBmp)
     unsigned char * pDestBits = pBmp->getPixels();
     DestPict.data[0] = pDestBits;
     DestPict.linesize[0] = pBmp->getStride();
+#if LIBAVFORMAT_BUILD >= ((50<<16)+(0<<8)+0)
+    ::PixelFormat DestFmt;
+#else
     int DestFmt;
+#endif
     switch(pBmp->getPixelFormat()) {
         case R8G8B8X8:
         case R8G8B8A8:
             // XXX: Unused and broken.
-            DestFmt = PIX_FMT_RGBA32;
+            DestFmt = PIX_FMT_BGRA;
             break;
         case B8G8R8X8:
         case B8G8R8A8:
-            DestFmt = PIX_FMT_RGBA32;
+            DestFmt = PIX_FMT_BGRA;
             break;
         case R8G8B8:
             DestFmt = PIX_FMT_RGB24;
@@ -805,7 +855,7 @@ void FFMpegDecoder::convertFrameToBmp(AVFrame& Frame, BitmapPtr pBmp)
             DestFmt = PIX_FMT_BGR24;
             break;
         case YCbCr422:
-            DestFmt = PIX_FMT_YUV422;
+            DestFmt = PIX_FMT_YUYV422;
             break;
         default:
             AVG_TRACE(Logger::ERROR, "FFMpegDecoder: Dest format " 
@@ -818,31 +868,47 @@ void FFMpegDecoder::convertFrameToBmp(AVFrame& Frame, BitmapPtr pBmp)
     AVCodecContext *enc = m_pVStream->codec;
 #endif
     {
-//        ScopeTimer Timer(*m_pConvertImageProfilingZone);
-#ifdef AVG_ENABLE_SWSCALE
-        if (!m_pSwsContext) {
-            m_pSwsContext = sws_getContext(enc->width, enc->height, enc->pix_fmt,
-                        enc->width, enc->height, DestFmt, SWS_BICUBIC, 
-                        NULL, NULL, NULL);
-            if(!m_pSwsContext) {
-                AVG_TRACE(Logger::ERROR, "FFMpegDecoder: sws initialization failed.");
+//            ScopeTimer Timer(*m_pConvertImageProfilingZone);
+        if (DestFmt == PIX_FMT_BGRA && enc->pix_fmt == PIX_FMT_YUV420P) {
+            BitmapPtr pBmpY(new Bitmap(pBmp->getSize(), I8, Frame.data[0],
+                    Frame.linesize[0], false));
+            BitmapPtr pBmpU(new Bitmap(pBmp->getSize(), I8, Frame.data[1],
+                    Frame.linesize[1], false));
+            BitmapPtr pBmpV(new Bitmap(pBmp->getSize(), I8, Frame.data[2],
+                    Frame.linesize[2], false));
+            pBmp->copyYUVPixels(*pBmpY, *pBmpU, *pBmpV);
+        } else {
+            if (!m_pSwsContext) {
+                m_pSwsContext = sws_getContext(enc->width, enc->height, enc->pix_fmt,
+                            enc->width, enc->height, DestFmt, SWS_BICUBIC, 
+                            NULL, NULL, NULL);
+                if(!m_pSwsContext) {
+                    AVG_TRACE(Logger::ERROR, "FFMpegDecoder: sws initialization failed.");
+                }
+            }
+            sws_scale(m_pSwsContext, Frame.data, Frame.linesize, 0, 
+                enc->height, DestPict.data, DestPict.linesize);
+            if (pBmp->getPixelFormat() == B8G8R8X8) {
+                // Make sure the alpha channel is white.
+                // TODO: This is slow. Make OpenGL do it.
+                unsigned char * pLine = pBmp->getPixels();
+                IntPoint size = pBmp->getSize();
+                for (int y = 0; y<size.y; ++y) {
+                    unsigned char * pPixel = pLine;
+                    for (int x = 0; x < size.x; ++x) {
+                        pPixel[3] = 0xFF;
+                        pPixel += 4;
+                    }
+                    pLine = pLine + pBmp->getStride();
+                }
             }
         }
-        sws_scale(m_pSwsContext, Frame.data, Frame.linesize, 0, 
-            enc->height, DestPict.data, DestPict.linesize);
-#else
-        int rc = img_convert(&DestPict, DestFmt,
-                (AVPicture*)&Frame, enc->pix_fmt,
-                enc->width, enc->height);
-        if (rc != 0) {
-            AVG_TRACE(Logger::ERROR, "FFFMpegDecoder: img_convert failed.");
-        }
-#endif
     }
 }
        
-PixelFormat FFMpegDecoder::getPixelFormat()
+PixelFormat FFMpegDecoder::getPixelFormat() const
 {
+    assert(m_State != CLOSED);
     return m_PF;
 }
 
@@ -857,24 +923,39 @@ void FFMpegDecoder::initVideoSupport()
     }
 }
 
-FrameAvailableCode FFMpegDecoder::readFrameForTime(AVFrame& Frame, long long TimeWanted)
+int FFMpegDecoder::getNumFrames() const
 {
+    assert(m_State != CLOSED);
+    // This is broken for some videos, but the code here is correct.
+    // So fix ffmpeg :-).
+#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
+    return m_pVStream->r_frame_rate*(m_pVStream->duration/AV_TIME_BASE);
+#else
+    double timeUnitsPerSecond = 1/av_q2d(m_pVStream->time_base);
+    return int((m_pVStream->r_frame_rate.num/m_pVStream->r_frame_rate.den)*
+            (m_pVStream->duration/timeUnitsPerSecond));
+#endif 
+}
+
+FrameAvailableCode FFMpegDecoder::readFrameForTime(AVFrame& Frame, long long timeWanted)
+{
+    assert(m_State == DECODING);
     // XXX: This code is sort-of duplicated in AsyncVideoDecoder::getBmpsForTime()
     long long FrameTime = -1000;
 
 /*
     bool bDebug = (m_sFilename == "/home/uzadow/wos_videos/c-wars/thumbs/cwars-scene3.avi");
     if (bDebug) {
-        cerr << m_sFilename << " readFrameForTime " << TimeWanted 
+        cerr << m_sFilename << " readFrameForTime " << timeWanted 
                 << ", LastFrameTime= " << m_LastFrameTime << ", diff= " 
-                << TimeWanted-m_LastFrameTime << endl;
+                << timeWanted-m_LastFrameTime << endl;
     }
 */    
-    if (TimeWanted == -1) {
+    if (timeWanted == -1) {
         readFrame(Frame, FrameTime);
     } else {
         double TimePerFrame = 1000/m_FPS;
-        if (TimeWanted-m_LastVideoFrameTime < 0.5*TimePerFrame) {
+        if (timeWanted-m_LastVideoFrameTime < 0.5*TimePerFrame) {
 /*            
             if (bDebug) {
                 cerr << "   LastFrameTime = " << m_LastFrameTime << ", display again."
@@ -884,7 +965,7 @@ FrameAvailableCode FFMpegDecoder::readFrameForTime(AVFrame& Frame, long long Tim
             // The last frame is still current. Display it again.
             return FA_USE_LAST_FRAME;
         } else {
-            while (FrameTime-TimeWanted < -0.5*TimePerFrame && !m_bVideoEOF) {
+            while (FrameTime-timeWanted < -0.5*TimePerFrame && !m_bVideoEOF) {
                 readFrame(Frame, FrameTime);
 /*                
                 if (bDebug) {
@@ -906,6 +987,7 @@ void FFMpegDecoder::readFrame(AVFrame& Frame, long long& FrameTime)
 {
     assert(m_pDemuxer);
     if (m_bVideoEOF) {
+        seek(0);
         return;
     }
     if (m_bEOFPending) {
@@ -959,7 +1041,7 @@ void FFMpegDecoder::readFrame(AVFrame& Frame, long long& FrameTime)
             }
             int Len1 = avcodec_decode_video(enc, &Frame,
                     &gotPicture, m_pPacketData, m_PacketLenLeft);
-            if (Len1 < 0) {
+            if (Len1 <= 0) {
 //                AVG_TRACE(Logger::WARNING, "Error decoding " <<
 //                        m_sFilename);
                 m_PacketLenLeft = 0;
@@ -1001,32 +1083,16 @@ long long FFMpegDecoder::getFrameTime(AVPacket* pPacket)
     return FrameTime;
 }
 
-StreamSelect FFMpegDecoder::getMasterStream()
+long long FFMpegDecoder::getStartTime()
 {
-    if (m_pAStream) {
-        return SS_AUDIO;
+    if (m_pVStream) {
+        return m_VideoStartTimestamp;
     } else {
-        return SS_VIDEO;
+        return m_AudioStartTimestamp;
     }
 }
 
-long long FFMpegDecoder::getStartTime(StreamSelect Stream)
-{
-    switch(Stream) {
-        case SS_VIDEO:
-            assert(m_pVStream);
-            return m_VideoStartTimestamp;
-        case SS_AUDIO:
-            assert(m_pAStream);
-            return m_AudioStartTimestamp;
-        case SS_DEFAULT:
-            return getStartTime(getMasterStream());
-        default:
-            return -1;
-    }
-}
-
-double FFMpegDecoder::calcStreamFPS()
+double FFMpegDecoder::calcStreamFPS() const
 {
 #if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
     return m_pVStream->r_frame_rate;
@@ -1035,6 +1101,17 @@ double FFMpegDecoder::calcStreamFPS()
 #endif 
 }
 
+string FFMpegDecoder::getStreamPF() const
+{
+#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
+    AVCodecContext *pCodec = &m_pVStream->codec;
+#else
+    AVCodecContext *pCodec = m_pVStream->codec;
+#endif
+    ::PixelFormat pf = pCodec->pix_fmt;
+    return avcodec_get_pix_fmt_name(pf);
+}
+
 // TODO: this should be logarithmic...
 void FFMpegDecoder::volumize(AudioBufferPtr pBuffer)
 {
