@@ -22,6 +22,7 @@
 #ifndef _FFMpegDecoder_H_
 #define _FFMpegDecoder_H_
 
+#include "../avgconfigwrapper.h"
 #include "IVideoDecoder.h"
 #include "IDemuxer.h"
 
@@ -46,9 +47,13 @@
 #endif
 
 extern "C" {
+#ifdef HAVE_LIBAVFORMAT_AVFORMAT_H
+#include <libavcodec/avcodec.h>
+#include <libavformat/avformat.h>
+#include <libswscale/swscale.h>
+#else
 #include <ffmpeg/avcodec.h>
 #include <ffmpeg/avformat.h>
-#ifdef AVG_ENABLE_SWSCALE
 #include <ffmpeg/swscale.h>
 #endif
 }
@@ -58,33 +63,32 @@ namespace avg {
 
 class AudioBuffer;
 
-class FFMpegDecoder: public IVideoDecoder
+class AVG_API FFMpegDecoder: public IVideoDecoder
 {
     public:
         FFMpegDecoder();
         virtual ~FFMpegDecoder();
-        virtual void open(const std::string& sFilename, const AudioParams* AP,
-                YCbCrMode ycbcrMode, bool bThreadedDemuxer);
+        virtual void open(const std::string& sFilename, bool bThreadedDemuxer);
+        virtual void startDecoding(bool bDeliverYCbCr, const AudioParams* AP);
         virtual void close();
-        virtual StreamSelect getMasterStream();
-        virtual bool hasAudio();
-        virtual int getNumFrames();
-        virtual long long getDuration();
+        virtual DecoderState getState() const;
+        virtual VideoInfo getVideoInfo() const;
 
-        virtual double getNominalFPS();
-        virtual double getFPS();
-        virtual double getVolume();
-        virtual PixelFormat getPixelFormat();
+        virtual double getNominalFPS() const;
+        virtual double getFPS() const;
+        virtual double getVolume() const;
+        virtual PixelFormat getPixelFormat() const;
 
         // Called from video thread.
-        virtual IntPoint getSize();
-        virtual int getCurFrame();
-        virtual int getNumFramesQueued();
-        virtual long long getCurTime(StreamSelect Stream = SS_DEFAULT);
+        virtual IntPoint getSize() const;
+        virtual int getCurFrame() const;
+        virtual int getNumFramesQueued() const;
+        virtual long long getCurTime(StreamSelect Stream = SS_DEFAULT) const;
         virtual void setFPS(double FPS);
-        virtual FrameAvailableCode renderToBmp(BitmapPtr pBmp, long long TimeWanted);
+        virtual FrameAvailableCode renderToBmp(BitmapPtr pBmp, long long timeWanted);
         virtual FrameAvailableCode renderToYCbCr420p(BitmapPtr pBmpY, BitmapPtr pBmpCb, 
-                BitmapPtr pBmpCr, long long TimeWanted);
+                BitmapPtr pBmpCr, long long timeWanted);
+        virtual void throwAwayFrame(long long timeWanted);
         
         // Called from audio decoder thread
         virtual void setVolume(double Volume);
@@ -92,26 +96,28 @@ class FFMpegDecoder: public IVideoDecoder
 
         // Called from video and audio threads
         virtual void seek(long long DestTime);
-        virtual bool hasVideo();
-        virtual bool isEOF(StreamSelect Stream = SS_ALL);
+        virtual bool isEOF(StreamSelect Stream = SS_ALL) const;
 
     private:
         void initVideoSupport();
-        PixelFormat calcPixelFormat(YCbCrMode ycbcrMode);
+        PixelFormat calcPixelFormat(bool bUseYCbCr);
+        virtual long long getDuration() const;
+        virtual int getNumFrames() const;
 
+        DecoderState m_State;
         AVFormatContext * m_pFormatContext;
         PixelFormat m_PF;
         std::string m_sFilename;
+        bool m_bThreadedDemuxer;
 
         // Used from video thread.
-        FrameAvailableCode readFrameForTime(AVFrame& Frame, long long TimeWanted);
+        FrameAvailableCode readFrameForTime(AVFrame& Frame, long long timeWanted);
         void convertFrameToBmp(AVFrame& Frame, BitmapPtr pBmp);
         long long getFrameTime(AVPacket* pPacket);
-        double calcStreamFPS();
+        double calcStreamFPS() const;
+        std::string getStreamPF() const;
 
-#ifdef AVG_ENABLE_SWSCALE
         SwsContext * m_pSwsContext;
-#endif
         IntPoint m_Size;
         double m_TimeUnitsPerSecond;
         bool m_bUseStreamFPS;
@@ -146,7 +152,7 @@ class FFMpegDecoder: public IVideoDecoder
 
         // Used from video and audio threads.
         void readFrame(AVFrame& Frame, long long& FrameTime);
-        long long getStartTime(StreamSelect Stream = SS_DEFAULT);
+        long long getStartTime();
 
         IDemuxer * m_pDemuxer;
         AVStream * m_pVStream;
