@@ -22,6 +22,7 @@
 #ifndef _FFMpegDemuxer_H_
 #define _FFMpegDemuxer_H_
 
+#include "../avgconfigwrapper.h"
 #include "IDemuxer.h"
 
 #ifdef _WIN32
@@ -42,7 +43,11 @@ extern "C" {
 #pragma warning(push)
 #pragma warning(disable:4244)
 #endif
+#ifdef HAVE_LIBAVFORMAT_AVFORMAT_H
+#include <libavformat/avformat.h>
+#else
 #include <ffmpeg/avformat.h>
+#endif
 #ifdef _WIN32
 #pragma warning(pop)
 #endif
@@ -55,7 +60,7 @@ extern "C" {
 
 namespace avg {
 
-    class FFMpegDemuxer: public IDemuxer {
+    class AVG_API FFMpegDemuxer: public IDemuxer {
         public:
             FFMpegDemuxer(AVFormatContext * pFormatContext);
             virtual ~FFMpegDemuxer();
