@@ -22,6 +22,7 @@
 #ifndef _IDemuxer_H_
 #define _IDemuxer_H_
 
+#include "../api.h"
 #include <math.h>
 
 #ifdef _WIN32
@@ -42,7 +43,11 @@ extern "C" {
 #pragma warning(push)
 #pragma warning(disable:4244)
 #endif
+#ifdef HAVE_LIBAVFORMAT_AVFORMAT_H
+#include <libavformat/avformat.h>
+#else
 #include <ffmpeg/avformat.h>
+#endif
 #ifdef _WIN32
 #pragma warning(pop)
 #endif
@@ -52,7 +57,7 @@ extern "C" {
 
 namespace avg {
 
-    class IDemuxer {
+    class AVG_API IDemuxer {
         public:
             virtual ~IDemuxer() {};
            
