@@ -22,6 +22,7 @@
 #ifndef _PacketVideoMsg_H_
 #define _PacketVideoMsg_H_
 
+#include "../avgconfigwrapper.h"
 #include "../base/Queue.h"
 #include "../graphics/Bitmap.h"
 
@@ -39,14 +40,18 @@
 #endif
 
 extern "C" {
+#ifdef HAVE_LIBAVFORMAT_AVFORMAT_H
+#include <libavformat/avformat.h>
+#else
 #include <ffmpeg/avformat.h>
+#endif
 }
 
 #include <vector>
 
 namespace avg {
 
-class PacketVideoMsg {
+class AVG_API PacketVideoMsg {
     public:
         PacketVideoMsg(AVPacket * pPacket, bool bSeekDone);
         virtual ~PacketVideoMsg();
