@@ -22,11 +22,14 @@
 #ifndef _VideoDecoderThread_H_
 #define _VideoDecoderThread_H_
 
+#include "../api.h"
 #include "IVideoDecoder.h"
 #include "VideoMsg.h"
+#include "FrameVideoMsg.h"
 
 #include "../base/WorkerThread.h"
 #include "../base/Command.h"
+#include "../base/Queue.h"
 
 #include <boost/thread.hpp>
 
@@ -34,7 +37,10 @@
 
 namespace avg {
 
-class VideoDecoderThread: public WorkerThread<VideoDecoderThread> {
+typedef Queue<BitmapPtr> BitmapQueue;
+typedef boost::shared_ptr<BitmapQueue> BitmapQueuePtr;
+
+class AVG_API VideoDecoderThread: public WorkerThread<VideoDecoderThread> {
     public:
         VideoDecoderThread(CmdQueue& CmdQ, VideoMsgQueue& MsgQ, 
                 VideoDecoderPtr pDecoder);
@@ -43,10 +49,16 @@ class VideoDecoderThread: public WorkerThread<VideoDecoderThread> {
         bool work();
         void seek(long long DestTime);
         void setFPS(double FPS);
+        void returnFrame(FrameVideoMsgPtr pMsg);
 
     private:
+        BitmapPtr getBmp(BitmapQueuePtr BmpQ, const IntPoint& Size, 
+                PixelFormat pf);
         VideoMsgQueue& m_MsgQ;
         VideoDecoderPtr m_pDecoder;
+
+        BitmapQueuePtr m_pBmpQ;
+        BitmapQueuePtr m_pHalfBmpQ;
 };
 
 }
