@@ -24,22 +24,26 @@ void export_event();
 #ifndef WIN32
 void export_devices();
 #endif
+void export_anim();
 
 #include "WrapHelper.h"
 
 #include "../base/Logger.h"
 #include "../base/Exception.h"
+#include "../base/OSHelper.h"
 #include "../player/Player.h"
 #include "../player/AVGNode.h"
 #include "../player/DivNode.h"
 #include "../player/PanoImage.h"
 #include "../player/TrackerEventSource.h"
+#include "../player/TouchEvent.h"
 
 #include <boost/version.hpp>
 #include <boost/shared_ptr.hpp>
 
 using namespace boost::python;
 using namespace avg;
+using namespace std;
 
 void exception_translator(Exception const & e) 
 {
@@ -52,11 +56,10 @@ BOOST_PYTHON_MODULE(avg)
 
     scope().attr("__doc__") =
         "The main libavg module.\n"
-        "G{classtree Node}\n"
-        "G{classtree Event}\n"
-        "G{classtree Bitmap}\n"
-        "G{classtree Tracker TrackerCalibrator}\n"
-        "G{classtree Logger}\n"
+        "G{classtree Node}\n\n"
+        "G{classtree Bitmap}\n\n"
+        "G{classtree Tracker TrackerCalibrator}\n\n"
+        "G{classtree Logger}\n\n"
         "G{classtree ConradRelais ParPort}";
 
 #if (BOOST_VERSION / 100000) > 1 || ((BOOST_VERSION / 100) % 1000) >= 33
@@ -66,14 +69,38 @@ BOOST_PYTHON_MODULE(avg)
     register_ptr_to_python<AVGNodePtr>();
     register_ptr_to_python<EventPtr>();
     register_ptr_to_python<MouseEventPtr>();
+    register_ptr_to_python<TouchEventPtr>();
 
     to_python_converter<IntPoint, Point_to_python_tuple<int> >();
-    DPoint_from_python_tuple();
+    DPoint_from_python_tuple<DPoint, double>();
+    DPoint_from_python_tuple<ConstDPoint, double>();
+    DPoint_from_python_tuple<IntPoint, int>();
+    
+    IntTriple_from_python_tuple();
+    
+   // IntTriple_from_python_tuple<IntTriple, int>();
+
+    to_python_converter<vector<DPoint>, to_list<vector<DPoint> > >();    
+    to_python_converter<vector<string>, to_list<vector<string> > >();    
+   
+    from_python_sequence<vector<DPoint>, variable_capacity_policy>();
+    from_python_sequence<vector<IntPoint>, variable_capacity_policy>();
+    from_python_sequence<vector<string>, variable_capacity_policy>();
+  
+    from_python_sequence<vector<IntTriple>, variable_capacity_policy>();
+
+    def("getMemoryUsage", getMemoryUsage,
+            "Returns the amount of memory used by the application in bytes. More\n"
+            "precisely, this function returns the resident set size of the process\n"
+            "in bytes. This does not include shared libraries or memory paged out to\n"
+            "disk.\n");
 
     class_<Logger>("Logger", 
             "Interface to the logger used by the avg player. Enables the setting\n"
-            "of different logging categories. Log output can be sent to the console, a\n"
-            "file or unix syslog.\n"
+            "of different logging categories. Categories can be set either by calling\n"
+            "Logger.setCategories or by setting the AVG_LOG_CATEGORIES environment\n"
+            "variable. Default categories are ERROR, WARNING, and APP. Log output\n"
+            "is sent to the console (stderr).\n"
             "Each log entry contains the time the message was written, the category\n"
             "of the entry and the message itself.\n",
             no_init)
@@ -81,18 +108,6 @@ BOOST_PYTHON_MODULE(avg)
                 return_value_policy<reference_existing_object>(),
                 "This method gives access to the logger. There is only one instance.\n")
         .staticmethod("get")
-        .def("setConsoleDest", &Logger::setConsoleDest,
-                "Sets the log to be output to the console (stderr, to be precise)."
-                "This is the default.")
-        .def("setFileDest", &Logger::setFileDest,
-                "Sets a file that the log should be written to. If opening the file\n"
-                "fails, the console is used instead.\n"
-                "@param filename: \n")
-        .def("setSyslogDest", &Logger::setSyslogDest,
-                "Causes log output to be written to the unix system log facility.\n"
-                "syslog ident is set to 'libavg'.\n"
-                "@param facility: Passed to the system log verbatim. See man 3 syslog.\n"
-                "@param logopt: Passed to the system log verbatim. See man 3 syslog.")
         .def("setCategories", &Logger::setCategories,
                 "Sets the types of messages that should be logged.\n" 
                 "@param categories: Or'ed list of categories. Possible categories are:\n"
@@ -105,14 +120,25 @@ BOOST_PYTHON_MODULE(avg)
                 "    - EVENTS: Outputs basic event data.\n"
                 "    - EVENTS2: Outputs all event data available.\n"
                 "    - CONFIG: Outputs configuration data.\n"
-                "    - WARNING: Outputs warning messages. Default is on.\n"
+                "    - WARNING: Outputs warning messages.\n"
                 "    - ERROR: Outputs error messages. Can't be shut off.\n"
-                "    - MEMORY: Outputs open/close information whenever a media file is accessed.\n"
-                "    - APP: Reserved for application-level messages issued by python code.\n")
+                "    - MEMORY: Outputs open/close information whenever a media file is\n"
+                "              accessed.\n"
+                "    - APP: Reserved for application-level messages issued by python\n"
+                "           code.\n"
+                "    - PLUGIN: Messages generated by loading plugins.\n"
+                "    - PLAYER: General libavg playback messages.\n")
+        .def("pushCategories", &Logger::pushCategories,
+                "Pushes the current set of categories on an internal stack. Useful\n"
+                "for saving and restoring the logging state so it can be changed\n"
+                "for a short amount of time.\n")
+        .def("popCategories", &Logger::popCategories,
+                "Pops the current set of categories from the internal stack, restoring\n"
+                "the state when the corresponding push was called.\n")
         .def("trace", &Logger::trace,
                 "Logs message to the log if category is active.\n"
-                "@param category: One of the categories listed for setCategories(). Should\n"
-                "in most cases be APP.\n"
+                "@param category: One of the categories listed for setCategories().\n"
+                "Should in most cases be APP.\n"
                 "@param message: The log message.\n")
         .def_readonly("NONE", &Logger::NONE)
         .def_readonly("BLTS", &Logger::BLTS)
@@ -125,6 +151,7 @@ BOOST_PYTHON_MODULE(avg)
         .def_readonly("ERROR", &Logger::ERROR)
         .def_readonly("MEMORY", &Logger::MEMORY)
         .def_readonly("APP", &Logger::APP)
+        .def_readonly("PLUGIN", &Logger::PLUGIN)
     ;
 
 #ifndef WIN32
@@ -132,18 +159,12 @@ BOOST_PYTHON_MODULE(avg)
 #endif
     export_event();
     export_node();
-
-    enum_<YCbCrMode>("YCbCrMode")
-        .value("shader", OGL_SHADER)
-        .value("mesa", OGL_MESA)
-        .value("apple", OGL_APPLE)
-        .value("none", OGL_NONE)
-        .export_values()
-    ;
+    export_anim();
 
     class_<TestHelper>("TestHelper", "Miscelaneous routines used by tests.", no_init)
         .def("useFakeCamera", &TestHelper::useFakeCamera, "")
         .def("fakeMouseEvent", &TestHelper::fakeMouseEvent, "")
+        .def("fakeTouchEvent", &TestHelper::fakeTouchEvent, "")
         .def("fakeKeyEvent", &TestHelper::fakeKeyEvent, "")
         .def("dumpObjects", &TestHelper::dumpObjects, "")
     ;
@@ -168,29 +189,35 @@ BOOST_PYTHON_MODULE(avg)
                 "Sets the location of the player window. Must be called before loadFile\n"
                 "or loadString.\n")
         .def("setOGLOptions", &Player::setOGLOptions,
-                "setOGLOptions(UsePOW2Textures, YCbCrMode, UsePixelBuffers, MultiSampleSamples)\n"
+                "setOGLOptions(UsePOW2Textures, UseYCbCrShaders, UsePixelBuffers, MultiSampleSamples)\n"
                 "Determines which OpenGL extensions to check for and use if possible.\n"
                 "Mainly used for debugging purposes while developing libavg, but can\n"
-                "also be used to work around buggy drivers. The values set here override\n"
-                "those in avgrc. Note that with the exception of MultiSampleSamples,\n"
-                "fallbacks are always used - if a feature is\n"
+                "also be used to work around buggy drivers. The values set here\n"
+                "override those in avgrc. Note that with the exception of\n"
+                "MultiSampleSamples, fallbacks are always used - if a feature is\n"
                 "specified that the system doesn't support, a less demanding one will\n"
                 "be used.\n"
-                "@param UsePOW2Textures: If True, restricts textures to power-of-two dimensions.\n"
-                "@param YCbCrMode: Selects the preferred method of copying video textures\n"
-                "to the screen. Can be shader, mesa, apple or none.\n"
-                "@param UsePixelBuffers: If False, disables the use of OpenGL pixel buffer\n"
-                "objects.\n"
-                "@param MultiSampleSamples: The number of samples per pixel to compute. This\n"
-                "costs performance and smoothes the edges of polygons. A value of 1 turns\n"
-                "multisampling (also knowna as FSAA - Full-Screen Antialiasing) off. Good\n"
-                "values are dependent on the graphics driver.\n")
+                "@param UsePOW2Textures: If True, restricts textures to power-of-two\n"
+                "dimensions.\n"
+                "@param UseYCbCrShaders: If True, shaders are used to copy YCbCr\n"
+                "to the screen. Results in a major video playback performance boost.\n"
+                "@param UsePixelBuffers: If False, disables the use of OpenGL pixel\n"
+                "buffer objects.\n"
+                "@param MultiSampleSamples: The number of samples per pixel to compute.\n"
+                "This costs performance and smoothes the edges of polygons. A value of\n"
+                "1 turns multisampling (also knowna as FSAA - Full-Screen Antialiasing)\n"
+                "off. Good values are dependent on the graphics driver.\n")
         .def("setMultiSampleSamples", &Player::setMultiSampleSamples,
                 "setMultiSampleSamples(multiSampleSamples)\n"
-                "@param MultiSampleSamples: The number of samples per pixel to compute. This\n"
-                "costs performance and smoothes the edges of polygons. A value of 1 turns\n"
-                "multisampling (also knowna as FSAA - Full-Screen Antialiasing) off. Good\n"
-                "values are dependent on the graphics driver.\n")
+                "@param multiSampleSamples: The number of samples per pixel to compute.\n"
+                "This costs performance and smoothes the edges of polygons. A value of\n"
+                "1 turns multisampling (also knowna as FSAA - Full-Screen Antialiasing)\n"
+                "off. Good values are dependent on the graphics driver.\n")
+        .def("enableAudio", &Player::enableAudio,
+                "enableAudio(bEnable)\n"
+                "Enables or disables audio playback. If audio playback is disabled, no\n"
+                "nodes with sound can be created. Mainly used to speed up the test\n"
+                "suite.\n")
         .def("loadFile", &Player::loadFile,
                 "loadFile(filename)\n"
                 "Loads the avg file specified in filename.\n"
@@ -229,15 +256,18 @@ BOOST_PYTHON_MODULE(avg)
                 "")
         .def("setFakeFPS", &Player::setFakeFPS,
                 "setFakeFPS(fps)\n"
-                "Sets a fixed number of virtual frames per second that are used as clock\n"
-                "source for video playback, animations and other time-based actions.\n"
-                "If a value of -1 is given as parameter, the real clock is used.\n"
+                "Sets a fixed number of virtual frames per second that are used as\n"
+                "clock source for video playback, animations and other time-based\n"
+                "actions. If a value of -1 is given as parameter, the real clock is\n"
+                "used. FakeFPS can be used to get reproducible effects for recordings\n"
+                "or automated tests. Setting FakeFPS has the side-effect of disabling\n"
+                "audio.\n"
                 "@param fps: \n")
         .def("getFrameTime", &Player::getFrameTime,
                 "getFrameTime() -> time\n"
-                "Returns the number of milliseconds that have elapsed since playback has\n"
-                "started. Honors FakeFPS. The time returned stays constant for an entire\n"
-                "frame; it is the time of the last display update.\n")
+                "Returns the number of milliseconds that have elapsed since playback\n"
+                "has tarted. Honors FakeFPS. The time returned stays constant for an\n"
+                "entire frame; it is the time of the last display update.\n")
         .def("createNode", &Player::createNodeFromXmlString,
                 "createNode(xml) -> node\n"
                 "Creates a new Node. This node can be used as\n"
@@ -254,10 +284,14 @@ BOOST_PYTHON_MODULE(avg)
                 "@param args: a dictionary specifying attributes of the node.")
         .def("addTracker", &Player::addTracker,
                 return_value_policy<reference_existing_object>(),
-                "addTracker(configFilename)\n"
+                "addTracker()\n"
                 "Adds a camera-based tracker to the avg player. The tracker can be\n"
-                "configured using the config file given and immediately starts\n"
+                "configured using the default config file and immediately starts\n"
                 "reporting events.")
+        .def("getTracker", &Player::getTracker,
+                return_value_policy<reference_existing_object>(),
+                "getTracker()\n"
+                "returns a tracker previously created with addTracker.")
         .def("setInterval", &Player::setInterval,
                 "setInterval(time, pyfunc) -> id\n"
                 "Sets a python callable object that should be executed regularly.\n"
@@ -283,10 +317,15 @@ BOOST_PYTHON_MODULE(avg)
                 "clearInterval(id) -> ok\n"
                 "Stops a timeout, an interval or an onFrameHandler from being called.\n"
                 "Returns True if there was an interval with the given id, False if not.\n"
-                "@param id: An id returned by setInterval, setTimeout or setOnFrameHandler.\n")
+                "@param id: An id returned by setInterval, setTimeout or\n"
+                "setOnFrameHandler.\n")
         .def("getMouseState", &Player::getMouseState,
                 "getMouseState() -> event\n"
                 "Returns an interface to the last mouse event.\n")
+        .def("getKeyModifierState", &Player::getKeyModifierState,
+                "getKeyModifierState() -> KeyModifier\n"
+                "Returns the current modifier keys (shift, ctrl) pressed. The return\n"
+                "value is several KeyModifier values or'ed together.\n")
         .def("screenshot", &Player::screenshot,
                 return_value_policy<manage_new_object>(),
                 "screenshot() -> bitmap\n"
@@ -299,6 +338,13 @@ BOOST_PYTHON_MODULE(avg)
                 "showCursor(show)\n"
                 "Shows or hides the mouse cursor.\n"
                 "@param show: True if the mouse cursor should be visible.\n")
+        .def("setCursor", &Player::setCursor,
+                "setCursor(bitmap, hotspot)\n"
+                "Sets the mouse cursor to the bitmap given. The bitmap must have a size\n"
+                "divisible by 8 and an RGBA pixel format. The cursor generated is\n"
+                "binary black and white with a binary transparency channel. hotspot is\n"
+                "the relative position of the actual pointing coordinate in the\n"
+                "bitmap.\n")
         .def("getElementByID", &Player::getElementByID,
                 "getElementByID(id) -> node\n"
                 "Returns an element in the avg tree.\n"
@@ -320,10 +366,18 @@ BOOST_PYTHON_MODULE(avg)
                 "values. 1.0 is identity, higher values give a brighter image, lower\n"
                 "values a darker one.\n"
                 "@param red, green, blue: \n")
-        .def("getGPUMemoryUsage", &Player::getGPUMemoryUsage,
-                "getGPUMemoryUsage() -> bytes\n"
-                "Get the amount of memory currently allocated for textures storaging\n"
-                "on the graphics adapter.\n")
+        .def("setMousePos", &Player::setMousePos,
+                "setMousePos(pos)\n"
+                "Sets the position of the mouse cursor. Generates a mouse motion event.\n"
+                "@param pos: new coordinates as a Point2D.\n")
+        .def("loadPlugin", &Player::loadPlugin,
+                "loadPlugin(name)\n"
+                "load a Plugin and extend the XML DTD.\n"
+                "@param name: name of the plugin (without directory and\n"
+                "file extension)\n")
+        .add_property("pluginPath", &Player::getPluginPath, &Player::setPluginPath,
+                "A colon-separated list of directories where the player\n"
+                "searches for plugins when loadPlugin() is called.\n")
         .add_property("volume", &Player::getVolume, &Player::setVolume,
                 "Total audio playback volume. 0 is silence, 1 passes media file\n"
                 "volume through unchanged. Values higher than 1 can be used to\n"
